/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// RngStream by Pierre L'Ecuyer, University of Montreal
// Adapted to GTNetS by George F. Riley, Georgia Tech.

#ifndef RNGSTREAM_H
#define RNGSTREAM_H
#include <string>

 
//Doc:ClassXRef
class RngStream {
public:
  RngStream (const char *name = "");
  static void SetPackageSeed (const unsigned long seed[6]);
  void ResetStartStream ();
  void ResetStartSubstream ();
  void ResetNextSubstream ();
  void SetAntithetic (bool a);
  void IncreasedPrecis (bool incp);
  void SetSeed (const unsigned long seed[6]);
  void AdvanceState (long e, long c);
  void GetState (unsigned long seed[6]) const;
  void WriteState () const;
  void WriteStateFull () const;
  double RandU01 ();
  long RandInt (long i, long j);
private:
  double Cg[6], Bg[6], Ig[6];
  bool anti, incPrec;
  std::string name;
  static double nextSeed[6];
  double U01 ();
  double U01d ();
};

#endif
 

