/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
                                                                                
* Permission is hereby granted for any non-commercial use of this code
                                                                                
*/
                                                                                
// Boeing MANET OSPF6D port.  Copyright 2004, The Boeing Company

// Wrapper for OSPF6D

// Uncomment below to enable debug level 0
//#define DEBUG_MASK 0x1
// 
/*+----------- tabstop 4 ---------+*/

#include "debug.h"
#include <assert.h>
#include <iomanip>
#include "node.h"
#include "application-ospf6d.h"
#include "ipv4.h"
#include "ospfdemux.h"

#include "privs.h"
#include "memory.h" // for memory_init()
#include "command.h" //for cmd_init()
#include "lib/if.h"  // for if_init()
#include "log.h"
#include "sim.h"
#include "ospf6_message.h" //for ReceiveMessage()
#include "ospf6_interface.h" //ospf6_interface_lookup_by_ifindex
#include "ospf6_top.h" //for ospf6_delete and OSPF6_STAT_LENGTH
#include "ospf6_area.h" //for ospf6_area
#include "ospf6_neighbor.h" //for ospf6_neighbor
#include "routing-ospf6.h"

#include "globalstats.h"
#include "estat.h"
#include "thread.h" //XXX for add_unuse
extern void thread_master_free(struct thread_master*);

using namespace std;

/*
 * Global variables needed for the quagga C libraries 
 * They are reassigned to the corresponding instance variable, as needed,
 * via the SetGlobals() method
  */

Node *sim_node;

// static variables
OSPF6DApplication* OSPF6DApplication::instance_ = NULL;

extern struct ospf6 *ospf6;
extern struct thread_master *master;
extern struct host* phost;
extern struct List* iflist;
extern struct zebra_privs_t ospf6d_privs;
extern struct zlog *zlog_default;
extern pVector vtyvec;
extern pVector Vvty_serv_thread;
extern char *vty_cwd;
extern pVector cmdvec;
extern struct route_table *ifaddr_ipv4_table; //lib/if.c

extern void access_list_init();  // defined in lib/filter.c
extern void prefix_list_init();  // defined in lib/plist.c

/*
 *  class OSPF6DEvent
*/

//OSPF6DEvent constructor method
OSPF6DEvent::OSPF6DEvent(Event_t ev) : TimerEvent(ev), t(NULL) 
{
}

//OSPF6DEvent destructor method
OSPF6DEvent::~OSPF6DEvent() 
{
}

/*
 * class OSPF6InterfacesConfig
 */
OSPF6InterfacesConfig::OSPF6InterfacesConfig()
{
}

OSPF6InterfacesConfig::~OSPF6InterfacesConfig()
{
  struct OSPF6InterfaceConfig *InterConf;
  while (!vpInterfaces.empty())
  {
    InterConf = vpInterfaces.back();
    vpInterfaces.pop_back();
    delete [] InterConf->name;
    if (InterConf->type)
      delete [] InterConf->type;
    if (InterConf->flooding)
      delete [] InterConf->flooding;
    if (InterConf->AdjConnectivity)
      delete [] InterConf->AdjConnectivity;
    if (InterConf->LSAFullness)
      delete [] InterConf->LSAFullness;
    delete InterConf;
  }
}

void OSPF6InterfacesConfig::SetInterfaceHelloInterval(char *name, int hello_int)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->HelloInterval = hello_int;
}

void OSPF6InterfacesConfig::SetInterfaceAckInterval(char *name, int ack_int)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->AckInterval = ack_int;
}

void OSPF6InterfacesConfig::SetInterfaceFlooding(char *name, char *type)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	if (ifc->flooding)
		delete [] ifc->flooding;
  ifc->flooding = new char[strlen(type)+1];
  strcpy(ifc->flooding, type);
}

void OSPF6InterfacesConfig::SetInterfaceType(char *name, char *type)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	if (ifc->type)
		delete [] ifc->type;
  ifc->type = new char[strlen(type)+1];
  strcpy(ifc->type, type);
}


void OSPF6InterfacesConfig::SetInterfaceCost(char *name, int cost)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->cost = cost;
}

void OSPF6InterfacesConfig::SetInterfaceDeadInterval(char *name, int dead)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->DeadInterval = dead;
}

void OSPF6InterfacesConfig::SetInterfaceRetransmitInterval(char *name, int rx)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->RxmtInterval = rx;
}

void OSPF6InterfacesConfig::SetInterfaceArea(char *name, IPAddr_t a)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->area = a;
}

void OSPF6InterfacesConfig::SetInterfaceDiffHellos(char *name, bool diff)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->diff_hellos = diff;
}

void OSPF6InterfacesConfig::SetInterfaceJitter(char *name, int jitter_msec)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->jitter = jitter_msec;
}

void OSPF6InterfacesConfig::SetInterfaceFloodDelay(char *name, 
		                                               int flood_delay_msec)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->flood_delay = flood_delay_msec;
}

void OSPF6InterfacesConfig::SetInterfaceMTU(char *name, 
		                                        int mtu_bytes)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->ifmtu = mtu_bytes;
}

//OR
void OSPF6InterfacesConfig::SetInterfacePushBackInterval(char *name, int pb_int)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->PushbackInterval = pb_int;
}

void OSPF6InterfacesConfig::SetInterfaceSmartPeering(char *name, bool smart)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->smart_peering = smart;
}

void OSPF6InterfacesConfig::SetInterfaceUnsynchAdj(char *name, bool adj)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->unsynch_adj = adj;
}

void OSPF6InterfacesConfig::SetInterfaceTopoReduc(char *name, bool tr)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
	ifc->topo_reduc = tr;
}

//MDR
void OSPF6InterfacesConfig::SetInterfaceBackupWaitInterval(char *name,
                                                           int bw_int)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->BackupWaitInterval = bw_int;
}

void OSPF6InterfacesConfig::SetInterfaceTwoHopRefresh(char *name, int count)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->TwoHopRefresh = count;
}

void OSPF6InterfacesConfig::SetInterfaceHelloRepeatCount(char *name, int count)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->HelloRepeatCount = count;
}

void OSPF6InterfacesConfig::SetInterfaceAdjConnectivity(char *name, char* conn)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  if (ifc->AdjConnectivity)
    delete [] ifc->AdjConnectivity;
  ifc->AdjConnectivity = new char[strlen(conn)+1];
  strcpy(ifc->AdjConnectivity, conn);
}

void OSPF6InterfacesConfig::SetInterfaceLSAFullness(char *name, char* fullness)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  if (ifc->LSAFullness)
    delete [] ifc->LSAFullness;
  ifc->LSAFullness = new char[strlen(fullness)+1];
  strcpy(ifc->LSAFullness, fullness);
}

void OSPF6InterfacesConfig::SetInterfaceNonPersistentMDR(char *name,
                                                         bool nonpers)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  
  ifc = FindInterface(name);
  if (!ifc)
    ifc = AddInterface(name);
  ifc->NonPersistentMDR = nonpers;
}

void OSPF6InterfacesConfig::CreateConfFileInterfaces(FILE *fp)
{
  vpIntersConfig::iterator it;
  for (it = vpInterfaces.begin(); it!=vpInterfaces.end(); ++it)
	{
    //Always called since intialized in AddInterface()
    fprintf(fp,"interface %s\n", (*it)->name);
    fprintf(fp,"  ipv6 ospf6 priority %d\n", (*it)->priority);
    fprintf(fp,"  ipv6 ospf6 transmit-delay %d\n", (*it)->transmit_delay);
    fprintf(fp,"  ipv6 ospf6 instance-id %d\n", (*it)->instance_id);
    fprintf(fp,"  ipv6 ospf6 ifmtu %d\n", (*it)->ifmtu);

    //Only called if it has been intialized by SetInterface..()
    if ((*it)->type)
      fprintf(fp,"  ipv6 ospf6 network %s\n", (*it)->type);
    if ((*it)->flooding)
      fprintf(fp,"  ipv6 ospf6 network flood %s\n", (*it)->flooding);
    if ((*it)->cost > 0)
      fprintf(fp,"  ipv6 ospf6 cost %d\n", (*it)->cost);
    if ((*it)->HelloInterval > 0)
      fprintf(fp,"  ipv6 ospf6 hello-interval %d\n", (*it)->HelloInterval);
    if ((*it)->DeadInterval > 0)
      fprintf(fp,"  ipv6 ospf6 dead-interval %d\n", (*it)->DeadInterval);
    if ((*it)->RxmtInterval > 0)
      fprintf(fp,"  ipv6 ospf6 retransmit-interval %d\n", (*it)->RxmtInterval);
#if defined(OSPF6_CONFIG) && defined(OSPF6_JITTER)
    if ((*it)->jitter > 0)
      fprintf(fp,"  ipv6 ospf6 jitter %d\n", (*it)->jitter);
#endif //JITTER
#if defined(OSPF6_CONFIG) && defined(OSPF6_DELAYED_FLOOD)
    if ((*it)->flood_delay > 0)
      fprintf(fp,"  ipv6 ospf6 flood-delay %d\n", (*it)->flood_delay);
#endif //DELAYED_FLOOD

    //Both MPR and CDS
    if ((*it)->AckInterval > 0)
      fprintf(fp,"  ipv6 ospf6 ackinterval %d\n", (*it)->AckInterval);
    if ((*it)->diff_hellos)
      fprintf(fp,"  ipv6 ospf6 diffhellos\n");

    //OR
    if ((*it)->PushbackInterval > 0)
      fprintf(fp,"  ipv6 ospf6 pushbackinterval %d\n", (*it)->PushbackInterval);
    if ((*it)->smart_peering)
      fprintf(fp,"  ipv6 ospf6 smartpeering\n");
    if ((*it)->unsynch_adj)
      fprintf(fp,"  ipv6 ospf6 unsynchadj\n");

    //OSPF-MPR
    if ((*it)->topo_reduc)
      fprintf(fp,"  ipv6 ospf6 toporeduc\n");

    //MDR
    if ((*it)->BackupWaitInterval > 0)
      fprintf(fp,"  ipv6 ospf6 backupwaitinterval %d\n", 
                                                    (*it)->BackupWaitInterval);
    if ((*it)->TwoHopRefresh > 0)
      fprintf(fp,"  ipv6 ospf6 twohoprefresh %d\n", (*it)->TwoHopRefresh);
    if ((*it)->HelloRepeatCount > 0)
      fprintf(fp,"  ipv6 ospf6 hellorepeatcount %d\n", (*it)->HelloRepeatCount);
    if ((*it)->AdjConnectivity)
      fprintf(fp,"  ipv6 ospf6 adjacencyconnectivity %s\n", 
                                                     (*it)->AdjConnectivity);
    if ((*it)->LSAFullness)
      fprintf(fp,"  ipv6 ospf6 lsafullness %s\n", (*it)->LSAFullness);
    if ((*it)->NonPersistentMDR)
      fprintf(fp,"  ipv6 ospf6 nonpersistent_mdr\n");
  }
}

void OSPF6InterfacesConfig::CreateConfFileAreaAttach(FILE *fp)
{
  vpIntersConfig::iterator it;
  for (it = vpInterfaces.begin(); it!=vpInterfaces.end(); ++it)
  {
    fprintf(fp,"  interface %s area %s\n", 
				(*it)->name, IPAddr::ToDotted((*it)->area));
  }
}

struct OSPF6InterfaceConfig *OSPF6InterfacesConfig::FindInterface(char *name)
{
  vpIntersConfig::iterator it;
  for (it = vpInterfaces.begin(); it!=vpInterfaces.end(); ++it)
  {
    if (strcmp(name, (*it)->name)==0)
      return *it;
  }
  return NULL;
}

struct OSPF6InterfaceConfig *OSPF6InterfacesConfig::AddInterface(char *name)
{
  struct OSPF6InterfaceConfig *ifc = NULL;
  ifc = FindInterface(name);
  if (ifc)
    return ifc;

  //create new interface
  ifc = new (struct OSPF6InterfaceConfig);
  memset (ifc, 0, sizeof(struct OSPF6InterfaceConfig));
  ifc->name = new char[strlen(name)+1];
  strcpy(ifc->name, name);
  vpInterfaces.push_back(ifc);

	//intialize values
  /*
   * Important Note:  This is the location for default values not set
   * by the set_ospf6_config_arguments() function in args.cc 
   */
  ifc->priority = 1;
  ifc->transmit_delay = 1;
  ifc->instance_id = 0;
  ifc->ifmtu = 1500;
  ifc->area = 0;
 
  return ifc;
}        

/*
 *  class OSPF6DApplication
*/

// Constructors
OSPF6DApplication::OSPF6DApplication() 
	: stats_enabled(false), start_stat_time(0), polling_interval(0)
{
  // This will end up registering OSPF protocol at the IP layer, for recv
  OSPF6Demux::Register(this);

  Routing::SetRouting(new RoutingOSPF6);
	memset (statistics, 0, sizeof(double) * OSPF6_STAT_LENGTH);
}

// Copy constructor
OSPF6DApplication::OSPF6DApplication(const OSPF6DApplication& c)
{
}

Application*
OSPF6DApplication::Copy() const
{
  return new OSPF6DApplication(*this);
}

OSPF6DApplication::~OSPF6DApplication()
{
  struct OSPF6DInstance *ospf6inst;
	int num_nodes = vpInstances.size();
  while (!vpInstances.empty())
  {
    ospf6inst = vpInstances.back();
		ExtractStats(ospf6inst);
		ospf6inst->~OSPF6DInstance();
		vpInstances.pop_back();
  }
  instance_ = NULL;

  thread_master_free(master);

  if (stats_enabled) {
    os.open(statfile.c_str(), ios::out);
	  PrintStats(num_nodes);
    EStat::Instance()->EStatPrint();

	  // close stat file
	  if (os.is_open())
	    os.close();
	  stats_enabled = false;
  }

  // print out verbose stats to .tr file
  //  Stats::Print();
  // OR
  TraceEvent_sim(1, "Simulator: %llu OSPF TX packet drops",Stats::ospfTxPktsDropped);
  TraceEvent_sim(1, "Simulator: %llu OSPF TX packet collisions",Stats::ospfTxCollisions);
  TraceEvent_sim(1, "Simualtor: %llu OSPF RX Errors",Stats::ospfRxErrors);

}

// OSPF6DApplication Methods
OSPF6DInstance *
OSPF6DApplication::AddNode(Node* n) {
  return AddNode(n, NULL);
}

void OSPF6DApplication::CollectStats(const char *name, Time_t time)
{
	statfile = name;
	stats_enabled = true;
	start_stat_time = time;
}

// Second version of this, allowing for command line argument string
// to be printed
void OSPF6DApplication::CollectStats(const char *name, const char *args, Time_t time)
{
	statfile = name;
  argstring = args;
	stats_enabled = true;
	start_stat_time = time;
}


void OSPF6DApplication::ExtractStats(OSPF6DInstance *ospf6inst)
{
	ospf6inst->SetGlobals();
	if (!ospf6)
	{
		cerr << "ospf6 pointer is NULL" << endl;
		return;
	}
	for (int i = 0; i < OSPF6_STAT_LENGTH; i++)
	{
	  statistics[i] += ospf6->statistics[i];
	}
}

void OSPF6DApplication::PrintStats(int num_nodes)
{

	if (!stats_enabled || !os || !os.is_open())
		return;
	
	Time_t duration = Simulator::Now() - start_stat_time; 
	double total_pkt = 0;
	double total_byte = 0;

	os.setf(ios_base::fixed, ios_base::floatfield);
	os.precision(2);
  if (argstring.length()) {
    os << argstring << endl << endl;
  }
  if (polling_interval) {
    os << "Neighbor statistics (at physical layer):" << endl;
	  os << "--------------------------------------------------" << endl;
    os << "Number of nodes: " << Node::nextId << endl;
    os << "Neighbor threshold distance (m): " << 
      Node::GetNode(0)->GetRadioRange() << endl;
    double total_neighbor_changes = 0;
    double total_neighbor_average = 0;
    for (int i=0; i < num_nodes; i++) {
      OSPF6DInstance* oi = Node::GetNode(i)->GetOspf6dInst();
      total_neighbor_average += (oi->GetNeighborRunningAverage() /
        oi->GetNeighborRunningAverageTime());
      total_neighbor_changes += (oi->GetNeighborChanges() / 
        oi->GetNeighborChangesTime());
    }
    os << "Average neighbor density (neighbors/# of nodes):  " << 
      total_neighbor_average/num_nodes/num_nodes << endl;
    os << "Average neighbor changes/time (changes/sec/node): " << 
      total_neighbor_changes/num_nodes << endl << endl;
  } else {
    os << "Neighbor statistics logging not enabled" << endl << endl;
  }

	//find ospf6 packet totals
	for (int i = 0; i <= OSPF6_LSUPDATE_BYTE_SENT; i++)
	{
		if (i % 2 == 1)
			total_byte += statistics[i];
		else
			total_pkt += statistics[i];
	}

	//print stats
	os << "OSPFv3 Statistics" << endl;
	os << "           | Packets          Bytes         Kbps" << endl;
	os << "----------------------------------------------------"<< endl;
	os << "Hello       |"; 
	PrintOspf6PktLine(statistics[OSPF6_HELLO_SENT], 
			              statistics[OSPF6_HELLO_BYTE_SENT],
										duration);
	os << "LSAck       |";
	PrintOspf6PktLine(statistics[OSPF6_LSACK_SENT], 
			              statistics[OSPF6_LSACK_BYTE_SENT],
										duration);
	os << "LSR         |";
	PrintOspf6PktLine(statistics[OSPF6_LSREQ_SENT], 
			              statistics[OSPF6_LSREQ_BYTE_SENT],
										duration);
	os << "DBDESC      |";
	PrintOspf6PktLine(statistics[OSPF6_DBDESC_SENT], 
			              statistics[OSPF6_DBDESC_BYTE_SENT],
										duration);
	os << "LSU         |";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_SENT], 
			              statistics[OSPF6_LSUPDATE_BYTE_SENT],
										duration);
	os << "  LSUfld    |  ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_MULTI_SENT], 
			              statistics[OSPF6_LSUPDATE_MULTI_BYTE_SENT],
										duration);
	os << "  LSUuni    |  ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_UNI_SENT], 
			              statistics[OSPF6_LSUPDATE_UNI_BYTE_SENT],
										duration);
	os << "    LSUcol  |    ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_UNI_COL_SENT], 
			              statistics[OSPF6_LSUPDATE_UNI_COL_BYTE_SENT],
										duration);
	os << "    LSUrxmt |    ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_UNI_RXMT_SENT], 
			              statistics[OSPF6_LSUPDATE_UNI_RXMT_BYTE_SENT],
										duration);
	os << "    LSUdbex |    ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_UNI_DBEX_SENT], 
			              statistics[OSPF6_LSUPDATE_UNI_DBEX_BYTE_SENT],
										duration);
	os << "    LSUstale|    ";
	PrintOspf6PktLine(statistics[OSPF6_LSUPDATE_UNI_STALE_SENT], 
			              statistics[OSPF6_LSUPDATE_UNI_STALE_BYTE_SENT],
										duration);
	os << "            |" << endl;;
	os << "Total       |";
	PrintOspf6PktLine(total_pkt, 
			              total_byte,
										duration);

  double neighborpernode = 
	       (statistics[OSPF6_DURATION_OF_NUM_NEIGHS] == 0 ? 0 :
					statistics[OSPF6_NUM_NEIGH_TIMES_DURATION_OF_NUM_NEIGHS] /
				  statistics[OSPF6_DURATION_OF_NUM_NEIGHS]);
  double neigh_change_node_sec =
	       (statistics[OSPF6_DURATION_OF_NUM_NEIGHS] == 0 ? 0 :
					statistics[OSPF6_CHANGE_OF_NUM_NEIGHS] /
				  statistics[OSPF6_DURATION_OF_NUM_NEIGHS]);
  double neighlifetime = 
	       (statistics[OSPF6_NEIGH_DEATHS] == 0 ? 0 :
					statistics[OSPF6_NEIGH_LIFETIME] /
				  statistics[OSPF6_NEIGH_DEATHS]);

  double adjpernode = 
	       (statistics[OSPF6_DURATION_OF_NUM_ADJ] == 0 ? 0 :
					statistics[OSPF6_NUM_ADJ_TIMES_DURATION_OF_NUM_ADJ] /
				  statistics[OSPF6_DURATION_OF_NUM_ADJ]);
  double adj_change_node_sec =
	       (statistics[OSPF6_DURATION_OF_NUM_ADJ] == 0 ? 0 :
					statistics[OSPF6_CHANGE_OF_NUM_ADJ] /
				  statistics[OSPF6_DURATION_OF_NUM_ADJ]);

  double lsasoutsync = 
           (statistics[OSPF6_DATABASE_EXCHANGES] == 0 ? 0 :
           statistics[OSPF6_NUM_LSA_DIFFS] / 
           statistics[OSPF6_DATABASE_EXCHANGES]);
  double secperorigrtrlsa = (statistics[OSPF6_ORIG_RTR_LSA] == 0 ? 0 :
                            duration /
                            (statistics[OSPF6_ORIG_RTR_LSA] / num_nodes));
  double routechanges = (statistics[OSPF6_ROUTE_CHANGES] == 0 ? 0 :
                        statistics[OSPF6_ROUTE_CHANGES] / (duration*num_nodes));
  double adjaccum = statistics[OSPF6_ADJ_ACCUM] / num_nodes;
  double rtrlsahopcount = (statistics[OSPF6_ROUTER_LSA_INSTALL] == 0 ? 0 :
                          statistics[OSPF6_ROUTER_LSA_HOPCOUNT]/
                          statistics[OSPF6_ROUTER_LSA_INSTALL]);

  double relayselpernode = 
	       (statistics[OSPF6_DURATION_OF_NUM_RELSEL] == 0 ? 0 :
					statistics[OSPF6_NUM_RELSEL_TIMES_DURATION_OF_NUM_RELSEL] /
				  statistics[OSPF6_DURATION_OF_NUM_RELSEL]);
  double relaysellifetime = 
	       (statistics[OSPF6_RELSEL_DEATHS] == 0 ? 0 :
					statistics[OSPF6_RELSEL_LIFETIME] /
				  statistics[OSPF6_RELSEL_DEATHS]);
	
  os.setf(ios_base::fixed, ios_base::floatfield);
  os.precision(2);
  os << endl << "OSPF Network Characteristics" << endl;
  os << "--------------------------------------------------" << endl;
  os << "OSPF 2way_neighbors/node: " << neighborpernode << endl;
  os << "OSPF 2way_neighbor_changes/node/sec: " << neigh_change_node_sec <<endl;
  os << "avg_sec/OSPF 2way neighbor_life: " << neighlifetime << endl;
  os << "OSPF adjacencies/node: " << adjpernode << endl;
  os << "OSPF adjacency_changes/node/sec: " << adj_change_node_sec << endl; 
  os << "avg_LSAsOutSync: " << lsasoutsync << endl;
  os << "Secs per router-LSA origination:  " << secperorigrtrlsa << endl;
  os << "Router-LSA hopcount/Router-LSA installs:  " << rtrlsahopcount << endl;
  os << "Route changes/node/sec:  " << routechanges << endl;
  os << "Accumulated adjacencies/node:  " << adjaccum << endl;
#ifdef OSPF6_MANET_MPR_ADJ_REDUC_OSPF_MPR
  os << "Sync Node changes: "<<statistics[OSPF6_SYNC_NODE_CHANGE]<<endl;
#endif //OSPF6_MANET_MPR_ADJ_REDUC_OSPF_MPR

  os << endl << "Flooding Statistics" << endl;
  os << "--------------------------------------------------" << endl;
  os << "relaysel/node: " << relayselpernode << endl;
  os << "avg_sec/relaysel_life: " << relaysellifetime << endl;
  os.precision(0);
  os << "suppressed_LSAs: "<< statistics[OSPF6_LSA_FLOOD_SUPPRESS]<<endl;
  os << "relay_flood_LSAs: "<< statistics[OSPF6_LSA_FLOOD_RELAY]<<endl;
  os << "nonrelay_flood_LSAs: "<<statistics[OSPF6_LSA_FLOOD_NONRELAY]<<endl;
#ifdef OSPF6_MANET_MPR_RETRANS_OSPF_MPR
  os << "relay_flood_LSAs retransmissions: " << statistics[OSPF6_LSA_FLOOD_RELAY_RETRANS] << endl;
#endif //OSPF6_MANET_MPR_RETRANS_OSPF_MPR

  os << endl << "Neighborhood Statistics" << endl;
  os << "--------------------------------------------------" << endl;
  os << "Hello received: " << statistics[OSPF6_HELLO_RECV] << endl;
  os << "Inactivity calls: " << statistics[OSPF6_INACTIVITY_CALLS] << endl;

#ifdef OSPF6_MANET_MPR_TOPO_REDUC
  os << endl << "Topology Reduction Statistics" << endl;
  os << "--------------------------------------------------" << endl;
  os << "Advertized neighbors: "<< statistics[OSPF6_LSA_NEIGHBOR_ADV]<<endl;
  os << "Total FULL neighbors: "<< statistics[OSPF6_LSA_NEIGHBOR_FULL]<<endl;
#endif //OSPF6_MANET_MPR_TOPO_REDUC

  os << endl;
}

void OSPF6DApplication::PrintOspf6PktLine(double pkt,double byte,Time_t duration)
{
	//XXX adding ipv4 header to ospf6 bytes. Should this be done????
	
	os.setf(ios_base::fixed, ios_base::floatfield);
	os.precision(0);
	os << setw(8) << pkt 
		<< setw(14) << byte+20*pkt;
	os.precision(2);
  os << setw(12) << (double) ((8*(byte+20*pkt)) / duration) / 1000 
		<< endl;;
}

OSPF6DInstance *
OSPF6DApplication::AddNode(Node* n, char *file) 
{
  if (instance_ == NULL) 
    instance_ = this;
  vpOSPF6Instances::iterator it;
  char buf[32];
  OSPF6DInstance *ospf6inst;

  //add a loopback interface
  sprintf(buf, "0.0.%d.0", n->Id()+1);
  IPAddr_t ip = IPAddr(buf);
  n->AddInterface(L2Proto802_3(), ip, Mask(24), MACAddr::Allocate(),false,"lo");

  //Install ospf6 instance in instance vector
  ospf6inst = new OSPF6DInstance(n, file);
  ospf6inst->SetApp(this);
  for (it = vpInstances.begin(); it!=vpInstances.end(); ++it)
  {
    if ((*it)->GetNode()->Id() == n->Id())
    {
      printf("node %d already added\n", n->Id());
      exit(0);
    }
  }
  vpInstances.push_back(ospf6inst);

	//set time for statistics to begin being collected
	ospf6inst->SetStartStatTime(start_stat_time);

  //Set neighbor polling interval
  ospf6inst->SetNeighborPollingInterval(polling_interval);

  //set L3 protocol
  ospf6inst->SetL3Proto(IPV4::Instance());

  return ospf6inst;
}

// Main packet receive method for OSPF6D
// From here, need to find correct instance of protocol and send to it
void 
OSPF6DApplication::Receive(Node *n, Packet* p, 
		                       IPAddr_t src, IPAddr_t dst, Interface *i)
{
  OSPF6DInstance* ospfinst;

  // Demultiplex received packets and send to appropriate instance
  vpOSPF6Instances::iterator it;
  for (it = vpInstances.begin(); it!=vpInstances.end(); ++it)
  {
    if ((*it)->GetNode()->Id() == n->Id())
    {
      ospfinst = (*it);
      break;
    }
  }
  assert(ospfinst);
  ospfinst->ReceiveMessage(p, src, dst, i);
  // clean up packet
  delete p;

}
void 
OSPF6DApplication::LogNeighbors(double interval)
{
  polling_interval=interval;
}

// Return handle to appropriate OSPF6DInstance
// This is called when trying to schedule a timer
//
// Note:  early in simulation, inst_ospf might not be set yet, so
//        we fall back on comparing the router_id in that case
//
// XXX may not be needed anymore due to thread_master->ospf6_inst pointer
//
OSPF6DInstance * 
OSPF6DApplication::LookupInstance(struct ospf6 *o) 
{
  vpOSPF6Instances::iterator it;
  for (it = vpInstances.begin(); it !=vpInstances.end(); ++it)
  {
    if ((*it)->get_inst_ospf6() == o) {
      //cout << "Match on ospf6 found " << (*it)->get_router_id() << endl;
      return (*it);
    } else if ((*it)->get_router_id() == o->router_id) {
      //cout << "Match on router_id found " << (*it)->get_router_id() << endl;
      return (*it);
    }
  }
  cout << "ID not found " << o->router_id << " for instance " << o << endl;
  exit(1);
}


/*
 *  class OSPF6DInstance
 */
OSPF6DInstance::OSPF6DInstance(Node *n, char *file) 
: node(n), inst_ospf6(NULL), needcreateconf(false), start_stat_time(0),
neighbor_list(NULL), last_neighbor_list(NULL), neighbor_polling_interval(0),
neighbor_last_time(0), neighbor_running_average(0), 
minLSInterval(0), minLSArrival(0),
neighbor_running_average_time(0), neighbor_changes(0), neighbor_changes_time(0) 
{
  node->SetOspf6dInst(this); // for simulator to access ospf6 data structers

  if (file == NULL) {
    char cwd[MAXPATHLEN];
    file = (char *) malloc(MAXPATHLEN);

     memset (cwd,0,MAXPATHLEN);
     memset (file,0,MAXPATHLEN);
     getcwd (cwd, MAXPATHLEN);
   
     if (mkdir("conf", 0777) < 0 && errno !=  EEXIST) {
      printf("OSPF6DInstance::OSPF6DInstance failed errno=%d : %s\n", 
                                                  errno, strerror(errno));
      exit(0);
     }
    sprintf(file,"%s/conf/ospf6d_%d.conf",cwd,node->Id()+1);
    needcreateconf = true;
  }

  config_file = file;

  if (needcreateconf) 
    free(file); 
  Ospf6Main();
}

//Copy Constructor
OSPF6DInstance::OSPF6DInstance(const OSPF6DInstance& o) 
{  //TODO  copy
}

OSPF6DInstance::~OSPF6DInstance() 
{
  if (neighbor_list)
    delete [] neighbor_list;
  if (last_neighbor_list)
    delete [] last_neighbor_list;
}

const OSPF6DInstance& OSPF6DInstance::operator= (const OSPF6DInstance& right)   {
  // TODO 
}
     
void
OSPF6DInstance::ScheduleTimer(Event_t e, OSPF6DEvent*& ev, Time_t t) {
  ev = new OSPF6DEvent(e);
  timer.Schedule(ev, t, (TimerHandler *)this);
}

void
OSPF6DInstance::ScheduleTimerWithThread(Event_t e, OSPF6DEvent*& ev, Time_t t, struct thread *thr) {
  ev = new OSPF6DEvent(e);
  ev->t = thr;
  thr->sim_event = (void *) ev;  // Back pointer for possible thread cancel
  timer.Schedule(ev, t, (TimerHandler *)this);
}

void
OSPF6DInstance::Timeout(TimerEvent *pTimerEvent)
{
  OSPF6DEvent *ev = (OSPF6DEvent *) pTimerEvent;
  struct thread* t = ev->t;
  SetGlobals();

  switch (ev->event)
  {
    case OSPF6DEvent::TYPE_FETCH :
      // Fetch queued threads until they are exhausted
      while (thread_fetch(master, &thread))
        thread_call(&thread);
      // Come back SCHED_GRANULARITY seconds later to look for more threads
      ScheduleTimer(OSPF6DEvent::TYPE_FETCH, FetchTimeout, SCHED_GRANULARITY);
      break;
    case OSPF6DEvent::TYPE_TIMER :
      (*t->func) (t);
      THREAD_OFF(t);
      ev = NULL;
			//put the thread back on the unused list, so it is not lost
      break;
		case OSPF6DEvent::TYPE_START :
			/* Read configuration and command line options 
			 * parse config file 
			 */
			if (needcreateconf) CreateConfFile();
#ifdef QUAGGA_0_96_5
   vty_read_config (NULL, (char *) config_file.c_str(), NULL);
#else
			vty_read_config ((char *) config_file.c_str(), NULL);
#endif // QUAGGA_0_96_5



			inst_ospf6 = ospf6; 
			TraceEvent_sim(2, "Ospf6 Router Started");
			break;
		case OSPF6DEvent::TYPE_NEIGHBOR_LOG :
      UpdateNeighborStats();
      neighbor_last_time = Simulator::Now();
      memcpy(last_neighbor_list, neighbor_list, (num_nodes * sizeof(char)));
      ScheduleTimer(OSPF6DEvent::TYPE_NEIGHBOR_LOG, NeighborLogTimeout, 
        neighbor_polling_interval);
      break;
    default :
      cerr << "Unknown timer type: " << ev->event << " at time " <<
        Simulator::Now() << endl;
      exit(1);
      break;
  }

  if(ev)     // even may have already been deleted
	delete ev;
}

void
OSPF6DInstance::SetGlobals()
{
  ospf6 = inst_ospf6;
  master = inst_master;
  phost = inst_phost;
  cmdvec = inst_cmdvec;
  iflist = inst_iflist;
  vtyvec = inst_vtyvec;
  Vvty_serv_thread = inst_Vvty_serv_thread;
	vty_cwd = inst_vty_cwd;
  sim_node = node;
  zlog_default = inst_zlog_default;
	ifaddr_ipv4_table = inst_ifaddr_ipv4_table;
}

void
OSPF6DInstance::Ospf6Main()
{
  // Make master thread
  inst_master = thread_master_create();
  bzero (inst_master, sizeof (struct thread_master));
  inst_master->ospf6_inst = (void *) this;


#ifdef QUAGGA_0_96_5
   inst_zlog_default = openzlog ("quagga-gtnets", ZLOG_STDOUT, ZLOG_OSPF6,
                            LOG_CONS|LOG_NDELAY|LOG_PERROR|LOG_PID,
                            LOG_DAEMON);
#else
   inst_zlog_default = openzlog ("quagga-gtnets", ZLOG_OSPF6,
                            LOG_CONS|LOG_NDELAY|LOG_PID,
                            LOG_DAEMON);
#endif //QUAGGA_0_96_5

  //zprivs_init (&ospf6d_privs);  //Unused
  /* initialize zebra libraries */
  //signal_init ();    // Unused
  cmd_init (1);    
  inst_cmdvec = cmdvec;
  inst_phost = phost;

  vty_init (master);
  inst_vtyvec = vtyvec;
  inst_Vvty_serv_thread = Vvty_serv_thread;
	inst_vty_cwd = vty_cwd;

  memory_init ();    

  SetGlobals(); // Set global variables to this instance

  if_init ();
	inst_ifaddr_ipv4_table = ifaddr_ipv4_table;
  inst_iflist = iflist;

  access_list_init ();
  prefix_list_init ();

  /* initialize ospf6 */
  ospf6_init ();

  /* sort command vector */
  sort_node ();
}

void
OSPF6DInstance::Start(double delay)
{
  SetGlobals(); // Set global variables to this instance

#ifndef SIM_REPLACE_SCHEDULER
  //Schedule timer to do the thread fetch
  ScheduleTimer(OSPF6DEvent::TYPE_FETCH, FetchTimeout, 0);
#endif
	ScheduleTimer(OSPF6DEvent::TYPE_START, StartTimeout, delay);
  /* Neighbor logging.
   * We will keep track of the neighbor vector over time.  A vector is
   * needed because if we a pure counter will only track the total
   * number of neighbors and won't have information as to whether an
   * equal number of neighbors were added and deleted in the last interval.
   */
  num_nodes = Node::nextId;
  assert(neighbor_list = new char[num_nodes]);
  assert(last_neighbor_list = new char[num_nodes]);
  memset((void *) neighbor_list, 0, num_nodes * sizeof(char));
  memset((void *) last_neighbor_list, 0, num_nodes * sizeof(char));
  ScheduleTimer(OSPF6DEvent::TYPE_NEIGHBOR_LOG, NeighborLogTimeout, 
    start_stat_time);
  neighbor_last_time = start_stat_time;
}

void OSPF6DInstance::Finalize()
{
	SetGlobals(); // Set global variables to this instance
	ospf6_delete(ospf6);
}

void
OSPF6DInstance::CreateConfFile()
{
  FILE *fp = NULL;
  fp = fopen (config_file.c_str(), "w");
  if (fp == NULL) {
    printf("Cannot open file to write in OSPF6Instance::CreateConfFile\n");
    exit (0);
  }
  //Logging
  if (logfile.length() > 0)
  {
    fprintf(fp,"log file %s\n",logfile.c_str());
    //    fprintf(fp,"debug ospf6 spf process\n");
    //    fprintf(fp,"debug ospf6 message hello\n");
    //    fprintf(fp,"debug ospf6 message lsupdate\n");
    fprintf(fp,"debug ospf6 message dbdesc\n");
    //    fprintf(fp,"debug ospf6 message lsack\n");
    //    fprintf(fp,"debug ospf6 message lsreq\n");
    //    fprintf(fp,"debug ospf6 lsa router\n");
    //    fprintf(fp,"debug ospf6 neighbor\n"); 
    //    fprintf(fp,"debug ospf6 interface\n");
    //    fprintf(fp,"debug ospf6 database\n");
  }

  fprintf(fp,"no log stdout\n");
  fprintf(fp,"hostname %s\n",hostname.c_str());
  fprintf(fp,"password ospf6\n");

  //Interface config
  ifc.CreateConfFileInterfaces(fp);
    
  //Define as OSPF6 router
  fprintf(fp,"router ospf6\n");
  fprintf(fp,"  router-id %s\n",IPAddr::ToDotted(routerid));
  //Only set if intialized by Set..() 
  if (start_stat_time > 0)
    fprintf(fp,"  router statistics start-time %.0f\n", start_stat_time);
  if (minLSInterval > 0)
    fprintf(fp,"  router minls-interval %d\n", minLSInterval);
  if (minLSArrival > 0)
    fprintf(fp,"  router minls-arrival %d\n", minLSArrival);
  ifc.CreateConfFileAreaAttach(fp);

  fclose(fp);    
}

void
OSPF6DInstance::SetHostName(char *name)
{
	hostname = name;
}

void
OSPF6DInstance::SetRouterId(IPAddr_t ip)
{
  if (ip < 1)
  {
    printf("routerid must be greater than 0 in SetRouterId()\n");
    exit(0);
  }
  routerid = ip;
  // Add to Node's static vector container indexed by router ID
  Node::RouterIdToNodeInit((int)(routerid & 0xff), GetNode());
}
void OSPF6DInstance::SetMinLSInterval(int ls_int)
{
	minLSInterval = ls_int;
}

void OSPF6DInstance::SetMinLSArrival(int ls_int)
{
	minLSArrival = ls_int;
}

void
OSPF6DInstance::SetLogFileName(char *name)
{
   char cwd[MAXPATHLEN];

   memset (cwd,0,MAXPATHLEN);
   getcwd (cwd, MAXPATHLEN);
   
   if (mkdir("log", 0777) < 0 && errno !=  EEXIST) {
     printf("OSPF6DInstance::SetLogFileName failed errno=%d : %s\n", 
                                              errno, strerror(errno));
     exit(0);
   }
	 strcat(cwd,"/log/");
	 strcat(cwd,name);
	 logfile = cwd;
}

void OSPF6DInstance::SetInterfaceMTU(Interface *i, int mtu_bytes)
{
  ifc.SetInterfaceMTU(i->GetName(), mtu_bytes);
}

void OSPF6DInstance::SetInterfaceJitter(Interface *i, int jitter_msec)
{
  ifc.SetInterfaceJitter(i->GetName(), jitter_msec);
}

void OSPF6DInstance::SetInterfaceFloodDelay(Interface *i, int flood_delay_msec)
{
  ifc.SetInterfaceFloodDelay(i->GetName(), flood_delay_msec);
}

void OSPF6DInstance::SetInterfaceType(Interface *i, char *type)
{
  ifc.SetInterfaceType(i->GetName(), type);
}

void OSPF6DInstance::SetInterfaceFlooding(Interface *i, char *type)
{
  ifc.SetInterfaceFlooding(i->GetName(), type);
}

void OSPF6DInstance::SetInterfaceCost(Interface *i, int cost) 
{
  ifc.SetInterfaceCost(i->GetName(), cost);
}

void OSPF6DInstance::SetInterfaceArea(Interface *i, IPAddr_t a) 
{
  ifc.SetInterfaceArea(i->GetName(), a);
}

void OSPF6DInstance::SetInterfaceHelloInterval(Interface *i, int hello_int)
{
  ifc.SetInterfaceHelloInterval(i->GetName(), hello_int);
}

void OSPF6DInstance::SetInterfaceDeadInterval(Interface *i, int dead)
{
  ifc.SetInterfaceDeadInterval(i->GetName(), dead);
}

void OSPF6DInstance::SetInterfaceRetransmitInterval(Interface *i, int rx) 
{
  ifc.SetInterfaceRetransmitInterval(i->GetName(), rx);
}

void OSPF6DInstance::SetInterfaceAckInterval(Interface *i, int ack_int)
{
  ifc.SetInterfaceAckInterval(i->GetName(), ack_int);
}

void OSPF6DInstance::SetInterfaceDiffHellos(Interface *i, bool diff) 
{
  ifc.SetInterfaceDiffHellos(i->GetName(), diff);
}

//OR
void OSPF6DInstance::SetInterfacePushBackInterval(Interface *i, int pb_int)
{
  ifc.SetInterfacePushBackInterval(i->GetName(), pb_int);
}

void OSPF6DInstance::SetInterfaceSmartPeering(Interface *i, bool smart)
{
  ifc.SetInterfaceSmartPeering(i->GetName(), smart);
}

void OSPF6DInstance::SetInterfaceUnsynchAdj(Interface *i, bool adj)
{
  ifc.SetInterfaceUnsynchAdj(i->GetName(), adj);
}

void OSPF6DInstance::SetInterfaceTopoReduc(Interface *i, bool tr)
{
  ifc.SetInterfaceTopoReduc(i->GetName(), tr);
}

//MDR
void OSPF6DInstance::SetInterfaceBackupWaitInterval(Interface *i, int bw_int)
{
  ifc.SetInterfaceBackupWaitInterval(i->GetName(), bw_int);
}

void OSPF6DInstance::SetInterfaceTwoHopRefresh(Interface *i, int interval)
{
  ifc.SetInterfaceTwoHopRefresh(i->GetName(), interval);
}

void OSPF6DInstance::SetInterfaceHelloRepeatCount(Interface *i, int count)
{
  ifc.SetInterfaceHelloRepeatCount(i->GetName(), count);
}

void OSPF6DInstance::SetInterfaceAdjConnectivity(Interface *i, char* conn) 
{
  ifc.SetInterfaceAdjConnectivity(i->GetName(), conn);
}

void OSPF6DInstance::SetInterfaceLSAFullness(Interface *i, char* fullness) 
{
  ifc.SetInterfaceLSAFullness(i->GetName(), fullness);
}

void OSPF6DInstance::SetInterfaceNonPersistentMDR(Interface *i, bool nonpers) 
{
  ifc.SetInterfaceNonPersistentMDR(i->GetName(), nonpers);
}

// Transform calls of the form sendmsg(ospf6_sock, &smsghdr, 0) to
// IPV4::DataRequest(Node *n, Packet *p, void *v) where the
// last argument is IPV4ReqInfo
int 
OSPF6DInstance::SendMessage(int s, const struct msghdr *msg, int flags) 
{
  int ttl = 1; //XXX should complete setsockopt_sim/getsockopt_sim for TTL
  IPAddr_t srcv4;
  IPAddr_t dstv4;
  struct in6_addr *srcv6; 
  struct in6_addr *dstv6;

  dstv6 = &((struct sockaddr_in6 *) msg->msg_name)->sin6_addr;
  srcv6 = &((in6_pktinfo *)((struct cmsghdr *)(msg->msg_control)+1))->ipi6_addr;

  ipv6_to_ipv4(srcv6, &srcv4);
  ipv6_to_ipv4(dstv6, &dstv4);  

  IPV4ReqInfo r(dstv4, srcv4, ttl, ProtoNumber);
  Packet* p = new Packet();
  Data* d = new Data(msg->msg_iov[0].iov_len, (char*)msg->msg_iov[0].iov_base);
  p->PushPDU(d);

  l3Proto->DataRequest(node, p, (void *)&r, GetNode()->GetIfByIP(srcv4));
  return msg->msg_iov[0].iov_len;  //assumes 1 element vector
}

// Transform received packets so that they can be received by
// the ospf6 C function ospf6_message_process(message, src, dst, o6i)
void 
OSPF6DInstance::ReceiveMessage(Packet *p, IPAddr_t srcv4, 
		                           IPAddr_t dstv4, Interface *i)
{
  struct in6_addr src;
  struct in6_addr dst;
  struct ospf6_header *oh;
  struct ospf6_interface *oi;
  Data *d;
  char srcname[64], dstname[64];
  Size_t len;

  SetGlobals();
	
  ipv4_to_ipv6(&srcv4, &src);
  ipv4_to_ipv6(&dstv4, &dst);

  d = (Data*) p->PopPDU();
  oh = (struct ospf6_header *) d->data;
  len = d->size;

  oi = ospf6_interface_lookup_by_ifindex (i->GetIfIndex());
  if (!oi)
    return;

  /* Log */
  if (IS_OSPF6_DEBUG_MESSAGE (oh->type, RECV))
    {
      inet_ntop (AF_INET6, &src, srcname, sizeof (srcname));
      inet_ntop (AF_INET6, &dst, dstname, sizeof (dstname));
      zlog_info ("%s received on %s",
                 OSPF6_MESSAGE_TYPE_NAME (oh->type), oi->interface->name);
      zlog_info ("    src: %s", srcname);
      zlog_info ("    dst: %s", dstname);
/*  Will fail if appended LLS is present
      if (len != ntohs (oh->length))
        zlog_info ("Message length does not match actually received: %d", len);
*/
      switch (oh->type)
        {
          case OSPF6_MESSAGE_TYPE_HELLO:
#ifdef OSPF6_MANET_MPR_FLOOD
            if (oi->type == OSPF6_IFTYPE_MANETRELIABLE)
              ospf6_mhello_print(oh, len);
            else
              ospf6_hello_print (oh);
#else
            ospf6_hello_print (oh);
#endif //OSPF6_MANET_MPR_FLOOD
            break;
          case OSPF6_MESSAGE_TYPE_DBDESC:
            ospf6_dbdesc_print (oh);
            break;
          case OSPF6_MESSAGE_TYPE_LSREQ:
            ospf6_lsreq_print (oh);
            break;
          case OSPF6_MESSAGE_TYPE_LSUPDATE:
            ospf6_lsupdate_print (oh);
            break;
          case OSPF6_MESSAGE_TYPE_LSACK:
            ospf6_lsack_print (oh);
            break;
          default:
            zlog_info ("Unknown message");
            break;
        }
    }

  if (CHECK_FLAG (oi->flag, OSPF6_INTERFACE_PASSIVE))
    {
      if (IS_OSPF6_DEBUG_MESSAGE (oh->type, RECV))
        zlog_info ("Ignore message on passive interface %s",
                   oi->interface->name);
    }

  switch (oh->type)
    {
      case OSPF6_MESSAGE_TYPE_HELLO:
        ospf6_hello_recv (&src, &dst, oi, oh, len);
        break;

      case OSPF6_MESSAGE_TYPE_DBDESC:
        ospf6_dbdesc_recv (&src, &dst, oi, oh, len);
        break;

      case OSPF6_MESSAGE_TYPE_LSREQ:
        ospf6_lsreq_recv (&src, &dst, oi, oh);
        break;

      case OSPF6_MESSAGE_TYPE_LSUPDATE:
        ospf6_lsupdate_recv (&src, &dst, oi, oh);
        break;

      case OSPF6_MESSAGE_TYPE_LSACK:
        ospf6_lsack_recv (&src, &dst, oi, oh);
        break;

      default:
        if (IS_OSPF6_DEBUG_MESSAGE (OSPF6_MESSAGE_TYPE_UNKNOWN, RECV))
          zlog_info ("Unknown message");
        break;
    }
}
void OSPF6DInstance::SetNeighborPollingInterval(double interval)
{
  neighbor_polling_interval = interval;
}

void OSPF6DInstance::UpdateNeighborStats()
{
  int neighbor_count_ = 0;
  int changes_ = 0;
  Time_t since = Simulator::Now() - neighbor_last_time;
  if (node->IsMobile()) 
    node->UpdateLocation();
  const NodeVec_t& nodes = Node::GetNodes();
  for (NodeVec_t::const_iterator i = nodes.begin(); i != nodes.end(); ++i) 
  {
    Node* n = *i;
    if (node->Id() == n->Id()) 
      continue;    // skip self node
    if (n->IsMobile()) 
      n->UpdateLocation();
    Meters_t d = node->Distance(n);
    if (d < node->GetRadioRange()) {
      neighbor_count_++;
      neighbor_list[n->Id()] = 1;
    } else {
      neighbor_list[n->Id()] = 0;
    }
    if (neighbor_list[n->Id()] != last_neighbor_list[n->Id()])
      changes_++;
  }
  // Update averages and counts
  neighbor_running_average += (neighbor_count_ * since);
  neighbor_running_average_time += since;
  if (since) {
    neighbor_changes += changes_;
    neighbor_changes_time += since;
  }
  return;
}
