/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Worm Application class
// Monirul I Sharif.  Georgia Tech, Fall 2003

// Define a Worm Application class

#ifndef __worm_application_h__
#define __worm_application_h__

#include "common-defs.h"
#include "application.h"
#include "timer.h"
#include "udp.h"
#include "rng.h"

// This version only supports UDP packets

//Doc:ClassXRef
class WormApplication : public Application, public TimerHandler {
  //Doc:Class Define a application that works as the Sapphire worm
  //Doc:Class 
public:
  // Define a number of constructors for different needs
  //Doc:Method
  WormApplication(Node*);   // Just specify endpoint
    //Doc:Desc Specifies a node to attach the worm application to. 

  void Timeout(TimerEvent *);  // Timer Handler

  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop

  virtual void AttachNode(Node*); // Note which node attached to
  virtual Application* Copy() const;// Make a copy of the application

public:

  void Infect();
  void StartInfection();
  bool IsInfected();
  Node* AttachedNode();
  static void SetDefaultScanRate(Count_t );
  static void SetDefaultBaseIP(IPAddr_t );
  static void SetDefaultScanRange(Count_t);

   
private:
  bool		infected;      // indicates if this node is in infected stage
  bool          vulnerable;
  UDP*		udp;
  Node*		node;
  Timer         timer;
  Random*       rngT;    // random number generator for targeting
  TimerEvent*   timeoutevent;
private:
  static Count_t defaultscanspersecond;
  static Size_t defaultinfectionlength;
  static PortId_t defaultinfectionport;
  static double defaultvulnerability;
  static IPAddr_t defaultbaseIP;
  static Count_t defaultIPscanrange;

  Size_t infectionlength;
  Count_t scanspersecond;
  Count_t IPscanrange;
  IPAddr_t lasttarget;
  //  PortId_t infectionport; // we are not letting someone change that

  IPAddr_t GenerateNextIPAddress();
  void SendWormPacket();
  bool PacketIsWorm(Packet *);   // I am going to use this to check if packet 
                                 // is worm
  
  // the following method is called when packets are received from the port
  virtual void Receive(Packet *, L4Protocol *, Seq_t); // Data received
  void ScheduleNextPacket();
};

#endif

