/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

* $Id: application.cc,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

*/

// Georgia Tech Network Simulator - Layer 5 Application base class
// George F. Riley.  Georgia Tech, Spring 2002

// Define the layer 5 application interface

#include <iostream>

#include "application.h"
#include "simulator.h"
#include "event.h"

using namespace std;

// Define the start/stop events
class ApplicationEvent : public Event {
public:
  typedef enum { START, STOP } ApplicationEvent_t;
  ApplicationEvent() { }
  ApplicationEvent(Event_t ev) : Event(ev) { }
};

// Application Methods

// Copy constructor
Application::Application(const Application& a)
  : deleteOnComplete(a.deleteOnComplete), copyOnConnect(a.copyOnConnect)
{
}

Application::~Application()
{ // Destructor
}

void Application::Start(Time_t t)
{ // Schedule the Start event
  ApplicationEvent* startEvent = new ApplicationEvent(ApplicationEvent::START);
  Simulator::instance->Schedule(startEvent, t, this);
}

void Application::Stop(Time_t t)
{ // Schedule the Stop Event
  ApplicationEvent* stopEvent = new ApplicationEvent(ApplicationEvent::STOP);
  Simulator::instance->Schedule(stopEvent, t, this);
}

void Application::Handle(Event* e, Time_t)
{
  ApplicationEvent* ev = (ApplicationEvent*)e;
  switch(ev->event) {
    case ApplicationEvent::START :
      StartApp(); // Call the application startup code
      break;
    case ApplicationEvent::STOP :
      StopApp();  // Call the application shutdown code
      break;
  };
  delete ev;
}

// Subclasses should over-ride all the below as required
void Application::Receive(Packet* p,L4Protocol*,Seq_t)
{ // Data has been received 
  if (p) delete p;
}

void Application::Sent(Count_t, L4Protocol*)
{ // Data has been sent (and Ack'ed if reliable protocol)
}

void Application::CloseRequest(L4Protocol* l4)
{ // Connection has closed
  l4->Close();  // Just close back
}

void Application::Closed(L4Protocol*)
{ // Connection has closed
}

void Application::ConnectionComplete(L4Protocol* p)
{ //. Connection request succeeded
}

void Application::ConnectionFailed(L4Protocol*)
{ // Connection request failed
};

bool Application::ConnectionFromPeer(L4Protocol*, IPAddr_t, PortId_t)
{
  return true; // Accepted by default
}

void Application::AttachNode(Node*)
{ // Overridden by subclass if needed
}

  
