/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Track min/max/average
// George F. Riley.  Georgia Tech, Winter 2002

// Defines a statistics collection for min/max/average

#ifndef __average_min_max_h__
#define __average_min_max_h__

#include <iostream>
#include <vector>

#include "common-defs.h"
#include "statistics.h"

//Doc:ClassXRef
class AverageMinMax : public Statistics {
  //Doc:Class Class {\tt AverageMinMax} defines a statistics collection
  //Doc:Class object that  dynamically tracks the average, minimum,
  //Doc:Class and maximum values for the measured stsatistics.
public:
  //Doc:Method
  AverageMinMax();
    //Doc:Desc Default constructor, no arguments.
  virtual ~AverageMinMax();
  bool Record(double);
  void Reset();
  void Log(std::ostream&, const char* hdr = NULL, const char sep = ',');
  Statistics* Copy() const;       // Make a copy of the statistics
  //Doc:Method
  Count_t Samples() { return nSamples;}
    //Doc:Desc Get a count of the number of sampled measurements.
    //Doc:Return Count of number of samples.

public:
  double               minSample;
  double               maxSample;
  double               total;
  Count_t              nSamples;
};
#endif



