// Specification of the Distributed Simulated Backplane
// George F. Riley, Georgia Tech, Summer 2000

#ifndef __BACKPLANE_H__
#define __BACKPLANE_H__
#include <sys/types.h>
// Misc types
typedef unsigned int   isize_t; // Item size type
typedef unsigned int   itemid_t;// Item Identifier type

#ifndef linux
#ifndef u_long
typedef unsigned long  u_long;
#endif
#ifndef u_short
typedef unsigned short u_short;
#endif
#ifndef u_char
typedef unsigned char  u_char;
#endif
#endif

typedef unsigned long  nodeid_t;// Node id type

// Definitions for FLAGS values
#define PROTO_OPTIONAL 0
#define PROTO_REQUIRED 1

#define ITEM_OPTIONAL  0
#define ITEM_REQUIRED  1
#define ITEM_BYTE_SWAP 2

// Query application if a protocol is contained in an internal format message
// DSim can skip the entire data item list if not contained
typedef int (*ExportQueryCb_t)(char*, int, void*);

// Define the prototype for the Data Item export callbacks
// Parameters are :
//  char* Source buffer
//  char* Return buffer
//  int   Return buffer maximum length (bytes!)
//  void* User context
// Return value is number of bytes actually copied to return buffer
typedef int(*ItemExportCb_t)(char*, char*, int, void*);

// Define the prototype for the Data Item import callback
// Parameters are:
// char* Source buffer
// int   Length of source buffer
// char* Return buffer
// void* User context
typedef int(*ItemImportCb_t)(char*, int, char*, void*);


  // Defien the prototype for the Data Item "Default" callback
// Parameters are:
// char* Return buffer
// void* User context
typedef int(*ItemDefaultCb_t)(char*, void*);

// Define the handle to DSim data structures (blind pointers)
typedef void* DSHandle_t; // Pointer to global info structure
typedef void* PRHandle_t; // Pointer to a protocol info structure
typedef void* ITHandle_t; // Pointer to a data item info structure

// Initialize the DSim backplane
DSHandle_t InitializeDSim(nodeid_t myid);

// Register a new protocol
PRHandle_t RegisterProtocol(char*, int, ExportQueryCb_t, void*);

// Regster a new data item
ITHandle_t RegisterItem(PRHandle_t, char*, int, isize_t,
                        ItemExportCb_t, void*,
                        ItemImportCb_t, void*,
                        ItemDefaultCb_t,void*);

// Informs the backplane that all registration has completed, and the
// consenus protocol should be performed.
void RegistrationComplete(nodeid_t myid, unsigned int npeers);

// Find out how big the generic message buffers need to be (bytes)
isize_t InquireMessageSize();

// Export a message to generic format
int ExportMessageWrapper( char*, char*, int, char*, int*);
  // char*       - Pointer to input buffer
  // char*       - Pointer to baggage buffer
  // int         - Length of baggage buffer
  // char*       - Pointer to output buffer
  // int*        - Length of output buffer (in/out)

// Import a message from generic forma
int ImportMessageWrapper( char*, int, char*, char*, int*);
  // char*       - Pointer to input buffer
  // int         - Length of input buffer
  // char*       - Pointer to output buffer
  // char*       - Pointer to baggage buffer
  // int*        - Length of baggage buffer

// Get the current handle
DSHandle_t GetDSimHandle(void);

// Convert size in bits to size in bytes
int SizeInBytes(int);

char* RoundUpBuffer(char*, char*, int, int); // Insure proper byte alignment

int GetPeerSocket (int ); // Get the socket handle for the specified peer

// Put a data item on a tcp socket, 32bit item length followed by data
int PutCountedItem(const int s, const char* b, const int l);

// Get a data item from a tcp socket.  32 bit length followe by the data
int GetCountedItem(int s, char* b, int l);

// The versions using CQueue structures as Import input and Export output
// These versions avoid an extra data copy at each end
#include "cq.h"
int ExportMessageQ( char*, char*, int, CQueue*, char* pBase, size_t, size_t);
  // Same as above, except CQueue instead of Outputbuffer, length
  // Last param is pIn to use (since CQueue may not be updated yet)
int ImportMessageQ( CQueue*, char* pBase, size_t, char*, char*, int*, size_t);
  // Same as above, except CQueue instead of Inputbuffer, length

#endif



