/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Common Definitions
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __common_defs_h__
#define __common_defs_h__

typedef double         Time_t;         // Simulation time
typedef unsigned long  SimulatorUid_t; // Unique ID for each event
typedef unsigned long  Event_t;        // Idenifies events in handler
typedef unsigned long  HandlerId_t;    // Unique ID for each handler
typedef unsigned long  Count_t;        // For counting things
typedef long           NCount_t;       // For counting things, with negative
typedef double         DCount_t;       // Floating point counts
typedef unsigned long  FId_t;          // For flow identifiers
typedef unsigned long  Size_t;         // For sizes
typedef double         Rate_t;         // Rate (bits/sec)
typedef unsigned long  IPAddr_t;       // IP Address
typedef unsigned long  Mask_t;         // IP Address masks
typedef unsigned long  Proto_t;        // Protocol identifier
typedef unsigned int   Layer_t;        // Protocol layer number
typedef unsigned long  Version_t;      // Which version of protocol
typedef unsigned long  NodeId_t;       // For identifying nodes
typedef unsigned long  PortId_t;       // Port number
typedef unsigned long  Bitmap_t;       // 32 bit flags
typedef unsigned short Byte_t;         // 8 bit quantity (not really..)
typedef unsigned short Word_t;         // 16 bit quantity
typedef unsigned long  Long_t;         // 32 but quantity
typedef unsigned long  Seq_t;          // Sequence numbers
typedef double         Mult_t;         // Dimensionless multiplier
typedef float          Weight_t;       // Link Cost Weight factor
typedef double         Meters_t;       // Meters for node locations
typedef unsigned long  Dist_t;         // For distances in routing
typedef unsigned long  SystemId_t;     // System identifier for distributed
typedef unsigned long  Color_t;        // For R/G/B components of pixels
typedef double         Angle_t;        // Angles (radians)
typedef double         Joules_t;       // Essentially a 3.6e6 factor to kWhr

#define ALLBITS        ((Bitmap_t)0xffffffffl)
#define IPADDR_NONE    ((IPAddr_t)0x0)
#define NO_PORT        ((PortId_t)0x0)
#define MASK_ALL       ((Mask_t)0xffffffffl)
#define MAX_COUNT      ((Count_t)0xffffffffl)
#define NODE_NONE      ((NodeId_t)0xffffffffl)
#define INFINITE_TIME  ((Time_t)1e307)
#define INFINITE_VALUE 1e307
#define SPEED_LIGHT    ((Mult_t)299792458.0)

#define nil         0

#endif
