// Circular message passing queue
// George F. Riley, Georgia Tech, Winter 2001

#ifndef __CQ_H__
#define __CQ_H__
#include <stddef.h>

// This defines the Queue management structure. All buffer pointers are
// relative to pBase. This is to allow this to work inshared memory
// regions, where the two processors will not have the same (necessarily)
// logical memory addresses for the common region.
typedef struct {
    size_t pFirst;
    volatile size_t pIn;
    volatile size_t pOut;
    size_t pLimit;
} CQueue;

#ifdef __cplusplus
extern "C" {
#endif
void   QSetBuf(CQueue*, size_t ofs, size_t l); // Set the data buffer address
size_t QAdvance(CQueue*, size_t, size_t); // Advance ptr by specified amt
size_t QData(CQueue* );  // How much data is in the buffer?
size_t QSpace(CQueue* ); // How much space is available in the buffer
void   QPut(CQueue*, char*, char*, size_t); // Put some data in the buffer
void   QGet(CQueue*, char*, char*, size_t); // Get some data from the buffer
size_t QDiff(CQueue*, size_t, size_t); // Distance from p1 to p2
// QPutN puts without chaning pIn, instead returns advance value
size_t QPutN(CQueue*, char*, char*, size_t, size_t);
// QGetN gets without chaning pOut, instead returns advance value
size_t QGetN(CQueue*, char*, char*, size_t, size_t);
// Debug Routines
void   QDump(CQueue*, char*); // Debug..
void   QDumpPtrs(CQueue*); // Debug..
void   QDumpPartial(CQueue*, char*, size_t, size_t);
#ifdef __cplusplus
}
#endif

#ifdef OLD_CLASS_VERSION
class CQueue {
  public :
    CQueue() : pFirst(0), pIn(0), pOut(0), pLimit(0) { };
    CQueue( char* b, size_t l ) : pFirst(b), pIn(b), pOut(b), pLimit(b+l) { };
    void SetBuf(char* buf, size_t l); // Set the data buffer address
    size_t Data();  // How much data is in the buffer?
    size_t Space(); // How much space is available in the buffer
    void   Put(char*, size_t); // Put some data in the buffer
    void   Get(char*, size_t); // Get some data from the buffer
  private :
    char* pFirst;
    char* pIn;
    char* pOut;
    char* pLimit;
};
#endif
#endif

