/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Data Descriptors
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __datapdu_h__
#define __datapdu_h__

#include "pdu.h"
#include "seq.h"

class Packet;
//Doc:ClassXRef
class Data : public PDU {
public:
  Data();
  Data(Size_t s, char* d = nil, Count_t msg = 0, Count_t resp = 0);
  Data(const std::string&); // Construct from string
  Data(char*, Size_t&, Packet*); // Construct from serialized buffer
  Data(const Data&);   // Copy constructor
  virtual ~Data();     // Destructor
  Size_t Size() const { return size;}
  // Serialization
  Size_t SSize();                   // Size needed for serialization
  char*  Serialize(char*, Size_t&); // Serialize to a buffer
  char*  Construct(char*, Size_t&); // Construct from buffer
  virtual void Clear();// Remove all associated data
  virtual void Add(Size_t s, const char* d = 0);// Add some data to end
  virtual void Remove(Size_t);      // Remove data from head
  //  virtual Data* RemovePointer(Size_t);      // Remove, return pointer
  // Inquire available data from (f,o) sequence pair
  virtual Size_t SizeFromSeq(const Seq&, const Seq&);
  // Inquire available data from offset
  virtual Size_t SizeFromOffset(Size_t);
  // Available size from sequence difference 
  virtual Size_t OffsetFromSeq(const Seq&, const Seq&);
  virtual Data* CopyFromOffset(Size_t, Size_t);  // Size, offset, ret pointer
  // Copy data, size, offset specified by sequence difference
  virtual Data* CopyFromSeq(Size_t, const Seq&, const Seq&);
  PDU*   Copy();                // Create a copy of this header
  PDU*   CopyS(Size_t);         // Copy with new size
  PDU*   CopySD(Size_t, char*); // Copy with new size, new data

  virtual char* Contents() const { return data;}
public:
  Size_t size;        // Number of data bytes
  char* data;         // Corresponding data (may be null)
  // The next two fields allow simulated applications to exchange some info
  Size_t msgSize;     // Total size of message
  Size_t responseSize;// Size of response requested
};


#endif

