/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Duplex Link class
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __duplexlink_h__
#define __duplexlink_h__

// Shortcut to create two point-to-point links between two nodes

#include "linkp2p.h"

class Interface;
//Doc:ClassXRef
class DuplexLink {
public:
  DuplexLink(Node* ln, Node* rn);               // Local/Remote nodes
  DuplexLink(Node* ln, const Linkp2p& ll,
             Node* rn, const Linkp2p& rl);      // Local/Remote nodes/links
  DuplexLink(Interface* li, Interface* ri);     // Local/Remote interfaces
  DuplexLink(Interface* li, const Linkp2p& ll,
             Interface* ri, const Linkp2p& rl); // Local/Remote interfaces
  DuplexLink(Node* ln, IPAddr_t la, Mask_t lm,  // Local node/address/mask
             Node* rn, IPAddr_t ra, Mask_t rm); // Remote node/address/mask
  DuplexLink(Node* ln, IPAddr_t la, Mask_t lm,  
             const Linkp2p& ll, // Local link
             Node* rn, IPAddr_t ra, Mask_t rm,
             const Linkp2p& rr);// Remote link
  ~DuplexLink(); // Destructor
public:
  Link*      local;
  Link*      remote;
  Interface* localif;
  Interface* remoteif;
private:
  void AddSimplexLinks(Node* ln, IPAddr_t la, Mask_t lm, const Link& ll,
                       Node* rn, IPAddr_t ra, Mask_t rm, const Link& rl);
};

#endif

