/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// GTitm Class Definitions
// Talal Jaafar, Georgia Tech, SPring 2002

#ifndef __GTITM_H__
#define __GTITM_H__


#include "transitnode.h"

typedef std::vector<Node*>  NodeList_t;
typedef std::vector<SubDomain*> SubList_t;
typedef std::vector<TransitNode*> TNodeList_t;

//Doc:ClassXRef
class GTitm {
//Doc:Class The class {\tt GTitm} defines an interface to the Georgia Tech Internetwork Topology Models (GT-ITM)
//Doc:Class Given the number of transit nodes, subdomains, subnodes, the topology is
//Doc:Class generated,and accessed through this object handler 
public:
  //Doc:Method
  GTitm(int avgStubsPerTrans, int TransNodes, int avgNodesPerStub, int LeafNum);
  //Doc:Desc This method is the default constructor
  //Doc:Arg1 Number of stubdomains on average per transit node
  //Doc:Arg2 Number of transit nodes
  //Doc:Arg3 Number of nodes on average per stubdomain
  //Doc:Arg4 Number of leaf nodes per node

  //Doc:Method
  TNodeList_t GetTList();
  //Doc:Desc This method is used to return the list of all Transit nodes
  //Doc:Return A vector of transit nodes

  //Doc:Method
  SubList_t   GetSList();
  //Doc:Desc This method is used to return the list of all SubDomains
  //Doc:Return A vector of subdomains

  //Doc:Method
  SubList_t     GTitm::GetSList(int nodeNum);
  //Doc:Desc This method is used to return the list of SubDomains associated with the given Transit node
  //Doc:Arg1 Transit node number
  //Doc:Return A vector of subdomains

  //Doc:Method
  NodeList_t    GTitm::GetNList(int subNum);
  //Doc:Desc This method is used to return the list of Nodes in  the subdomain with id of "subNum"
  //Doc:Arg1 Subdomain number
  //Doc:Return A vector of nodes

  //Doc:Method
  Node*         GTitm::GetTransNode(int TnodeNum);
  //Doc:Desc This method is used to return a transit node
  //Doc:Arg1 Transit node number
  //Doc:Return A pointer to the transit node
    
  //Doc:Method
  Node*         GTitm::GetSubNode(int TnodeNum, int nodeNum);
  //Doc:Desc This method is used to return a node given a Tnode id and the node id
  //Doc:Arg1 Transit node number
  //Doc:Arg2 Node number
  //Doc:Return A pointer to a subnode (node)

private:
  TNodeList_t T;         // list of the Transit nodes
  SubList_t  S;          // list of the SubDomains
};

#endif
