/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

#ifndef __ieee80211_h__
#define __ieee80211_h__

#include "sys/types.h"

#ifndef u_int32_t
#define u_int32_t unsigned long
#endif

/* Moved to l2proto802.11.h
#define MAC_ProtocolVersion     0x00

#define MAC_Type_Management     0x00
#define MAC_Type_Control        0x01
#define MAC_Type_Data           0x02
#define MAC_Type_Reserved       0x03

#define MAC_Subtype_RTS         0x0B
#define MAC_Subtype_CTS         0x0C
#define MAC_Subtype_ACK         0x0D
#define MAC_Subtype_DATA        0x00
*/

class PHY_MIB {
public:
        u_int32_t       CWMin;
        u_int32_t       CWMax;
        double          SlotTime;
        double          CCATime;
        double          RxTxTurnaroundTime;
        double          SIFSTime;
        u_int32_t       PreambleLength;
        u_int32_t       PLCPHeaderLength;
        double          PLCPDataRate;
};

class MAC_MIB {
public:
        //              MACAddress;
        //              GroupAddresses;
        u_int32_t       RTSThreshold;
        u_int32_t       ShortRetryLimit;
        u_int32_t       LongRetryLimit;
        u_int32_t       FragmentationThreshold;
        u_int32_t       MaxTransmitMSDULifetime;
        u_int32_t       MaxReceiveLifetime;
        //              ManufacturerID;
        //              ProductID;

        u_int32_t       TransmittedFragmentCount;
        u_int32_t       MulticastTransmittedFrameCount;
        u_int32_t       FailedCount;
        u_int32_t       RetryCount;
        u_int32_t       MultipleRetryCount;
        u_int32_t       FrameDuplicateCount;
        u_int32_t       RTSSuccessCount;
        u_int32_t       RTSFailureCount;
        u_int32_t       ACKFailureCount;
        u_int32_t       ReceivedFragmentCount;
        u_int32_t       MulticastReceivedFrameCount;
        u_int32_t       FCSErrorCount;
};

/*
 * IEEE 802.11 Spec, section 15.3.2
 *      - default values for the DSSS PHY MIB
 */
#define DSSS_CWMin                      31
#define DSSS_CWMax                      1023
#define DSSS_SlotTime                   0.000020        // 20us
#define DSSS_CCATime                    0.000015        // 15us
#define DSSS_RxTxTurnaroundTime         0.000005        // 5us
#define DSSS_SIFSTime                   0.000010        // 10us
#define DSSS_PreambleLength             144             // 144 bits
#define DSSS_PLCPHeaderLength           48              // 48 bits
#define DSSS_PLCPDataRate               1.0e6           // 1Mbps

#define DSSS_MaxPropagationDelay        0.000002        // 2us   XXXX

#define MAC_RTSThreshold                0               // bytes
#define MAC_ShortRetryLimit             7               // retransmittions
#define MAC_LongRetryLimit              4               // retransmissions
#define MAC_FragmentationThreshold      2346            // bytes
#define MAC_MaxTransmitMSDULifetime     512             // time units
#define MAC_MaxReceiveLifetime          512             // time units

static PHY_MIB PMIB =
{
        DSSS_CWMin, DSSS_CWMax, DSSS_SlotTime, DSSS_CCATime,
        DSSS_RxTxTurnaroundTime, DSSS_SIFSTime, DSSS_PreambleLength,
        DSSS_PLCPHeaderLength, DSSS_PLCPDataRate
};

static MAC_MIB MMIB =
{
        MAC_RTSThreshold, MAC_ShortRetryLimit,
        MAC_LongRetryLimit, MAC_FragmentationThreshold,
        MAC_MaxTransmitMSDULifetime, MAC_MaxReceiveLifetime,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

#define PLCP_HDR_LEN                            \
        ((PMIB.PreambleLength >> 3) +        \
         (PMIB.PLCPHeaderLength >> 3))

#define ETHER_HDR_LEN11                         \
        (PLCP_HDR_LEN +                         \
         sizeof(struct hdr_mac802_11) +         \
         ETHER_FCS_LEN)

#define ETHER_RTS_LEN                           \
        (PLCP_HDR_LEN + 20)

#define ETHER_CTS_LEN                           \
         (PLCP_HDR_LEN + 14)

#define ETHER_ACK_LEN                           \
         (PLCP_HDR_LEN + 14)

#define ETHER_DATA_LEN                           \
         (PLCP_HDR_LEN + 34)

#endif
