/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Simplified 802.11-ish layer (Boeing)

#include <iostream>

#include "debug.h"
#include "ieee802.11.h"
#include "l2protosimple.h"
#include "l2proto802.11.h"
#include "interface.h"
#include "ipv4.h"
#include "trace.h"
#include "node.h"
#include "hex.h"
#include "macaddr.h"
#include "queue.h"
#include "link.h"
#ifdef ETRACE
#include "etrace.h"
#endif 

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

// Uncomment below to enable debug level 0
//#define DEBUG_MASK 0x01

#define MYIP    ((string)iface->GetIPAddr())

using namespace std;

L2ProtoSimple::L2ProtoSimple() : L2Proto()
{ // Constructor
  dataRate =  11000000.0;
  //dataRate = iface->pLink->Bandwidth();    // iface not available here
  seqNo = 0;            // data frame sequence number

}

L2ProtoSimple::~L2ProtoSimple()
{
}

void L2ProtoSimple::BuildDataPDU(MACAddr src, MACAddr dst, Packet* p)
{
  Word_t dur = usec(txtime(ETHER_ACK_LEN, dataRate)
		      + DSSS_MaxPropagationDelay);
  DATA_frame* l2pdu = new DATA_frame(dur, dst, src, ++seqNo, 0);
  p->PushPDU(l2pdu);
}

void L2ProtoSimple::DataRequest(Packet* p)
{
  bool delete_packet = false; //FALSE;
  DEBUG0(cout << MYIP << ": DataRequest uid " << p->uid
	   << " at " << Simulator::Now() << endl);

  L2Header802_11* l2pdu = (L2Header802_11*)p->PeekPDU();
  DATA_frame* df = (DATA_frame*)l2pdu;
  if (df->addr1.IsBroadcast()) 
    delete_packet = true; //TRUE;
#ifdef ETRACE
  ETrace::Instance()->TracePacket(p, iface->GetNode(), iface,2, "s");
#endif
  iface->GetNode()->TracePDU(this, l2pdu);
  iface->GetLink()->Transmit(p, iface, iface->GetNode(), dataRate);
  if (delete_packet)
    delete p;
}

void L2ProtoSimple::DataIndication(Packet* p)
{
#ifdef ETRACE
  ETrace::Instance()->TracePacket(p, iface->GetNode(), iface,2, "r"); // 2 for layer 2
#endif
  L2HeaderSimple* l2pdu = (L2HeaderSimple*)p->PopPDU();
  MACAddr dst;

  iface->GetNode()->TracePDU(this, l2pdu);
      DATA_frame* dataFrame = (DATA_frame*)l2pdu;
      dst = dataFrame->addr1;
      if (dst.IsBroadcast()) {
		// Broadcast
		DEBUG0(cout << MYIP << ": received BC-DATA uid " << p->uid
			   << " at " << Simulator::Now() << endl);
		iface->HandleLLCSNAP(p);
	  }
      else if (dst == iface->GetMACAddr()) {
		// Unicast, it's for me
		DEBUG0(cout << MYIP << ": received DATA uid " << p->uid
			   << " at " << Simulator::Now() << endl);
		  iface->HandleLLCSNAP(p);
	  }
	  else {
		// Unicast, not for me
		delete p;
	  }
}

// L2SimpleHeader methods

L2Proto* L2ProtoSimple::Copy() const
{
  return new L2ProtoSimple(*this);
}

