/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
                                                                                
* Permission is hereby granted for any non-commercial use of this code
*/

// Glue code between ospf6d and gtnets
// Copyright 2004, The Boeing Company

#include "/usr/include/net/if.h"
#include "sim.h"
#include "stdio.h"
#include "stdlib.h"
#include "math.h"
#include "../simulator.h"
#include "../node.h"
#include "../macaddr.h" // for class MACAddr
#include "../interface.h" //for class Inteface
#include "../interface-wireless.h" //for class InterfaceWireless
#include "../rng.h"	// for Random::Value()
#include "lib/if.h" //for struct interface
#include "application-ospf6d.h"
#include "ospf6_proto.h"  //for ALLSPFROUTER6 and ALLDROUTERS6
#include "prefix.h"  //for struct prefix
#include "ospf6_route.h"  //for route_lookup
#include "ospf6_area.h"
#include "ospf6_interface.h"
#include "ospf6_neighbor.h"
#include "etrace.h"

#define TIMER_SECOND_MICRO 1000000L
#define TIMER_SECOND_MICRO_FLOAT 1000000.0
#define IPV4_LINKLOCAL 0x80000000

extern struct thread_master *master;
extern Node *sim_node;
extern struct List *iflist;

Uniform* simrng = NULL;

int gettimeofday_sim(struct timeval *tv, struct timezone *tz)

{
  double simtime = (double) Simulator::Now();
  tv->tv_sec = (time_t) floor(simtime);
  simtime -= tv->tv_sec;
  tv->tv_usec = (time_t) floor(simtime * TIMER_SECOND_MICRO);
  return 0;
}

void localtime_sim(struct tm *retTime)
{
  double simtime = (double) Simulator::Now();
  time_t clock;
  struct tm *tmp;
  time (&clock);
  tmp = localtime (&clock);

  memset(retTime, 0, sizeof(retTime));

  retTime->tm_hour = (int)floor(simtime/(60*60));
  retTime->tm_min = (int)floor(simtime/(60)) - 60*retTime->tm_hour;
  retTime->tm_sec = (int)floor(simtime) - 60*60*retTime->tm_hour - 
                                          60*retTime->tm_min;
  retTime->tm_mday  = tmp->tm_mday;
  retTime->tm_mon   = tmp->tm_mon;
  retTime->tm_year  = tmp->tm_year;
  retTime->tm_wday  = tmp->tm_wday;
  retTime->tm_yday  = tmp->tm_yday;
  retTime->tm_isdst = tmp->tm_isdst;
}

int sendmsg_wrapper(struct thread *thd)
{
  int stat=0;
  struct ospf6_msg *msg = (struct ospf6_msg *) THREAD_ARG(thd);
  stat = msg->the_instance->SendMessage(msg->sockId,msg->msgHdr,msg->flags);

  free(msg->msgHdr->msg_name);
  free(msg->msgHdr->msg_iov[0].iov_base);
  free(msg->msgHdr->msg_iov);
  free(msg->msgHdr->msg_control);
	
  free(msg->msgHdr); //XXX Insure
  free(msg);
  return stat;
}

#ifdef OSPF6_JITTER
int sendmsg_sim(int s, const struct msghdr *msg, int flags, int msec_jitter) 
{
#else
int sendmsg_sim(int s, const struct msghdr *msg, int flags) 
{
	int msec_jitter = 100;
#endif //JITTER
	/*
	 * This is to allow delay in sending all ospf6d messages
	 * this is a very nasty copy.
	 */
  OSPF6DInstance* ospf6_inst = (OSPF6DInstance *) master->ospf6_inst;
  struct ospf6_msg *mesg=(struct ospf6_msg *) malloc(sizeof(struct ospf6_msg));
  mesg->msgHdr = (struct msghdr *) malloc(sizeof(struct msghdr));
  
  mesg->the_instance = ospf6_inst;
  mesg->sockId = s;
  mesg->msgHdr->msg_name = (void *) malloc(msg->msg_namelen);
  mesg->msgHdr->msg_namelen = msg->msg_namelen;
  memcpy(mesg->msgHdr->msg_name,msg->msg_name,msg->msg_namelen);

  mesg->msgHdr->msg_iov = (struct iovec *) malloc (sizeof(struct iovec) * 2);
  mesg->msgHdr->msg_iov[0].iov_base = (void *) malloc(msg->msg_iov[0].iov_len);
  memcpy(mesg->msgHdr->msg_iov[0].iov_base, msg->msg_iov[0].iov_base,msg->msg_iov[0].iov_len);
  mesg->msgHdr->msg_iov[0].iov_len = msg->msg_iov[0].iov_len;
  mesg->msgHdr->msg_iov[1].iov_base = NULL;
  mesg->msgHdr->msg_iov[1].iov_len = 0;
  mesg->msgHdr->msg_iovlen = msg->msg_iovlen;

  mesg->msgHdr->msg_control = (void *) malloc (msg->msg_controllen);
  mesg->msgHdr->msg_controllen = msg->msg_controllen;
  memcpy(mesg->msgHdr->msg_control,msg->msg_control,msg->msg_controllen);

  mesg->msgHdr->msg_flags = msg->msg_flags;

  mesg->flags = flags;

  //this delays the ospf6d message between 0 and msec_jitter
  long r = (long) floor(((double) rand_sim())/RAND_MAX*msec_jitter);
  mesg->thd = thread_add_timer_msec(master,sendmsg_wrapper,mesg,r);
  
// UGLY.  Since writes are async, we do not know if it was
// successful or not!  "Faking" success may or may not be the
// right thing.
  return (mesg->msgHdr->msg_iov[0].iov_len);

 // return ospf6_inst->SendMessage(s, msg, flags);

}

/*
 *  Address type         Binary prefix        IPv6 notation   
 *  ------------         -------------        -------------   
 *  Unspecified          00...0  (128 bits)   ::/128         
 *  Loopback             00...1  (128 bits)   ::1/128       
 *  Multicast            11111111             FF00::/8     
 *  Link-local unicast   1111111010           FE80::/10      
 *  Site-local unicast   1111111011           FEC0::/10      
 *
 *  IPv4-mapped IPv6     00...0(80 bits):FFFF:(32 bit IPv4 address)
 *  Global unicast       (everything else)
 *
 */

//
// - If IPv6 address is link-local or site-local, then  use the 
//   IPv4 address of the interface.
// - If IPv6 address is IPv4-mapped IPv6, then use the embedded IPv4 addr.

//  ipv6 and ipv4 are in network byte order
void
ipv6_to_ipv4(struct in6_addr *ipv6, IPAddr_t *ipv4)
{
  char buf[128];

  if (IN6_IS_ADDR_MULTICAST(ipv6)) {
    // Must be either ALLSPFROUTERS or ALLDROUTERS in OSPF
    struct in6_addr allspfrouters6;
    struct in6_addr alldrouters6;
    inet_pton (AF_INET6, ALLSPFROUTERS6, &allspfrouters6);
    inet_pton (AF_INET6, ALLDROUTERS6, &alldrouters6);
    if (IN6_ARE_ADDR_EQUAL(ipv6, &allspfrouters6))
      *ipv4 = OSPF_ALLSPFROUTERS;
    else if (IN6_ARE_ADDR_EQUAL(ipv6, &alldrouters6))
      *ipv4 = OSPF_ALLDROUTERS;
    else 
    {
      inet_ntop(AF_INET6, ipv6, buf, sizeof(buf));
      printf("Unknown ipv6 multicast address %s\n", buf);
      exit(0);
    }
  } else if (IN6_IS_ADDR_V4MAPPED(ipv6)) {
    // take last 4 bytes of ipv6 address
    memcpy((void *) ipv4, (void *) &(ipv6->s6_addr32[3]), sizeof(ipv4)); 
  } else if (IN6_IS_ADDR_LINKLOCAL(ipv6)) {
    memcpy((void *) ipv4, (void *) &(ipv6->s6_addr32[3]), sizeof(ipv4)); 
  } else if (IN6_IS_ADDR_SITELOCAL(ipv6)) {
    // These are deprecated and unlikely to show up
    printf("SiteLocal addr\n");
  } else if (IN6_IS_ADDR_UNSPECIFIED(ipv6))
    printf("Unspecified addr\n");
  else if (IN6_IS_ADDR_V4COMPAT(ipv6))
    printf("V4Compat addr\n");
  else if (IN6_IS_ADDR_LOOPBACK(ipv6))
    printf("Loopback addr\n");
  else {
      inet_ntop(AF_INET6, ipv6, buf, sizeof(buf));
      printf("unknown ipv6 address type %s\n", buf);
      exit(0);
  }
}

void
ipv4_to_ipv6(IPAddr_t *ipv4, struct in6_addr *ipv6)  
{
  char buf[32];

  if (IN_MULTICAST(*ipv4))
  {
    if (*ipv4 == OSPF_ALLSPFROUTERS)
      inet_pton (AF_INET6, ALLSPFROUTERS6, ipv6);
    else if (*ipv4 == OSPF_ALLDROUTERS)
      inet_pton (AF_INET6, ALLDROUTERS6, ipv6);
    else
    {
      inet_ntop(AF_INET, ipv4, buf, sizeof(buf));
      printf("Unknown ipv4 multicast address %s\n", buf);
      exit(0);            
    }
  }
  else if ((*ipv4 & IPV4_LINKLOCAL) == IPV4_LINKLOCAL)
  {  //this is an ipv4 linklocal address
    memset(ipv6, 0, sizeof(struct in6_addr));
    memcpy((void *) &(ipv6->s6_addr32[3]), (void *) ipv4, sizeof(ipv4));
    //translate back to link local address
    ipv6->in6_u.u6_addr8[0] = 0xfe;
    ipv6->in6_u.u6_addr8[1] = 0x80;
  }
  else
  { // Not a link local address, normal route lookup LookupRoute_ospf6sim ()
    memset(ipv6, 0, sizeof(struct in6_addr));
    memcpy((void *) &(ipv6->s6_addr32[3]), (void *) ipv4, sizeof(ipv4));
  }
}

void
if_config_sim (struct interface *ifp)
{
  Interface *i;
 
  i = sim_node->GetIfByName(ifp->name);

  if(i==NULL)
  {
    printf("Unknown interface %s\n", ifp->name);
    exit(0);
  }
  
  ifp->ifindex = i->GetIfIndex();
  SET_FLAG (ifp->status, ZEBRA_INTERFACE_ACTIVE);
  ifp->flags |= IFF_MULTICAST;
  ifp->flags |= IFF_UP;
  ifp->flags |= IFF_RUNNING;
  ifp->metric = 1;
  ifp->mtu = 1500;
  ifp->hw_type = 0;
//ifp->hw_addr = ;
  ifp->hw_addr_len = 6;
  ifp->desc = NULL;
  ifp->distribute_in = NULL;
  ifp->distribute_out = NULL;

  connected_init(ifp,i);
}

//
// Random::Value() returns (0,1).  rand() returns (0,RAND_MAX)
//
int rand_sim(void)
{
  if (!simrng)
    simrng = new Uniform(0,1);
  return ((int) (simrng->Value() * RAND_MAX));
}

void connected_init(struct interface *ifp, Interface *i)
{
  struct prefix p_linklocal, p_global, dest;

  memset (&p_linklocal, 0, sizeof (struct prefix));
  memset (&p_global, 0, sizeof (struct prefix));
  memset (&dest, 0, sizeof (struct prefix));// ospf does not need dest

  IPAddr_t ip = i->GetIPAddr();
  uint32_t ip_temp = ntohl(ip); 
  
  if ((ip & IPV4_LINKLOCAL) == IPV4_LINKLOCAL)
  {
    char buf[32];
    inet_ntop(AF_INET, &ip_temp, buf, sizeof(buf));
    printf("Interface address %s must not be greater than 128.0.0.0, range is reserved for ipv4 linklocal addresses in ospf6\n", buf);
    exit(0);
  }
  
  //global address  
  if (!i->GetL2Proto()->IsWireless()) //shouldn't have global for manet interf
  //if (strcmp(i->GetName(), "lo") != 0)
  {
    p_global.family = AF_INET6;
    p_global.prefixlen = 120;
    memcpy(&(p_global.u.prefix6.in6_u.u6_addr32[3]), &ip_temp, sizeof(ip_temp));
    connected_add_by_prefix(ifp, &p_global, &p_global);
  }

  //linklocal address
  if (strcmp(i->GetName(), "lo") != 0)
  {
    p_linklocal.family = AF_INET6;
    p_linklocal.prefixlen = 120;
    //RFC 2373 sec 2.5.8 -- link-local address format
    //define as link-local address
    p_linklocal.u.prefix6.s6_addr[0] = 0xfe;
    p_linklocal.u.prefix6.s6_addr[1] = 0x80;
    ip |= IPV4_LINKLOCAL;  //set flag to declare ipv4 linklocal address
    memcpy (&(p_linklocal.u.prefix6.in6_u.u6_addr32[3]), &ip, sizeof(ip));
    connected_add_by_prefix(ifp, &p_linklocal, &dest);
  }

/* The following was implemented per RFC2373. But we use IPv4 in the
 * last 32 bits instead for simplicity
  // prepare EUI-64 
  // RFC 2373 Appendix A
  // split 6byte mac address 
  // place fffe in the middle to make 64 bits for ipv6 address 
  
  MACAddr::MACAddr_t  mac_addr = (MACAddr::MACAddr_t) i->GetMACAddr();
  
  unsigned long upper, lower;
  upper = lower = 0;

  // mac_addr is 4 bytes should be 6 bytes
  // place 1st byte in lowest byte of upper
  upper  = mac_addr >> 24; 
  //shift up one byte to hold ff in 1st byte
  upper <<= 8;
  upper |= 0x000000ff;
  
  //place lowest 3 bytes of mac address in lower
  lower  = mac_addr;
  lower &= 0x00ffffff;
  //place fe in front of the 3bytes
  lower |= 0xfe000000;

  // change host to byte network order
  upper = htonl(upper);
  lower = htonl(lower);

  //place split mac address in bottom 64 bits of link-local address
  memcpy (& (p->u.prefix6.s6_addr[8]), &upper, sizeof(upper));
  memcpy (& (p->u.prefix6.s6_addr[12]), &lower, sizeof(lower));
  */

  //ip = htonl(ip);
}


int setsockopt_sim(int s, int  level,  int  optname,  const  void  *optval,
           int optlen)
{
  if (level == IPPROTO_IPV6)
  {
    bool b;
    struct ipv6_mreq *mreq6;
    Interface *i;
    struct in6_addr allspfrouters6;
    struct in6_addr alldrouters6;
    
    switch(optname)
    {
      case IPV6_JOIN_GROUP:
      case IPV6_LEAVE_GROUP:  //HACK until multicast available
        b = (optname == IPV6_JOIN_GROUP ? true : false);
        mreq6 = (struct ipv6_mreq *) optval; 
        i = sim_node->GetIfByIndex(mreq6->ipv6mr_interface);
        inet_pton (AF_INET6, ALLSPFROUTERS6, &allspfrouters6);
        inet_pton (AF_INET6, ALLDROUTERS6, &alldrouters6);
        if (IN6_ARE_ADDR_EQUAL(&mreq6->ipv6mr_multiaddr, &allspfrouters6))
          i->SetSpfGroup(b);
        else if (IN6_ARE_ADDR_EQUAL(&mreq6->ipv6mr_multiaddr, &alldrouters6))
          i->SetDrGroup(b);
        break;
        
      case IPV6_MULTICAST_LOOP:
        printf("IPV6_MULTICAST_LOOP\n");
        break;
        
#ifdef IPV6_RECVPKTINFO 
      case IPV6_RECVPKTINFO:
        printf("IPV6_RECVPKTINFO\n");
        break;
#else        
      case IPV6_PKTINFO:
        printf("IPV6_PKTINFO\n");
        break; 
#endif //IPV6_RECVPKTINFO    
      case IPV6_CHECKSUM:
        printf("IPV6_CHECKSUM\n");
        break;

      default:
        break;
    }
    return 0;
  } 
  else if (level == SOL_SOCKET)
  {
    switch(optname)
    {
      case SO_REUSEADDR:
        printf("SO_REUSEADDR\n");
        break;
      default:
        break;
    }
    return 0;
  }
  return -1;
}

int
setsockopt_ipv6_pktinfo_sim (int sock, int val)
{
  int ret;

#ifdef IPV6_RECVPKTINFO  /*2292bis-01*/
  ret = setsockopt_sim(sock, IPPROTO_IPV6, IPV6_RECVPKTINFO, &val, sizeof(val));
  if (ret < 0)
    zlog_warn ("can't setsockopt IPV6_RECVPKTINFO : %s", safe_strerror (errno));
#else /*RFC2292*/
  ret = setsockopt_sim(sock, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(val));
  if (ret < 0)
    zlog_warn ("can't setsockopt IPV6_PKTINFO : %s", safe_strerror (errno));
#endif /* INIA_IPV6 */
  return ret;
}

void LookupRoute_ospf6sim(IPAddr_t src, IPAddr_t dst,RoutingEntry* r )
{
  //RoutingEntry *r = new RoutingEntry();
  struct prefix prefix;
  struct ospf6_route *ospf6_route_entry;
  struct ospf6_nexthop *nexthop;

  //printf("NodeId in LookupRoute_ospf6sim %d\n",sim_node->Id());
  //Is this an ipv4 linklocal address
  if ((dst & IPV4_LINKLOCAL) == IPV4_LINKLOCAL)
  {
    r->nexthop = dst & (~IPV4_LINKLOCAL);
    r->interface = sim_node->LocalRoute(dst & (~IPV4_LINKLOCAL));
    return;
  }
	
  if (ospf6 == NULL)
  {  //ospf6 router is not enabled
    r->interface = NULL; // ipv4.cc DataRequest only checks this pointer.
    return;
  }

  prefix.family = AF_INET6;
  prefix.prefixlen = 120;
  
  uint32_t ip_temp = ntohl(dst);

  ipv4_to_ipv6((IPAddr_t *)&ip_temp, &prefix.u.prefix6);
  ospf6_route_entry = ospf6_route_lookup (&prefix, ospf6->route_table);
	
  //ospf6_route_table_show_sim (ospf6->route_table);
  //ospf6_show_neighbors_sim();

  if (ospf6_route_entry == NULL || ospf6_route_entry->nexthop == NULL)
  {
    r->interface = NULL; // ipv4.cc DataRequest only checks this pointer.
    return;
  }
    
  nexthop = &ospf6_route_entry->nexthop[0];  //XXX taking first entry
  IPAddr_t temp;
  ipv6_to_ipv4(&nexthop->address, &temp);
  
  r->nexthop = temp;
  r->nexthop &= (~IPV4_LINKLOCAL);  //convert linklocal to ipv4 understandable
  r->interface = sim_node->GetIfByIndex(nexthop->ifindex);

  return;
}

void
ospf6_route_table_show_sim (struct ospf6_route_table *table)
{
  int i;
  char destination[64], nexthop[64];
  char duration[16], ifname[IFNAMSIZ];
  struct timeval now, res;
  struct ospf6_route *route;

  printf("#####################################\n");
  printf("Node %d 's routing table:\n",sim_node->Id());
  for (route = ospf6_route_head (table); route; route = ospf6_route_next (route)) 
  {
  gettimeofday_sim (&now, (struct timezone *) NULL);

  timersub (&now, &route->changed, &res);
  timerstring (&res, duration, sizeof (duration));

  /* destination */
  if (route->type == OSPF6_DEST_TYPE_LINKSTATE)
    ospf6_linkstate_prefix2str (&route->prefix, destination,
                                sizeof (destination));
  else if (route->type == OSPF6_DEST_TYPE_ROUTER)
    inet_ntop (route->prefix.family, &route->prefix.u.prefix,
               destination, sizeof (destination));
  else
    prefix2str (&route->prefix, destination, sizeof (destination));

  /* nexthop */
  inet_ntop (AF_INET6, &route->nexthop[0].address, nexthop,
             sizeof (nexthop));
  if (! if_indextoname_sim (route->nexthop[0].ifindex, ifname))
    snprintf (ifname, sizeof (ifname), "%d", route->nexthop[0].ifindex);

  printf ("%c%1s %2s %-30s %-25s %6s %s\n",
           (ospf6_route_is_best (route) ? '*' : ' '),
           OSPF6_DEST_TYPE_SUBSTR (route->type),
           OSPF6_PATH_TYPE_SUBSTR (route->path.type),
           destination, nexthop, ifname, duration);

  for (i = 1; ospf6_nexthop_is_set (&route->nexthop[i]) &&
       i < OSPF6_MULTI_PATH_LIMIT; i++)
    {
      /* nexthop */
      inet_ntop (AF_INET6, &route->nexthop[i].address, nexthop,
                 sizeof (nexthop));
      if (! if_indextoname_sim (route->nexthop[i].ifindex, ifname))
        snprintf (ifname, sizeof (ifname), "%d", route->nexthop[i].ifindex);

      printf ("%c%1s %2s %-30s %-25s %6s %s\n",
               ' ', "", "", "", nexthop, ifname, "");
    }
    }
  printf("#####################################\n");
}

void
ospf6_show_neighbors_sim ()
{
  char router_id[16];
  char duration[16];
  struct timeval now, res;
  char nstate[16];
  char deadtime[16];
  long h, m, s;

  struct ospf6_neighbor *on;
  struct ospf6_interface *oi;
  struct ospf6_area *oa;
  plistnode i, j, k;

  printf("%-15s %3s %11s %6s/%-12s %11s %s[%s]\n",
           "Neighbor ID", "Pri", "DeadTime", "State", "IfState", "Duration",
           "I/F", "State");

  for (i = listhead (ospf6->area_list); i; nextnode (i))
    {
      oa = (struct ospf6_area *) getdata (i);
      for (j = listhead (oa->if_list); j; nextnode (j))
        {
          oi = (struct ospf6_interface *) getdata (j);
          for (k = listhead (oi->neighbor_list); k; nextnode (k))
            {
              on = (struct ospf6_neighbor *) getdata (k);

  /* Router-ID (Name) */
  inet_ntop (AF_INET, &on->router_id, router_id, sizeof (router_id));
  gettimeofday_sim (&now, NULL);

  /* Dead time */
  h = m = s = 0;
  if (on->inactivity_timer)
    {
      s = on->inactivity_timer->u.sands.tv_sec - now.tv_sec;
      h = s / 3600;
      s -= h * 3600;
      m = s / 60;
      s -= m * 60;
    }
  snprintf (deadtime, sizeof (deadtime), "%02ld:%02ld:%02ld", h, m, s);

  /* Neighbor State */
#ifdef OSPF6_CONFIG
        if(on->ospf6_if->type == OSPF6_IFTYPE_POINTOMULTIPOINT ||
           on->ospf6_if->type == OSPF6_IFTYPE_POINTOPOINT ||
           on->ospf6_if->type == OSPF6_IFTYPE_MANETRELIABLE)
#else
  if (if_is_pointopoint (on->ospf6_if->interface))
#endif //OSPF6_CONFIG
    snprintf (nstate, sizeof (nstate), "PointToPoint");
  else
    {
      if (on->router_id == on->drouter)
        snprintf (nstate, sizeof (nstate), "DR");
      else if (on->router_id == on->bdrouter)
        snprintf (nstate, sizeof (nstate), "BDR");
      else
        snprintf (nstate, sizeof (nstate), "DROther");
    }

  /* Duration */
  timersub (&now, &on->last_changed, &res);
  timerstring (&res, duration, sizeof (duration));

  printf ("%-15s %3d %11s %6s/%-12s %11s %s[%s]\n",
           router_id, on->priority, deadtime,
           ospf6_neighbor_state_str[on->state], nstate, duration,
           on->ospf6_if->interface->name,
           ospf6_interface_state_str[on->ospf6_if->state]);
            }
        }
    }
}


char *
if_indextoname_sim (unsigned int ifindex, char *name)
{
  plistnode node;
  struct interface *ifp;

  for (node = listhead (iflist); node; nextnode (node))
    {
      ifp = (struct interface *) getdata (node);
      if (ifp->ifindex == ifindex)
  {
    memcpy (name, ifp->name, IFNAMSIZ);
    return ifp->name;
  }
    }
  return NULL;
}

// tv contains simulation time that event (stored in t) should execute
void sim_add_timeout_event(Time_t time_, struct thread* thr)
{
  OSPF6DEvent* DaemonTimeout; // Event is allocated in ScheduleTimer()
                              // and deleted in Timeout()
  OSPF6DInstance* o6d = (OSPF6DInstance *) master->ospf6_inst;
  o6d->ScheduleTimerWithThread(OSPF6DEvent::TYPE_TIMER, DaemonTimeout, 
    time_, thr);
}

void sim_cancel_event(void *sim_ev)
{
    Simulator::instance->Cancel( (Event*) sim_ev);
    delete (Event *) sim_ev;
}

void TraceEvent_sim(int level, const char *fmt,...)
{
  char buf[ETRACE_MAXSTR];
  va_list al;
  va_start(al,fmt);
  vsnprintf((char *)&buf,ETRACE_MAXSTR,fmt,al);
  va_end(al);
#ifdef ETRACE
  ETrace::Instance()->TraceEvent(sim_node, buf, level); 
#endif //ETRACE
  return;
}

//STATS
bool collect_stats_sim(struct ospf6 *o)
{
	double simtime = (double) Simulator::Now();
	
	if (simtime >= o->start_stat_time)
		return true;
	return false;
}

