/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Point to Point Link class using RTIKIT
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __linkrti_h__
#define __linkrti_h__

// The LinkRTI objects are used for distributed simulations.  These
// links are used where a sending node is on a different process or
// processor than the receiving node.

#include "link-real.h"
#include "interface.h"
#include "l2proto802.3.h"

#ifdef HAVE_RTI
/* RTI/FM Related Includes */
extern "C" {
#include "rticore.h"
#include "brti.h"
}
#endif

class LinkRTI : public LinkReal {
public:
  LinkRTI(IPAddr_t i, Mask_t m, Interface*);
  LinkRTI(IPAddr_t i, Mask_t m, Interface*, Rate_t b, Time_t d);
  LinkRTI(const LinkRTI&);                 // Copy constructor
  ~LinkRTI() { };
  // Methods from Link
  Link*      Copy() const ;                // Make a copy of this link
  void       SetPeer(Interface* i) { pPeer = i;}
  Interface* GetPeer(Packet*) { return pPeer;}
  Interface* GetPeer(Count_t) { return pPeer;}
  bool       Transmit(Packet*, Interface*, Node*);     // Transmit a packet
  bool       Transmit(Packet*, Interface*, Node*, Rate_t);
  MACAddr    IPToMac(IPAddr_t);            // Convert peer IP to peer MAC
  Count_t    NeighborCount(Node*);         // Number of routing neighbors
  // Build neighbor list 
  void       Neighbors(NodeWeightVec_t&, Interface*, bool);
  void       AllNeighbors(NodeWeightVec_t&);          // Build neighbor list
  Count_t    PeerCount();                  // Return number of peers
  IPAddr_t   PeerIP(Count_t);              // Get peer IP Address (if known)
  IPAddr_t   NodePeerIP(Node*);            // Get peer IP Address (if known)
  bool       NodeIsPeer(Node*);            // Find out if node is a peer
  Count_t    NodePeerIndex(Node*);         // Get peer index for node
  IPAddr_t   DefaultPeer(Interface*);      // Get IP Addr of default peer
  bool       IsRTI();
  // LinkRTI only methods
  void       SetPeerIP(IPAddr_t i) { peerIP = i;}
private:
  void ConstructorHelper(Interface*);      // Creates the RTI Groups
public:
  IPAddr_t   ip;
  Mask_t     mask;
  IPAddr_t   peerIP;
  MACAddr    peerMAC;
  Interface* iFace;
  Interface* pPeer;                       // Peer interface (if using ghosts)
#ifdef HAVE_RTI
  RTI_ObjClassDesignator    objClass;     // Class for messages from peers
  RTI_ObjInstanceDesignator objInstance;  // Instance of above
#define MAX_MSG_LENGTH 5000
  static char*  WhereMessage(long, void*, long);
  static char   rxbuf[MAX_MSG_LENGTH];
  static char   txbuf[MAX_MSG_LENGTH];
  static Time_t minTxTime;
#endif
  static Count_t nUpdates;  // Debug, count number of update messages
  static Count_t nReflects; // Debug, count number of reflect messages
  static Count_t ignoredReflects; // Debug, count number of ignored reflect
};

#endif


