/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Link base class
// George F. Riley.  Georgia Tech, Spring 2002

#include "link.h"
#include "linkp2p.h"
#include "ratetimeparse.h"

// Static members
Rate_t   Link::defaultRate   = Rate("1Mb"); // Default link speed
Time_t   Link::defaultDelay  = Time("1ms"); // Default delay
Link*    Link::defaultLink;
Random*  Link::defaultJitter;
Random*  Link::berRng;
bool     Link::useSeqEvents = true;     // True if bypass central ev. list

// Constructors
Link::Link()
{
}

Link::Link(Rate_t, Time_t)
{
}

// Static Methods
void     Link::DefaultRate(Rate_t r)
{ 
  defaultRate = r;
  if (!defaultLink) defaultLink = new Linkp2p(defaultRate, defaultDelay);
  defaultLink->Bandwidth(r); // And change in the default link
  Linkp2p::Default().Bandwidth(r); // and for default p2p
}

void     Link::DefaultDelay(Time_t t)
{
  defaultDelay = t;
  if (!defaultLink) defaultLink = new Linkp2p(defaultRate, defaultDelay);
  defaultLink->Delay(t);
  Linkp2p::Default().Delay(t); // Change for default p2p
}

void     Link::DefaultJitter(const Random& j)
{
  defaultJitter = j.Copy();
  defaultLink->Jitter(j);
  Linkp2p::Default().Jitter(j); // Change for default p2p
}

Random*  Link::DefaultJitter()
{
  if (!defaultJitter) defaultJitter = new Constant(0);
  return defaultJitter;
}

void     Link::Default(const Link& l)
{
  if (defaultLink) delete defaultLink;
  defaultLink = l.Copy();
}

const Link& Link::Default()
{
  if (!defaultLink) defaultLink = new Linkp2p(defaultRate, defaultDelay);
  return *defaultLink;
}

void     Link::SeedBER(Seed_t s)
{ // seed the BER rng
  if (!berRng) berRng = new Uniform();
  berRng->Seed(s);
}
