/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Netowrk Simulator
// -- George F. Riley.
/*
  This is the file for the 802.2 llc/snap frame
  A low overhead (4 bytes less than ethernet encapsulation)
  We get to pass the 'ethertype' into the packet.
  For now SNAP is enough but maybe we need LLC later
*/

#ifndef __llcsnap_h__
#define __llcsnap_h__

#include "l2proto.h"
#include "tfstream.h"
#include "macaddr.h"

class LLCSNAPHeader : public PDU {
  public:
   LLCSNAPHeader(Word_t type = 0x800);
   LLCSNAPHeader(char*, Size_t&, Packet*);
   
   //Size_t Size() const { return 8; }
   Size_t Size() const { return txSize; }
   PDU*   Copy() {return new LLCSNAPHeader(*this); }
#ifdef ETRACE
   Layer_t Layer() { return 1;}         // The 802.11 will be 2
#else
   Layer_t Layer() { return 2;}         // Indicate layer 2 pdu (kind of)
#endif // ETRACE
   Count_t Version() { return 1; }
   Size_t SSize(){ return 8;};
   char*  Serialize(char*, Size_t&);
   char*  Construct(char*, Size_t&);
   void   Trace(Tfstream& tos, Bitmap_t b,
                Packet* = nil, const char* = nil) { }
  public:
   char   llc_dsap;
   char   llc_ssap;
   char   llc_ctrl;
   Word_t snap_oui;  /* org. unique id  usually 3 chars (accounted in Size) */
   Word_t snap_ethtype; /* right now this is what matters to me */
public:
  static Size_t txSize;
  static void TxSize(Size_t txs) { txSize = txs; }
  
};


#endif
