/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator -  MAC Address Class
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __macaddr_h__
#define __macaddr_h__

// Defines a class for managing 48 bit MAC addresses

#include <string>

#include "common-defs.h"

//Doc:ClassXRef
class MACAddr {
public:
  typedef unsigned long MACAddr_t;
public:
  MACAddr();
  MACAddr(MACAddr_t u, MACAddr_t l);
  MACAddr(const MACAddr& c);
  // Typecasts
  operator std::string() const;            // Typecast to a string
  operator MACAddr_t() const;              // Lower bits only
  // Assignment
  MACAddr& operator=(const MACAddr& s);
  // Define some comparison operators
  bool   operator==(const MACAddr& r) const;
  // Define some arithmetic operators
  // Addition advances lower only, ignores overflow
  MACAddr operator++();
  MACAddr operator++(int);
  // Member access
  MACAddr_t Upper() const;
  MACAddr_t Lower() const;
  void      Upper(MACAddr_t);
  void      Lower(MACAddr_t);
  void      IncUpper();    // Increment upper
  bool      IsBroadcast(); // True if address is broadcast address
  void      SetBroadcast();// Set to broadcast address
private:
#ifndef COMPACT
  MACAddr_t upper;
#endif
  MACAddr_t lower;
private:
  static char work[20];       // Enough for 32 bit hex and slight fudge
public:
  static MACAddr Allocate();  // Allocate a new MAC Range
  static MACAddr   next;      // Next available value
  static MACAddr   NONE;      // Define a "NONE" for comparision
};

// Non-Member Comparison operators

bool operator< (const MACAddr& l, const MACAddr& r);
bool operator<=(const MACAddr& l, const MACAddr& r); 
bool operator> (const MACAddr& l, const MACAddr& r);
bool operator>=(const MACAddr& l, const MACAddr& r);

// Non-Member Arithmetic operators
MACAddr operator-(const MACAddr& l, const MACAddr& r);
MACAddr operator+(const MACAddr& l, const Count_t  r);

#endif
