/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Manage IP Address Masks
// George F. Riley.  Georgia Tech, Spring 2002

#include "mask.h"

Mask_t RoutingMasks[Mask::MAX_MASK + 1] = {
  0x00000000, // Zero length mask not used
  0x80000000,
  0xc0000000,
  0xe0000000,
  0xf0000000,
  0xf8000000,
  0xfc000000,
  0xfe000000,
  0xff000000,
  0xff800000,
  0xffc00000,
  0xffe00000,
  0xfff00000,
  0xfff80000,
  0xfffc0000,
  0xfffe0000,
  0xffff0000,
  0xffff8000,
  0xffffc000,
  0xffffe000,
  0xfffff000,
  0xfffff800,
  0xfffffc00,
  0xfffffe00,
  0xffffff00,
  0xffffff80,
  0xffffffc0,
  0xffffffe0,
  0xfffffff0,
  0xfffffff8,
  0xfffffffc,
  0xfffffffe,
  0xffffffff};

// Constructor
Mask::Mask(Mask_t m) : mask(RoutingMasks[24]), nBits(24) // Default to 24 bit
{
  if (m <= MAX_MASK)
    {
      mask = RoutingMasks[m];  // Bit count specified
      nBits = m;
    }
  if (m >= RoutingMasks[1])
    {
      mask = m;         // Valid mask specified
      // Need to find bit count
      for (nBits = 1; nBits < MAX_MASK; ++nBits)
        {
          if (m == RoutingMasks[nBits]) break;
        }
    }
}

