/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Base class for re-using malloc'ed memory
// George F. Riley.  Georgia Tech, Winter 2002

#ifndef __memreuse_h__
#define __memreuse_h__

// This base class overrides the basic new and delete operators.
// The revised new operator first checks for a previously deleted
// object of the correct size, and returns it if one exists.  If no
// such object exists, then malloc is used normally.  When object
// are deleted, rather than using "free", they are placed in a stack
// of objects for later reuse by new.  The stacks are sorted by size,
// so each stack only contains object of a common size.

// For the DEBUG compile, this code is disabled
#include <vector>

#include "common-defs.h"

#define nil         0

typedef std::vector<void*>     PtrVec_t;
typedef std::vector<PtrVec_t*> PtrPtrVec_t;


// Base class for all objects needing malloc reuse
class ReuseBase {
public:
  static void* operator new(std::size_t);
  static void  operator delete(void*);
  static void DumpQ();
public:
  static PtrPtrVec_t q;          // A queue of deleted heap elements
  static unsigned long reused;
  static unsigned long newalloc;
  static unsigned long deleted;
};

#endif


