/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Random waypoint mobility models
// George F. Riley.  Georgia Tech, Spring 2002

// Define the random waypoint mobility model

#ifndef __mobililty_random_waypoint_h__
#define __mobililty_random_waypoint_h__

#include "common-defs.h"
#include "mobility.h"
#ifdef REPEATABLE_MOBILITY
#include "timer.h"
#endif

class Node;
class WirelessGrid;
class Random;

//Doc:ClassXRef
#ifdef REPEATABLE_MOBILITY
class RandomWaypoint : public Mobility, public TimerHandler {
#else
class RandomWaypoint : public Mobility {
#endif
public:
  RandomWaypoint(WirelessGrid&, const Random&, const Random&);
  RandomWaypoint(const RandomWaypoint&);
public:
  void UpdateLocation(Node*);  // Compute new location
  Mobility* Copy() const;      // Make a copy
  void Initialize();           // Initialize the mobility
  Angle_t Direction() const;   // Angle of motion
protected:
  Time_t        nextTime;    // Time to stop, or time to start
  WirelessGrid& grid;        // Associated Grid
  // Random number generators
  Random*  pauseRng;         // Time to pause
  Random*  velocityRng;      // Random velocity
#ifdef REPEATABLE_MOBILITY
  Timer timer;
  TimerEvent* timeoutevent; 
  void Timeout(TimerEvent*); // Timer handle method
  void ScheduleNextTimeout(Time_t);
#endif
};

#endif


