/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Specific waypoint mobility models
// George F. Riley.  Georgia Tech, Spring 2002

// Define the specific waypoint mobility model

#ifndef __mobililty_specific_waypoint_h__
#define __mobililty_specific_waypoint_h__

#include "common-defs.h"
#include "mobility.h"
#include "node.h"

class Node;
class WirelessGrid;
class Random;

//Doc:ClassXRef
class SpecificWaypoint : public Mobility {
public:
  SpecificWaypoint();
  SpecificWaypoint(const SpecificWaypoint&);
  virtual ~SpecificWaypoint();
public:
  void UpdateLocation(Node*);  // Compute new location
  Mobility* Copy() const;      // Make a copy
  void Initialize();           // Initialize the mobility
public:
  void AddWaypoint(Node*, const Waypoint&);// Add a new waypoint
  void SetLooping(bool l) { loop = l; }
  
private:
  void MoveToWaypoint(Node*, const Waypoint&); // Set moving parameters
protected:
  Time_t          lastTime;    // Time of last update
  Time_t          cycleStart;  // Time the waypoint cycle started
  bool            loop;        // True if waypoint looping
  WaypointIt_t    nextIt;      // Next waypoint iterator
  WaypointList_t  waypoints;   // Waypoint list
};

#endif


