/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Node class
// George F. Riley.  Georgia Tech, Spring 2002

#include <iostream>
#include <unistd.h>
#ifdef HAVE_QT
#include <qcolor.h>
#endif

//#define DEBUG_MASK 0x02
#include "debug.h"
#include "common-defs.h"
#include "node-ghost.h"
#include "interface-ghost.h"
#include "duplexlink.h"

using namespace std;

// Static members
Count_t        NodeGhost::count = 0;
RadioVec_t     NodeGhost::emptyIf; // EmptyList
CallBackList_t NodeGhost::emptyCB; // EmptyList

NodeGhost::NodeGhost(Node* n)
    : NodeImpl(n)
{
  count++; // debug
}

NodeGhost::~NodeGhost()
{
}

bool NodeGhost::IsReal()
{ // True if Real node.  Always false for Ghosts
  return false;
}


void NodeGhost::Broadcast(Packet* p, Proto_t proto)
{ // Broadcast a packet on all interfaces
  delete p; // Should never be called for ghosts
}

// Queue management

Queue* NodeGhost::GetQueue()
{
  return nil;
}

Queue* NodeGhost::GetQueue(Node* n)
{
  return nil;
}

// Applications
Application* NodeGhost::AddApplication(const Application& a)
{
  return nil;
}

// Routing Interface
void NodeGhost::DefaultRoute(RoutingEntry)
{ // Specify default route
}

void NodeGhost::DefaultRoute(Node*)
{ // Specify default route
}

void NodeGhost::AddRoute(IPAddr_t, Count_t, Interface*, IPAddr_t)
{
}

RoutingEntry NodeGhost::LookupRoute(IPAddr_t)
{ // Find a routing entry
  // Still not found, see if on directly connected interface
  RoutingEntry r;
  return r;
}

RoutingEntry NodeGhost::LookupRouteNix(Count_t)
{ // Lookup route by Neighbor Index
  RoutingEntry r;
  return r;
}

AODVRoutingEntry* NodeGhost::LookupRouteAODV(IPAddr_t)
{
  return nil;
}

void NodeGhost::SetRoutingAODV(void*)
{
}

RoutingAODV* NodeGhost::GetRoutingAODV()
{
  return nil;
}

Routing::RType_t NodeGhost::RoutingType()
{
  return Routing::STATIC;
}

void NodeGhost::InitializeRoutes()
{ // Routing initialization (if any)
}

Count_t NodeGhost::RoutingFibSize() const
{ // Get the size of the routing FIB for statistics measurement
  return 0;
}

RoutingNixVector* NodeGhost::GetNixRouting()
{ // Returns a pointer to the routing object if it's NixVector routing,
  // otherwise return nil
  return nil; // Not NixVector
}

// Protocol Graph Interface
#ifdef COMPACT
// No private protocol graphs in compact mode
Protocol* NodeGhost::LookupProto(Layer_t l, Proto_t p) // Lookup protocol by layer
{
  return ProtocolGraph::common->Lookup(l,p);
}

void NodeGhost::InsertProto(Layer_t, Proto_t, Protocol*) // Insert a protocol
{
  cout << "Private protocol graph not supported in compact mode" << endl;
  return;
}
#else
Protocol* NodeGhost::LookupProto(Layer_t, Proto_t)
{ // Lookup protocol by layer
  return nil;
}

void NodeGhost::InsertProto(Layer_t, Proto_t, Protocol*)
{ // Insert proto
}
#endif

// Port Demux Interface
bool NodeGhost::Bind(Proto_t, PortId_t, Protocol*)
{
  return false;
}

bool NodeGhost::Bind(Proto_t, PortId_t, IPAddr_t, PortId_t, IPAddr_t, Protocol*)
{ // Register port usage
  return false;
}

PortId_t NodeGhost::Bind(Proto_t, Protocol*)
{ // Choose available port and register
  return NO_PORT;
}

bool NodeGhost::Unbind(Proto_t, PortId_t, Protocol*)
{ // Remove port binding
  return false;
}

bool NodeGhost::Unbind(Proto_t proto, PortId_t, IPAddr_t, PortId_t, IPAddr_t,
                       Protocol*)
{ // Remove port binding
  return false;
}

Protocol* NodeGhost::LookupByPort(Proto_t, PortId_t)
{
  return nil;
}

Protocol* NodeGhost::LookupByPort(Proto_t proto, PortId_t, IPAddr_t, PortId_t,
                                  IPAddr_t)
{
  return nil;
}

// Packet tracing interface
// Trace header if enabled
bool NodeGhost::TracePDU(Protocol*, PDU*, Packet*, char*)
{
  return false;
}

void NodeGhost::SetTrace(Trace::TraceStatus)    // Set trace level this node
{
}

// Node Location Interface
void NodeGhost::SetLocation(Meters_t, Meters_t, Meters_t)
{ // Set x/y/z location
}

void NodeGhost::SetLocation(const Location& l)
{
}

void NodeGhost::SetLocationLongLat(const Location& l)
{
}

bool NodeGhost::HasLocation()
{
  return false;
}

Meters_t NodeGhost::LocationX()
{
  return 0;
}

Meters_t NodeGhost::LocationY()
{
  return 0;
}

Meters_t NodeGhost::LocationZ()
{
  return 0;
}

Location NodeGhost::GetLocation()
{
  return Location(0, 0);
}

Location NodeGhost::UpdateLocation()
{
  return GetLocation();
}

Mobility* NodeGhost::AddMobility(const Mobility&)
{ // Add a new mobility model for this node
  return nil;
}

Mobility* NodeGhost::GetMobility() const
{ // Get the mobility model for this node
  return nil;
}

bool NodeGhost::IsMoving()
{
  return false;
}

bool NodeGhost::IsMobile()
{
  return false;
}

QCanvasItem* NodeGhost::Display(QTWindow*)
{
  return nil;
}

QCanvasItem* NodeGhost::Display(const QPoint&, QTWindow*)
{
  return nil;
}

void NodeGhost::WirelessTxColor(const QColor&)
{
}

#ifndef HAVE_QT
QColor dum;
#endif

const QColor& NodeGhost::WirelessTxColor()
{
#ifdef HAVE_QT
  return Qt::black;
#else
  return dum;
#endif
}

bool NodeGhost::PushWirelessTx(QCanvasItem*)
{
  return false;
}

QCanvasItem* NodeGhost::PopWirelessTx()
{
  return nil;
}

void NodeGhost::PixelSize(Count_t)  // Set node size
{
}

Count_t NodeGhost::PixelSizeX()
{
  return 0;
}

Count_t NodeGhost::PixelSizeY()
{
  return 0;
}

void NodeGhost::Shape(Node::Shape_t ns) // Set node shape
{
}

Node::Shape_t NodeGhost::Shape()
{
  return Node::NONE;
}

void NodeGhost::Color(const QColor&)
{
}

bool NodeGhost::HasColor()
{
  return false;
}

QColor NodeGhost::Color()
{
  return QColor();
}

bool NodeGhost::WirelessTx()
{
  return false;
}

bool NodeGhost::WirelessRx()
{
  return false;
}

bool NodeGhost::WirelessCx()
{
  return false;
}

bool NodeGhost::WirelessRxMe()
{
  return false;
}

bool NodeGhost::WirelessRxZz()
{
  return false;
}

NodeAnimation* NodeGhost::GetNodeAnimation() const
{
  return nil;
}

void NodeGhost::IsSwitchNode(bool)
{
}

void NodeGhost::PacketRX(Packet*, Interface*)
{
}
