/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Base class for Notifications
// George F. Riley.  Georgia Tech, Spring 2002

#ifndef __notifier_h__
#define __notifier_h__

#include <list>
#include <deque>

// Used by protocols to receive notification of queue space availability
// Used by applications to receive a notfication of a packet transmission
//Doc:ClassXRef
class NotifyHandler {
public:
  virtual void Notify(void*) = 0;
};

//Doc:ClassXRef
class Notification {
public:
  Notification(NotifyHandler* n, void* v) : handler(n), userData(v) { }
  Notification(const Notification& n) 
    : handler(n.handler), userData(n.userData) { }
public:
  NotifyHandler* handler;
  void*          userData;
};

// Typedefs for the notification list
//typedef std::list<Notification>       NList_t;
typedef std::deque<Notification>       NList_t;

#endif


