/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Boeing MANET OSPF6D port.  Copyright 2004, The Boeing Company

// Demux object for receiving L3 packets

// This follows the convention for how layer-4 protocols register
// with the IP layer to have packets demultiplexed by port.
// Here, we instruct IP to pass all received packets for protocol 89
// to this object, which then sends them off to the OSPF6DApplication

#include <iostream>

#include "debug.h"
#include "ospfdemux.h"
#include "node.h"
#include "protograph.h"
#include "droppdu.h"
#include "icmp.h"

using namespace std;

// Define the pointer for the single demux instance
OSPF6Demux* OSPF6Demux::instance = NULL;
OSPF6DApplication* OSPF6Demux::app_instance = NULL;

OSPF6Demux::OSPF6Demux() 
{
  if (instance == NULL)
    { // First instance
      instance = this;
      ProtocolGraph::CreateCommon();
      DEBUG0((cout << "Inserting ospfdemux, l4, proto 89" 
              << endl));
      ProtocolGraph::common->Insert(4, 89, this); // Register
    }
}

// L3Protocol handle is unused
void 
OSPF6Demux::DataIndication(Node* n, Packet* p, IPAddr_t src, 
                              L3Protocol& l3, Interface* i, IPAddr_t dst)
{
	app_instance->Receive(n,p,src,dst,i);
}

// Static methods
OSPF6Demux* OSPF6Demux::Instance()
{
  return instance;
}

void OSPF6Demux::Initialize()
{
  if (!instance) new OSPF6Demux();
}

void OSPF6Demux::Register(OSPF6DApplication *a)
{
	if (!app_instance) 
		app_instance = a;
}
