/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Port Demultiplexer
// George F. Riley.  Georgia Tech, Spring 2002

// Implements a map of protocol instances by port number

#ifndef __portdemux_h__
#define __portdemux_h__

#include "common-defs.h"
#include "protocol.h"

#include <map>

// Minimum port number for transient ports
#define MIN_TRANSIENT 10000

// Largest allowable port number
#define MAX_PORT      0xffff

typedef std::pair<IPAddr_t, PortId_t>  PortIp_t;      // IP Address and Port
typedef std::pair<PortIp_t, PortIp_t>  SrcDstPair_t;  // Local/Remote IP/Port
typedef std::map<SrcDstPair_t, Protocol*> PortDemux_t;
typedef std::map<Proto_t, PortDemux_t> ProtoDemux_t;  // One demux per proto
typedef std::map<Proto_t, PortId_t>    NextPort_t;    // Next available port

//Doc:ClassXRef
class PortDemux {
public:
  PortDemux();
  bool      Bind(Proto_t, PortId_t, Protocol*);   // Local port
  bool      Bind(Proto_t, PortId_t, IPAddr_t, Protocol*);// Local port, ip
  bool      Bind(Proto_t,
                 PortId_t, IPAddr_t,
                 PortId_t, IPAddr_t,
                 Protocol*);                      // Bind to local/remote
  PortId_t  Bind(Proto_t, Protocol*);             // Choose available port
  bool      Rebind(Proto_t,
                   PortId_t, IPAddr_t,
                   PortId_t, IPAddr_t,
                   Protocol*);                    // Rebind to local/remote
  Protocol* Lookup(Proto_t, PortId_t);
  Protocol* Lookup(Proto_t,
                   PortId_t, IPAddr_t,
                   PortId_t, IPAddr_t);
  bool      Unbind(Proto_t, PortId_t, Protocol*); // Release port usage
  bool      Unbind(Proto_t,
                   PortId_t, IPAddr_t,
                   PortId_t, IPAddr_t,
                   Protocol*); // Release port usage
  void      DBDump(PortDemux_t&); // debug
private:
  PortDemux_t::iterator Locate(Proto_t,
                               PortId_t, IPAddr_t,
                               PortId_t, IPAddr_t); // Locate the binding

public:
  ProtoDemux_t protoDemux;
  NextPort_t   nextPort;
};

#endif
