// Georgia Tech Network Simulator

#ifndef __PROPAGATION_H
#define __PROPAGATION_H

#define PI	3.1415926535897

// Free space propagation
class Propagation
{
public:
  Propagation();

  virtual double Pr(double d);
#ifdef SIM
  virtual void SetPower(double,double) = 0;
#endif //SIM

  // static methods
  static void   DefaultLoss(double l) { defaultLoss = l; }
  static double DefaultLoss()         { return defaultLoss; }
  static void   DefaultGtx(double g)  { defaultGtx = g; }
  static double DefaultGtx()          { return defaultGtx; }
  static void   DefaultGrx(double g)  { defaultGrx = g; }
  static double DefaultGrx()          { return defaultGrx; }
  static void   DefaultPtx(double p)  { defaultPtx = p; }
  static double DefaultPtx()          { return defaultPtx; }
  static void   DefaultFreq(double f) { defaultFreq = f; }
  static double DefaultFreq()         { return defaultFreq; }

protected:
  double Loss;        // system loss
  double Lambda;      // wave length, c/f
  double Gtx;         // tx antenna gain, dB
  double Grx;         // rx antenna gain, dB
  double Ptx;         // tx power, mW
  double Freq;        // channel frequency, Hz
  double preFree;     // pre-calculated

#ifdef SIM
	double dSaved;      // cached range
	double requiredSingalSaved;// cached csThreshold
#endif //SIM

  // static members
  static double defaultLoss;
  static double defaultGtx;
  static double defaultGrx;
  static double defaultPtx;
  static double defaultFreq;
};

// Two-ray ground reflection
class PropaTwoRay : public Propagation
{
public:
  PropaTwoRay();

  double Pr(double d);
#ifdef SIM
  virtual void SetPower(double,double);
#endif //SIM

  // static methods
  static void   DefaultHtx(double h) { defaultHtx = h; }
  static double DefaultHtx()         { return defaultHtx; }
  static void   DefaultHrx(double h) { defaultHrx = h; }
  static double DefaultHrx()         { return defaultHrx; }


private:
  double Htx;         // tx antenna height
  double Hrx;         // rx antenna height
  double Crossover;   // crossover distance
  double preTworay;   // pre-calculated

  // static members
  static double defaultHtx;
  static double defaultHrx;
};

#endif
