/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - QT Graphics Window Interface
// George F. Riley.  Georgia Tech, Fall 2002

#include <iostream>
#include "math.h"

#ifndef HAVE_QT
#define COMPACT
#endif

//#define DEBUG_MASK 0x02
#include "debug.h"
#include "common-defs.h"
#include "simulator.h"
#include "node.h"
#include "event.h"
#include "link.h"
#include "linkp2p.h"
#include "node-real.h"
#include "interface-wireless.h"
#include "node-real.h"

#ifndef COMPACT
#include "moc_qtwindow.h"
#include <qvbox.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qpixmap.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qdesktopwidget.h>
#include <qtimer.h>
#include <qpainter.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#endif

using namespace std;

#ifdef NOT_USED
// Pixmaps for vcr controls
static const char * rewind_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"................",
".++..........++.",
".++........++++.",
".++......++++++.",
".++....++++++++.",
".++..++++++++++.",
".++++++++++++++.",
".++++++++++++++.",
".++++++++++++++.",
".++++++++++++++.",
".++..++++++++++.",
".++....++++++++.",
".++......++++++.",
".++........++++.",
".++.........+++.",
"................"};

static const char * next_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"................",
".++.....+.......",
".+++....++......",
".++++...+++.....",
".+++++..++++....",
".++++++.+++++...",
".+++++++++++++..",
".++++++++++++++.",
".++++++++++++++.",
".+++++++++++++..",
".++++++.+++++...",
".+++++..++++....",
".++++...+++.....",
".+++....++......",
".++.....+.......",
"................"};

#endif

#ifndef COMPACT
/* XPM */
#ifdef USE_LARGE_CAMERA
static const  char *filesave_xpm[] = {
/* width height num_colors chars_per_pixel */
"    32    32      151            2",
/* colors */
".. c None",
".# c #1f1f1f",
".a c #202020",
".b c #2f2720",
".c c #302020",
".d c #302820",
".e c #3f2720",
".f c #3f2820",
".g c #3f2f2f",
".h c #3f3730",
".i c #402820",
".j c #402f20",
".k c #403730",
".l c #4f2820",
".m c #4f302f",
".n c #4f403f",
".o c #4f484f",
".p c #50302f",
".q c #50403f",
".r c #5f2f20",
".s c #5f3020",
".t c #5f3830",
".u c #5f4f30",
".v c #5f4f50",
".w c #5f5760",
".x c #60372f",
".y c #60382f",
".z c #60482f",
".A c #604f40",
".B c #60504f",
".C c #60606f",
".D c #6f4030",
".E c #6f4730",
".F c #6f676f",
".G c #6f7790",
".H c #70402f",
".I c #704030",
".J c #704830",
".K c #704f3f",
".L c #705f50",
".M c #706860",
".N c #707780",
".O c #7f6850",
".P c #7f706f",
".Q c #804f2f",
".R c #80573f",
".S c #805f40",
".T c #806740",
".U c #807060",
".V c #807f80",
".W c #80808f",
".X c #8090b0",
".Y c #80a0d0",
".Z c #8f5730",
".0 c #8f5840",
".1 c #8f7f7f",
".2 c #8f8f90",
".3 c #905f40",
".4 c #906740",
".5 c #90684f",
".6 c #907f6f",
".7 c #908780",
".8 c #908f8f",
".9 c #9097a0",
"#. c #909fb0",
"## c #90a7cf",
"#a c #9f5f30",
"#b c #9f684f",
"#c c #9f7750",
"#d c #9f887f",
"#e c #9f8f80",
"#f c #9f9790",
"#g c #9f98a0",
"#h c #9fa0af",
"#i c #9fafd0",
"#j c #9fafdf",
"#k c #a06f4f",
"#l c #a0703f",
"#m c #a07f5f",
"#n c #a0906f",
"#o c #a09780",
"#p c #a09890",
"#q c #a09f9f",
"#r c #a0b0c0",
"#s c #a0b7e0",
"#t c #af7850",
"#u c #af875f",
"#v c #af978f",
"#w c #af9fa0",
"#x c #afa09f",
"#y c #afa0a0",
"#z c #afa7af",
"#A c #afa8a0",
"#B c #afa8b0",
"#C c #afafcf",
"#D c #afb7d0",
"#E c #b07f50",
"#F c #b07f5f",
"#G c #b0a08f",
"#H c #b0a79f",
"#I c #b0a8af",
"#J c #b0afa0",
"#K c #b0b0b0",
"#L c #b0c7ef",
"#M c #bf875f",
"#N c #bfa89f",
"#O c #bfaf9f",
"#P c #bfb0a0",
"#Q c #bfb0af",
"#R c #bfb7b0",
"#S c #bfb8bf",
"#T c #c0b09f",
"#U c #c0b7af",
"#V c #c0b8af",
"#W c #c0b8bf",
"#X c #c0bfb0",
"#Y c #c0c0bf",
"#Z c #c0d0df",
"#0 c #cf9f6f",
"#1 c #cfc0b0",
"#2 c #cfc0c0",
"#3 c #cfd7f0",
"#4 c #d0bfa0",
"#5 c #d0c8c0",
"#6 c #d0cfc0",
"#7 c #d0cfd0",
"#8 c #d0d7a0",
"#9 c #dfcfbf",
"a. c #dfd7cf",
"a# c #dfd7df",
"aa c #dfd8df",
"ab c #dfe0bf",
"ac c #dfe7ff",
"ad c #e0d7af",
"ae c #e0d7d0",
"af c #e0d8df",
"ag c #e0e0e0",
"ah c #efe0e0",
"ai c #efe7e0",
"aj c #efe7ef",
"ak c #efe88f",
"al c #efe8f0",
"am c #eff0a0",
"an c #f0e0c0",
"ao c #f0e7e0",
"ap c #f0e8e0",
"aq c #f0f0ef",
"ar c #f0f0ff",
"as c #f0f8ff",
"at c #fff7ff",
"au c #fff8f0",
/* pixels */
"............................................#o#N#N#v#P..........",
"........................................#H#U#H#N#U#5ae#Y#H......",
"......................................#H#v#e#N#U#1aeaeahah#H....",
"....................................#H#v.1#d#O#Y#5aeahapaqaa#H..",
"..................................#H#N.1.M.7#Ya#aeagapatapap#5..",
"..................................#1#y.V.7#q#I#S#Ya#agajalalah#O",
"...................8#y.V.P.h.h...1a.#1#P#W#q.V.9#.#.#Kaaatauau#1",
".............8#y#q#y#I#R#Y#q.e.c#Papapaual#B.G###r#i.X#Calauau#1",
".........p.f.8#K#S#R#K#S#S#S#q.b#1auauauau.X#rakab#L.Y#iasauau#7",
".......p.l.j.n#d#K#R#R#S#K#K#S#p#Pauauauau#D#8amac#L.Y##auauaual",
".....p.j.e.p.i.q#d#A#I#W#W#Q#S#S#Pauauauau#D#Zacac#L#j#sauauauag",
"...m.p.m.f.e.i.l.S#Raaaaagalatal#Xauauatah#K#L#3ac#3#j#3auauau#6",
"...l.m.m.e.i.y#b.Laiauauauauauau#Pauauahafaf#7#r#r#i#Dauauauauae",
"...e.f.j.b.e#b.R.haaauauauauauau#Hahaoaf#5#6#7#K#h#Ba#ajauauag#H",
".D.D.l.b.a.f#M.h.#aaauauauauauau.P#5aeae#1#Y#6#w.2.9#g#z#7#7ae..",
".p.s.s.b.a.f#F.t.baiaualagahalag.E#dae#1#P#J#Y#y.F.C.N.W.2#f#H..",
".e.p.p.a.a.f#E.D.calau.F.#.u#n#2.K.f#4#X#x.8#y#A.F.o.w.C.7#H....",
".t.x.f.a.a.e#E.D.caqau.g.z.A.A#J.R.f.O#Y#U#f#y#P#N.M.v#d#x......",
"...i.c.b.a.f#E.D.calaq.Bad#G.d#I.E.#.q#o#N#X#P#P#P#v#A#x........",
"...e.a.a.b.f#t.x.capap.M.L.k.oae.D.#.K.R.f.B#Y#1#1#V............",
"...e.b.b.b.f#t.t.calau#P.n.n#Qau.y.#.R.R.#.c....................",
"...i.e.c.b.f#t.x.haqauau#Y#Yaqau.p.#.R.R.c.h....................",
"...l.c.e.e.j#t.s.tauau#6.q.q#Nau.h.c.T.0.c.j....................",
"...c.e.i.i.j#F.x.Dauau.U.S.1.q#5.D.c.5.R.c.j....................",
"...e.i.l.l.y#b.x.taq#5.A#4an.1#9.H.c#c.Z.c.p....................",
"...e.l.l.l.3.0.r.5auae.U#N#O#e#9.D.c#m.4.r.I....................",
"...c.r.x.r.0.J#t#0arau#9.6#d#9au.J.c.R.3.s.I....................",
"...e.x.y.y.3.3.Z#uauauauauauauah.Q.l.D.0.x......................",
".....l.H.I.Z#k.y#mauauauau#9#v.4.J.r............................",
".......y.I.J#k.x.3#T#q#m.3.Q.Q.y.s..............................",
".........y.Z#t#l#a.Z.J.y.H.x....................................",
"...........y#l.I.x.x.x.........................................."
};
#else
static const char *filesave_xpm[] = {
"    16    16        4            1",
". c #040404",
"# c #808304",
"a c #bfc2bf",
"  c None",
"                ",
"..............  ",
".#.aaaaaaaa.a.  ",
".#.aaaaaaaa...  ",
".#.aaaaaaaa.#.  ",
".#.aaaaaaaa.#.  ",
".#.aaaaaaaa.#.  ",
".#.aaaaaaaa.#.  ",
".##........##.  ",
".############.  ",
".##.........#.  ",
".##......aa.#.  ",
".##......aa.#.  ",
".##......aa.#.  ",
" .............  ",
"                ",
};
#endif

static const char *record_xpm[] = {
/* width height ncolors chars_per_pixel */
"32 32 16 1",
/* colors */
"  c #000000000000",
"! c #800000000000",
"# c #000080000000",
"$ c #800080000000",
"% c #000000008000",
"& c #800000008000",
"' c #000080008000",
"( c #800080008000",
") c #c000c000c000",
"* c #ff0000000000",
"+ c #0000ff000000",
", c #ff00ff000000",
"- c #00000000ff00",
". c #ff000000ff00",
"/ c #0000ff00ff00",
"0 c #ff00ff00ff00",
/* pixels */
"                                ",
" 0000000000000000000000000000000",
"  0         0       000000000000",
" 0 --------- ------- 000000 0000",
" 0 ---------          0000 , 000",
"  0          --------     ,, (00",
" 00 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - ---------------     ,, ((0",
"0000  -  ----- - - -- (((( , ((0",
"00000 --------------- ((((( (((0",
"000000               (((0000((00",
"0000000(((((((((((((((((00000000",
"00000000(((((((((((((((000000000",
"0000               0000000000000",
"0000 ,0,0,0,0,0,0, (000000000000",
"0000 0,0,0,0,0,0,0 ((00000000000",
"0000 ,0,0               00000000",
"0000 0, , ,0,0,0,0,0,0, (0000000",
"0000 , 0  0,0,0,0,0,0,0 ((000000",
"0000 0, , ,0 0,0,0,0 0, ((000000",
"0000 ,0,0 0 0 0   0 0 0 ((000000",
"0000 0,0, ,0 0,0,0,0 0, ((000000",
"0000      0,0,0,0,0,0,0 ((000000",
"00000(((( ,0,0,0,0,0,0, ((000000",
"000000((( 0,0,0,0,0,0,0 ((000000",
"000000000               ((000000",
"0000000000((((((((((((((((000000",
"00000000000(((((((((((((((000000"
} ;

static const char * play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};

static const char * stop_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
" ++++++++++++++ ",
" ++++++++++++++ ",
" ++++++++++++++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" +++        +++ ",
" ++++++++++++++ ",
" ++++++++++++++ ",
" ++++++++++++++ ",
"                "};

static const char * pause_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"                "};
#endif

#ifdef HAVE_QT
// Define a non-filled circle canvas item. Used for wireless animation.
class QCanvasCircle : public QCanvasEllipse
{
public:
  QCanvasCircle(int radius, QCanvas * canvas) :
      QCanvasEllipse(2 * radius, 2 * radius, canvas)
  {
  }
  QCanvasCircle(QCanvas * canvas) : QCanvasEllipse(canvas)
  {
  }

  void setDiameter(int diameter)
  {
    setSize(diameter, diameter);
  }

  void setRadius(int radius)
  {
    setSize(2 * radius, 2 * radius);
  }

protected:
  void draw(QPainter & p)
  {
    p.setPen(pen());
    drawShape(p);
  }

  void drawShape(QPainter & p)
  {
    int penWidth = pen().width();
    if (penWidth > 4)
      {
        p.drawArc((int)(x() - width()/2 + penWidth /2),
                  (int)(y() - height()/2 + penWidth/2),
                  width() - penWidth,
                  height() - penWidth , 0,16*360);
      }
    else
      {
        p.drawArc((int)(x() - width()/2), int(y() - height()/2),
                  width(), height(), 0, 16*360);
      }
  }
};

// Subclass the QCanvasView class to process mouse clicks
class QSelectableView : public QCanvasView 
{
public:
  QSelectableView(QCanvas*, QWidget* parent=0, const char* name=0, WFlags f=0);
protected:
  void contentsMousePressEvent(QMouseEvent*);
  void contentsMouseMoveEvent(QMouseEvent*);
};

QSelectableView::QSelectableView(
	QCanvas* c, QWidget* parent,
	const char* name, WFlags f) :
    QCanvasView(c,parent,name,f)
{
}

void QSelectableView::contentsMousePressEvent(QMouseEvent* e)
{
  if (!Simulator::pNodeSelected) return; // No callback, nothing to do
  QPoint p = inverseWorldMatrix().map(e->pos());
  QCanvasItemList l=canvas()->collisions(p);
  for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
    {
      QCanvasItem* ci = (*it);
      // Check each node to see of this is the one
      const NodeVec_t& nodes = Node::GetNodes();
      for (NodeVec_t::size_type i = 0; i < nodes.size(); ++i)
        {
          Node* n = nodes[i];
          NodeAnimation* anim = n->GetNodeAnimation();
          if  (anim)
            { // Animation info exists
              if (ci == anim->nodeSprite ||
                  ci == anim->nodePolygon)
                { // Found it
                  Simulator::pNodeSelected(n);
                  break; // No need to keep looking
                }
            }
        }
    }
}


void QSelectableView::contentsMouseMoveEvent(QMouseEvent* e)
{
  cout << "Got mouse move event" << endl;
  QPoint p = inverseWorldMatrix().map(e->pos());
  QCanvasItemList l=canvas()->collisions(p);
  for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
    {
      QCanvasItem* ci = (*it);
      // Check each node to see of this is the one
      const NodeVec_t& nodes = Node::GetNodes();
      for (NodeVec_t::size_type i = 0; i < nodes.size(); ++i)
        {
          Node* n = nodes[i];
          NodeAnimation* anim = n->GetNodeAnimation();
          if  (anim)
            { // Animation info exists
              if (ci == anim->nodeSprite ||
                  ci == anim->nodePolygon)
                { // Found it
                  cout << "Mouse over node " << n->Id() << endl;
                  break; // No need to keep looking
                }
            }
        }
    }
}



// Constructor
QTWindow::QTWindow()
#ifndef COMPACT
  : app(nil), canvas(nil), view(nil), 
    updateRate(nil), slider(nil), simTime(nil), readyTimer(nil),
    nodePixelSize(10), zoomMult(1.0), border(10),
    //canvasX(640), canvasY(640),
    canvasX(3640), canvasY(3640),
    qtEvent(nil), topologyDisplayed(false), updateScheduled(false),
    ready(true), paused(false), quit(false), oneShot(false),
    playbackMode(false), recording(false), recordingFrame(0)
#endif
{
#ifndef COMPACT
#ifdef HUH
  int argc = 1;
  char* argv[] = { "GTNetS", nil};
  app = new QApplication(argc, argv);
#endif
  // Reduce window size to 512x512 if animation recorder enabled
  if (Simulator::recorderEnabled)
    {
      canvasX = 512;
      canvasY = 512;
    }
  
  int argc = 1;
  char* argv = "GTNetS";
  app = new QApplication(argc, &argv);
  QDesktopWidget* qd = QApplication::desktop();
  cout << "canvX " << canvasX << " canvY " << canvasY
       << " qdw " << qd->width() << " qdh " << qd->height()
       << endl;
  //if (canvasX > (qd->width()-20))  canvasX = qd->width() - 20;
  //if (canvasY > (qd->height()-100)) canvasY = qd->height() - 100;
  if (canvasX > (qd->width()-40))  canvasX = qd->width() - 40;
  if (canvasY > (qd->height()-150)) canvasY = qd->height() - 150;
  if (canvasX > canvasY) canvasX = canvasY;
  if (canvasY > canvasX) canvasY = canvasX;
  cout << "canvX " << canvasX << " canvY " << canvasY << endl;
  // Set the mapping from slider values to display update rate
  // For playback mode, these are "fraction of real time"
  updateRates[0]  = 0.000001;
  updateRates[1]  = 0.000002;
  updateRates[2]  = 0.000004;
  updateRates[3]  = 0.000008;
  updateRates[4]  = 0.000016;
  updateRates[5]  = 0.000032;
  updateRates[6]  = 0.000064;
  updateRates[7]  = 0.000125;
  updateRates[8]  = 0.000250;
  updateRates[9]  = 0.000500;
  updateRates[10] = 0.001000;
  updateRates[11] = 0.002000;
  updateRates[12] = 0.004000;
  updateRates[12] = 0.008000;
  updateRates[13] = 0.016000;
  updateRates[14] = 0.032000;
  updateRates[15] = 0.064000;
  updateRates[16] = 0.125000;
  updateRates[17] = 0.250000;
  updateRates[18] = 0.500000;
  updateRates[19] = 1.000000;

  currentUpdateRate = updateRates[MAX_SLIDER-1];
#endif
}

// Destructor
QTWindow::~QTWindow()
{
#ifndef COMPACT
  QApplication::exit(); // Stop processing and close windows
  delete app;           // Delete the application object
#endif
}

// Event Handler
void QTWindow::Handle(Event* e, Time_t t)
{
#ifndef COMPACT
  QTEvent* ev = (QTEvent*)e;
  if (!topologyDisplayed) DisplayAllNodes();
  switch (ev->event) {
    case QTEvent::START :
      cout << "Got start event" << endl;
      Play();
      break;
    case QTEvent::STOP :
      cout << "Got stop event" << endl;
      Stop();
      break;
    case QTEvent::PAUSE :
      cout << "Got pause event" << endl;
      Pause();
      break;
    case QTEvent::RECORD:
    case QTEvent::UPDATE :
      // Record must do everything UPDATE does
      DEBUG0((cout << "Hello from QTEvent Update time " 
              << Simulator::Now() << endl));
      updateScheduled = false;
      // Update the simtime lcd display
      simTime->display(Simulator::Now());
      // Show the topology (if changed)
      DisplayAllNodes();
      ProcessEvents();
      if (ev->event == QTEvent::RECORD)
        {
          RecordNextFrame(ev);
          if (!recording) delete ev;
        }
      else
        { // UPDATE event
          if (paused) WaitWhilePaused();
          // Schedule another update event in the future
          // But check for non-nil, since the quit event may delete the event
          if (qtEvent && !updateScheduled && !quit)
            {
              Simulator::instance->Schedule(qtEvent, currentUpdateRate, this);
              updateScheduled = true;
            }
        }
#ifdef OLD_WAY
      if (quit)
        {
          delete app;
          app = nil;
        }
#endif
      break;
    case QTEvent::WIRELESS_TX_START :
    case QTEvent::WIRELESS_TX_END :
#ifdef VERBOSE
      cout << "Wireless TX ";
      if (ev->event == QTEvent::WIRELESS_TX_START) 
        cout << "Start ";
      else
        cout << "End ";
      cout << " iter " << ev->iter
           << " time " << Simulator::Now() << endl;
#endif
      if (ev->event == QTEvent::WIRELESS_TX_START) 
        WirelessTxStart(ev->src, ev->iFace, ev->dst, ev->range, ++(ev->iter));
      else
        WirelessTxEnd(ev->src, ev->iFace, ev->dst, ev->range, ++(ev->iter));
      delete ev; // New event allocated in private subroutines
      break;
    }
  // Don't delete update event since reused above
#endif
}

void QTWindow::DisplayTopology()
{
  DEBUG0((cout << "Display Topology" << endl));
  DEBUG0((cout << " SmallestX " << Simulator::instance->SmallestX()
       << " SmallestY " << Simulator::instance->SmallestY()
       << " LargestX " << Simulator::instance->LargestX()
       << " LargestY " << Simulator::instance->LargestY()
          << endl));
#ifndef COMPACT
  DisplayTopologyAndReturn();
  app->exec(); // Run until quit button pressed
#endif
}

void QTWindow::DisplayTopologyAndReturn()
{
  DEBUG0((cout << "Display TopologyAndReturn" << endl));
#ifndef COMPACT
  if (canvas) return;            // Already initialized
  Initialize();                  // Pre-compute some coordinates
  box = new QVBox();      // Get a vertical box to hold quit and canvas
  box->resize(canvasX+8, canvasY+32 ); // Size of canvas plus a fudge

  QHBox* hbox = new QHBox(box);
  hbox->show();

  // Allocate a quitButton button and connect to application quit
  QPushButton* quitButton = new QPushButton("Quit", hbox);
  quitButton->setFont(QFont("Times", 12, QFont::Bold));
  quitButton->show();
  connect(quitButton, SIGNAL(clicked()), SLOT(Quit()));

  // Allocate a exitButton button and connect to application quit
  QPushButton* exitButton = new QPushButton("Exit", hbox);
  exitButton->setFont(QFont("Times", 12, QFont::Bold));
  exitButton->show();
  connect(exitButton, SIGNAL(clicked()), SLOT(Exit()));

  // Put the update rate lcd and slider
  updateRate  = new QLCDNumber( 6, hbox, "UpdateRate" );
  updateRate->setMode(QLCDNumber::Dec); // Decimal mode
  updateRate->setNumDigits(7);
  updateRate->setSegmentStyle(QLCDNumber::Filled);
  
  updateRate->display(updateRates[MAX_SLIDER-1]);
  // Create the slider
  slider = new QSlider( QWidget::Horizontal, hbox);
  connect( slider, SIGNAL(valueChanged(int)), SLOT(NewSliderValue(int)));
  // Create the simtime display
  simTime = new QLCDNumber(6, hbox, "SimTime");
  simTime->setMode(QLCDNumber::Dec); // Decimal mode
  simTime->setNumDigits(7);
  simTime->setSegmentStyle(QLCDNumber::Filled);
  simTime->display(0.0);
  slider->setRange( 0, MAX_SLIDER-1 );
  slider->setValue( MAX_SLIDER-1 );

  // Put the record button if enabled
  if (Simulator::recorderEnabled)
    {
      QPushButton* record = new QPushButton(QPixmap(record_xpm), 0,  hbox);
      record->setToggleButton(true);
      connect(record, SIGNAL(toggled(bool)), SLOT(Record(bool)));
    }
  else
    { // Put the save button
      QPushButton* save = new QPushButton(QPixmap(filesave_xpm), 0,  hbox);
      connect(save, SIGNAL(clicked()), SLOT(Save()));
    }
  
  
  // Put the vcr controls
  QPushButton* stop = new QPushButton( QPixmap( stop_xpm ), 0, hbox);
  connect(stop, SIGNAL(clicked()), SLOT(Stop()));
  stop->show();
  QPushButton* play = new QPushButton( QPixmap( play_xpm ), 0, hbox);
  connect(play, SIGNAL(clicked()), SLOT(Play()));
  QPushButton* pause = new QPushButton( QPixmap( pause_xpm ), 0, hbox);
  connect(pause, SIGNAL(clicked()), SLOT(Pause()));
  play->show();

  box->show();

  // Allocate the canvas and canvas view
  canvas = new QCanvas(canvasX, canvasY);
  canvas->setDoubleBuffering(true);
  
  view   = new QSelectableView(canvas, box);
  view->resize(canvasX, canvasY);
  //view->setFixedWidth(canvasX); // Try this, not sure
  //view->setFixedHeight(canvasY);
  view->show();
  app->setMainWidget(box);

  if (Simulator::pCustomBackground) Simulator::pCustomBackground(canvas);
  DisplayAllNodes(); // Display the topology
#ifdef DEBUG_SAVE
  // Debug...get a pixmap and save
  QPixmap pm = QPixmap::grabWidget(box);
  pm.save("testanim.png", "PNG");
#endif

#endif
}

void QTWindow::ProcessEvents()
{
#ifndef COMPACT
  if (!app) return;
  while(app && app->hasPendingEvents() && !quit)
    {
      app->processEvents(1);
    }
  if (app) app->processEvents(1); // Once more to clear quit events
  if (quit)
    {
      app->quit();
      delete app;
      app = nil;
      if (qtEvent)
        { // Cancel any pending event
          Simulator::instance->Cancel(qtEvent);
          delete qtEvent;
          qtEvent = nil;
        }
    }
#endif
}

void QTWindow::AnimationUpdateRate(Time_t t)
{
#ifndef COMPACT
  if (!slider) DisplayTopologyAndReturn();
  
  // Find nearest slider value
  for (Count_t i = 0; i < MAX_SLIDER; ++i)
    {
      if (t < updateRates[i])
        { // Found it
          slider->setValue(i);
          currentUpdateRate = t;
          updateRate->display(currentUpdateRate);
          cout << "Initial update rate " << currentUpdateRate << endl;
          return;
        }
    }
  slider->setValue(MAX_SLIDER - 1);
#endif
}

#ifndef COMPACT
void QTWindow::UpdateTopology(bool b)
{ // Displays the topology on demand
  DEBUG0((cout << "GTWindow::UpdateTopology (forced)" << endl));
  // Next line insures pacing at appropriate rate.
  // The ready flag is set to true when the readyTimer pops.
  if (paused) WaitWhilePaused();
  while(!ready && !quit) ProcessEvents();
  if (quit) return;
  if (playbackMode)
    {
      ready = false;
      readyTimer->start(1000/25, true);  // Use 25 frames per sec
    }

  // Update the simtime lcd display
  simTime->display(Simulator::Now());
  DisplayAllNodes(b); // Display the topology
  ProcessEvents();   // And allow QT to update
}

Count_t lineCount; // debug

void QTWindow::AddBackgroundLines(const LocationVec_t& points)
{
  bool first = true;
  QPoint prior; // Prior point
  Location priorLoc; // debug
   
  if (!canvas) DisplayTopologyAndReturn();
  DEBUG0((cout << "QTW::ABL points.size " << points.size() << endl));
  for (LocationVec_t::size_type i = 0; i < points.size(); ++i)
    {
      DEBUG0((cout << "locX " << points[i].X() 
              << " y " << points[i].Y() << endl));
      QPoint current = LocationToPixels(points[i]);
      // debug
      if (current.x() < 0 || current.x() >= canvasX ||
          current.y() < 0 || current.y() >= canvasY)
        {
          cout << "HuH?  Conversion out of range, x " << current.x()
               << " y " << current.y()
               << " mx " << points[i].X()
               << " my " << points[i].Y() << endl;
        }
      
      bool inRange = (points[i].Y() <= 60.0 && points[i].Y() >= 20.0 &&
                      priorLoc.Y() <= 60.0 && priorLoc.Y() >= 20.0);
      
      inRange = true;
      if (!first && inRange)
        {
          QCanvasLine* line = new QCanvasLine(canvas);
          line->setPen(Qt::black);
          line->setPoints(prior.x(), prior.y(), current.x(), current.y());
          DEBUG0((cout << "Adding point x0 " << prior.x()
                  << " y0 " << prior.y()
                  << " x1 " << current.x()
                  << " y1 " << current.y() << endl));
          
          line->show();
          lineCount++;
        }
      first = false;
      prior = current;
      priorLoc = points[i];
    }
  canvas->update(); // Show the updates
  DEBUG0((cout << "Cumulative Line Count " << lineCount << endl));
}

void QTWindow::PlaybackMode(bool pb)
{
  if (pb)
    { // Playback mode requested, create a pacing timer for real-time display
      playbackMode = true;
      readyTimer = new QTimer(this);
      connect(readyTimer, SIGNAL(timeout()), SLOT(TimerDone()));
      readyTimer->start(1000/25, true);  // Use 25 frames per sec
    }
  else
    {
      if (readyTimer)
        {
          readyTimer->stop();
          delete readyTimer;
          readyTimer = nil;
        }
      playbackMode = true;
      ready = true;
    }
}

void QTWindow::PlaybackPause()
{
  paused = true;
}


// Slots
void QTWindow::NewSliderValue(int v)
{
  currentUpdateRate = updateRates[v];
  updateRate->display(currentUpdateRate);
  if (!paused && qtEvent)
    { // Need to cancel pending update event and reschedule
      Simulator::instance->Cancel(qtEvent);
      updateScheduled = true;
      Simulator::instance->Schedule(qtEvent, currentUpdateRate, this);
    }
}

void QTWindow::Save()
{
  // Get a PNG image of the view
  QPixmap pm = QPixmap::grabWidget(view);
  QString fn = QFileDialog::getSaveFileName(".", "Images(*.png)", view,
                                            "Save File Dialog", 
                                            "Choose a file");
  if (!fn.isEmpty())
    {
      // Make sure ends with .png
      if (!fn.endsWith(".png")) fn += ".png";
      if ( QFile::exists( fn ))
        {
          QMessageBox mb( "GTNetS",
                          "Saving the file will overwrite the original file on the disk.\n"
                          "Do you really want to save?",
                          QMessageBox::Information,
                          QMessageBox::Yes | QMessageBox::Default,
                          QMessageBox::No, 0);
          mb.setButtonText( QMessageBox::Yes, "Yes" );
          mb.setButtonText( QMessageBox::No, "No" );
          int r = mb.exec();
          if (r == QMessageBox::No) return;
        }
      
#ifdef NEED_NEWER_QT  
           QMessageBox::question(
               view,
               tr("Overwrite File?"),
               tr("File %1 already exists."
                  "Do you want to overwrite it?")
               .arg( fn ),
               tr("&Yes"), tr("&No"),
               QString::null, 0, 1 ) )
        {
          return;
        }
#endif
      pm.save(fn, "PNG");
    }
}

void QTWindow::Record(bool)
{
  if (recording)
    {
      cout << "Stop recording" << endl;
      recording = false;
      // No need to delete the event, as it will do nothing
      // when schedule and be deleted then.
    }
  else
    {
      cout << "Start recording" << endl;
      recordingFrame = 0;
      recording = true;
      // Create the event and record the first frame
      QTEvent* ev = new QTEvent(QTEvent::RECORD);
      RecordNextFrame(ev);
    }
}

void QTWindow::Play()
{
  cout << "Start animation" << endl;
  paused = false;
  // Schedule update event
  if (!qtEvent)
    {
      qtEvent = new QTEvent(QTEvent::UPDATE);
    }
  else
    {
      if (updateScheduled)
        {
          Simulator::instance->Cancel(qtEvent);
        }
    }
  Simulator::instance->Schedule(qtEvent, currentUpdateRate, this);
  updateScheduled = true;
}

void QTWindow::Stop()
{
  if (!paused)
    {
      cout << "Stop animation" << endl;
      paused = true;
      if (qtEvent)
        { // Should always be true, but just in case
          Simulator::instance->Cancel(qtEvent);
          delete qtEvent;
          qtEvent = nil;
        }
    }
}

void QTWindow::Pause()
{
  DEBUG0((cout << "QTWindow::Pause, paused " << paused 
          << " now " << Simulator::Now() << endl));
  if (paused)
    { // Pressing pause while already paused is a "one-shot"
      oneShot = true;
      return;
    }
  paused = true;
  WaitWhilePaused();
#ifdef OLD_WAY
  if (quit)
    {
      delete app;
      app = nil;
    }
#endif
}

void QTWindow::Quit()
{
  Exit();
  cout << "Quit button pushed" << endl;
  Simulator::instance->Halt();
  //app->quit();
  //if (paused)
  //  { // We need an update event to complete the exit
  //    if (!qtEvent) qtEvent = new QTEvent(QTEvent::UPDATE);
  //    Simulator::instance->Schedule(qtEvent, 0.0, this);
  //  }
}

void QTWindow::Exit()
{ // Exit causes the animation window to close, but the simulation continues.
  quit = true;
  //app->quit();
  //delete app;
  //app = nil;
  //if (qtEvent)
  //  {
  //    Simulator::instance->Cancel(qtEvent);
  //    delete qtEvent;
  //    qtEvent = nil;
  //  }
}

void QTWindow::TimerDone()
{
  ready = true;
}


#endif


// Private members
#ifndef COMPACT
void QTWindow::Initialize()
{
  // Compute usage pixel w/h in canvas, allowing for border and node pixel size
  QTCoord_t cw = canvasX - 2 * border - nodePixelSize;
  QTCoord_t ch = canvasY - 2 * border - nodePixelSize;

  // Get range of node coordinates (meters)
  smallestX = Simulator::instance->SmallestX();
  smallestY = Simulator::instance->SmallestY();
  largestX = Simulator::instance->LargestX();
  largestY = Simulator::instance->LargestY();
  Meters_t deltaX = largestX - smallestX;
  Meters_t deltaY = largestY - smallestY;

  DEBUG0((cout << " SX " << smallestX << " LX " << largestX 
          << " SY " << smallestY << " LY " << largestY
          << " DX " << deltaX << " DY " << deltaY << endl));
  
  // Make sure the region is square to maintain proper aspect ratio
  if (deltaX < deltaY) 
    {
      Meters_t adder = (deltaY - deltaX) / 2;
      cout << "Adjusting X by += " << adder << endl;
      smallestX -= adder;
      largestX += adder;
    }
  else if (deltaY < deltaX) 
    {
      Meters_t adder = (deltaX - deltaY) / 2;
      cout << "Adjusting Y by += " << adder << endl;
      smallestY -= adder;
      largestY += adder;
    }

  DEBUG0((cout << " SX " << smallestX << " LX " << largestX 
          << " SY " << smallestY << " LY " << largestY
          << " DX " << deltaX << " DY " << deltaY << endl));
  
  cout << " SX " << smallestX << " LX " << largestX 
       << " SY " << smallestY << " LY " << largestY
       << " DX " << deltaX << " DY " << deltaY 
       << " cw " << cw << " ch " << ch
       << endl;

  // Calculate the meters to pixels conversions
  metersToPixelsX = cw / (largestX - smallestX);
  metersToPixelsY = ch / (largestY - smallestY);
}

QPoint QTWindow::NodeLocation(Node* n)
{
  Location l = n->UpdateLocation();
  DEBUG0((cout << "nx " << l.X() << " ny " << l.Y()
          << " mtpx " << metersToPixelsX
          << " mtpy " << metersToPixelsY << endl));
  return LocationToPixels(l);
#ifdef MOVED_TO_METHOD
  // Calculate coordinates of CENTER of the node
#ifdef THIS_IS_UPSIDE_DOWN
  return QPoint((QTCoord_t)((l.X() - smallestX)*metersToPixelsX) 
                 - nodePixelSize / 2 + border * 2,
                (QTCoord_t)((l.Y() - smallestY)*metersToPixelsY) 
                 - nodePixelSize / 2 + border * 2);
#endif
  return QPoint((QTCoord_t)((l.X() - smallestX)*metersToPixelsX) 
                 - nodePixelSize / 2 + border * 2,
                (QTCoord_t)((largestY - l.Y())*metersToPixelsY)
                + nodePixelSize / 2 + border * 2);
#endif
}

void QTWindow::DrawNode(Node* n, Count_t i)
{
  QPoint p = NodeLocation(n); // Get location
  QCanvasItem* r;
  if (i >= nodeItems.size())
    { // New one
      r = n->Display(p, this);
      if (!r) return; // Not displayed
      nodeItems.push_back(r);
    }
  else
    { // Use existing
      n->Display(p, this);
    }
}

void QTWindow::DrawP2P(Node* f, Node* t)
{ // Draw p2p link connecting node "from" to node "to"
  QPoint fp = NodeLocation(f);
  QPoint tp = NodeLocation(t);
  // Create a new canvas item
  QCanvasLine* cl = new QCanvasLine(canvas);
  linkItems.push_back(cl);
  cl->show();
  cl->setPoints(fp.x(), fp.y(), tp.x(), tp.y());
  Interface* ifaceSrc = f->GetIfByNode(t);
  Link* linkSrc = ifaceSrc->GetLink();
  Interface* ifaceDst = t->GetIfByNode(f);
  Link* linkDst = ifaceDst->GetLink();
  displayedLinks.push_back(NNIfLink(f, t, ifaceSrc, ifaceDst,
                                    linkSrc, linkDst, cl));
  displayedLinks.push_back(NNIfLink(t, f, ifaceDst, ifaceSrc,
                                    linkDst, linkSrc, cl));
  // If interface is down in either direction color it red
  if (ifaceSrc->IsDown() || ifaceDst->IsDown())
    {
      cl->setPen(Qt::red);
    }
}

void QTWindow::RedrawP2P(Interface* ifaceSrc, Interface* ifaceDst)
{ // Used to re-draw a link during failure or recovery
  for (NNIfLVec_t::size_type i = 0; i < displayedLinks.size(); ++i)
    {
      NNIfLink& nn = displayedLinks[i];
      if ((nn.ifaceDest == ifaceDst && nn.ifaceSource == ifaceSrc) ||
          (nn.ifaceDest == ifaceSrc && nn.ifaceSource == ifaceDst))
        { // Found it
          QCanvasPolygonalItem* cl = nn.canvasItem;
          if (ifaceDst->IsDown() || ifaceSrc->IsDown())
            { // Color red if either interface is down
              cl->setPen(Qt::red);
            }
          else
            {
              cl->setPen(Qt::black);
            }
          canvas->update();
          return;
        }
    }
}

  
void QTWindow::DrawQ(Node* f, Node* t)
{ // Draw queue on the link connecting node "from" to node "to"
  // Not implemented yet...not quite sure what to do.
}

Count_t showCount = 0;

void QTWindow::DisplayAllNodes(bool forced)
{
  DEBUG0((cout << "QTWindow::DAN, hm " << Simulator::instance->HasMobility()
          << " td " << topologyDisplayed
          << " forced " << forced 
          << "  nodes.size " << Node::GetNodes().size()
          << endl));
  const NodeVec_t& nodes = Node::GetNodes();
  // Check if called before topology created..if so just return
  if (nodes.empty()) return;
  if (!topologyDisplayed) Initialize(); // Initialize some canvas values
  showCount = 0;
  if (Simulator::instance->HasMobility() || !topologyDisplayed || forced)
    { // Need to re-display
      if (!topologyDisplayed || forced) displayedLinks.clear();
      for (NodeVec_t::size_type i = 0; i < nodes.size(); ++i)
        {
          Node* n = nodes[i];
          if (n->Show())
            {
              DrawNode(n, i);
              showCount++;
              if (!topologyDisplayed || forced)
                {
                  // Draw the links to neighbors
                  NodeWeightVec_t nwv;
                  n->Neighbors(nwv); // Get the neighbors
                  for (NodeWeightVec_t::size_type j = 0; j < nwv.size(); ++j)
                    {
                      Node* nn = nwv[j].node;
                      if (nn->Id() > n->Id() && nn->Show())
                        { // Only draw to higher nodes (lower already drawn)
                          DEBUG0((cout << "Drawing p2p link" << endl));
                          DrawP2P(n, nn);
                        }
                    }
                }
            }
        }
      topologyDisplayed = true;
      DEBUG0((cout << "Displayed " << showCount << " nodes" << endl));
    }
  DisplayAllPackets();
  canvas->update(); // Show the updates
}

void QTWindow::DisplayAllPackets()
{
  DEBUG(2,(cout << "Displaying all pkts, dl size "
           << displayedLinks.size() << endl));
  // Start by clearing existing
  for (CItemVec_t::size_type i = 0; i < packetItems.size(); ++i)
    {
      delete packetItems[i];
    }
  packetItems.clear();

  for (NNIfLVec_t::size_type i = 0; i < displayedLinks.size(); ++i)
    {
      Interface* id = displayedLinks[i].ifaceDest;
      Linkp2p*   link = (Linkp2p*)displayedLinks[i].linkSource;
      Time_t     delay = link->Delay();  // Propagation delay along link
      Rate_t     bw = link->Bandwidth(); // Link bandwidth
      
      // The destination interface has a list of pending packets
      EventCQ* pending = id->GetEventCQ();
      if (!pending) continue; // No event cq yet, so no packets
      while(playbackMode && !pending->empty())
        { // remove any rx events n the past
          EventPair* ep = pending->front();
          if (ep->time > Simulator::Now()) break; // Received in future, keep
          pending->pop_front();       // Remove this one
          LinkEvent* evRx = (LinkEvent*)ep->event;
          DEBUG0((cout << "QTW removing evRx time " << ep->time
                  << " now " << Simulator::Now() << endl));

          delete evRx->p;
          delete evRx;
        }


      for (Count_t j = 0; j < pending->size();  ++j)
        {
          EventPair* ev = (*pending)[j];
          LinkEvent* le = (LinkEvent*)(ev->event);
          Packet*    p = le->p;
          QColor     c(Qt::blue);
          // See if the packet has a color assigned
          if (p->IsColored())
            {
              c = QColor(p->R(), p->G(), p->B());
            }
          
          // The timestamp in the event is last bit received.
          // We also need first bit time.
          Count_t    pSize = p->Size()*8; // Size in bits
          pSize -= pSize/10; // Shorten a bit to show breaks between packets
          Time_t     lastBitTime  = ev->time;
          Time_t     firstBitTime = ev->time - (double)(pSize)/bw;
          // Check if Pkt sent in future (playback). ignore
          if ((firstBitTime - delay) > Simulator::Now())
            {
              DEBUG0((cout << "Ignoring pkt sent in future" << endl));
              break;
            }

          // Check if last bit received in the past (playback), ignore
          if (lastBitTime < Simulator::Now())
            {
              DEBUG0((cout << "Ignoring pkt received in past" << endl));
              break;
            }


          if (firstBitTime < Simulator::Now()) firstBitTime = Simulator::Now();
          Time_t fbFuture = firstBitTime - Simulator::Now();
          Time_t lbFuture = lastBitTime - Simulator::Now();
          if (lbFuture > delay) lbFuture = delay;
          // Compute fraction of the link delay for first and last bit times.
          // This is the "distance" along the link line for the display
          Mult_t dfb = 1.0 - fbFuture/delay;
          Mult_t dlb = 1.0 - lbFuture/delay;
          DEBUG0((cout << "Displaying pkt, dfb " << dfb 
                  << " dlb " << dlb << endl));
          DisplayOnePacket(displayedLinks[i].s, displayedLinks[i].d,
                           dfb, dlb, c);
        }
    }
}

void QTWindow::Clear()
{ // Remove all items from the canvas
  // First the nodes
  for (CItemVec_t::size_type i = 0; i < nodeItems.size(); ++i)
    {
      delete nodeItems[i];
    }
  nodeItems.clear();
  // Next the links
  for (CItemVec_t::size_type i = 0; i < linkItems.size(); ++i)
    {
      delete linkItems[i];
    }
  linkItems.clear();
  // Finally the packets
  for (CItemVec_t::size_type i = 0; i < packetItems.size(); ++i)
    {
      delete packetItems[i];
    }
  packetItems.clear();
  // Note no longer displayed so it will redraw next time
  topologyDisplayed = false;
}

void QTWindow::DisplayOnePacket(Node* s, Node* d,
                                Mult_t firstBit, Mult_t lastBit,
                                const QColor& c)
{
  QPoint sp = NodeLocation(s); // Get location of source
  QPoint dp = NodeLocation(d); // Get location of destination
  // Compute length of line connecting src/dst
  double dx = dp.x() - sp.x();
  double dy = dp.y() - sp.y();
  DEBUG(2,(cout << " sp.x " << sp.x() << " sp.y " << sp.y()
           << " dp.x " << dp.x() << " dp.y " << dp.y()
           << " dx " << dx << " dy " << dy << endl));
  double dist = sqrt(dx*dx + dy*dy);
  DEBUG(2,(cout << "Distance node " << s->Id() << " to " << d->Id() 
           << " is " << dist << endl));
  // Compute the angle of a line from src to dst
  double theta = atan2(dy, dx);
  DEBUG(2,(cout << "Angle is " << 360.0 * theta / ( 2.0 * M_PI)
           << " degrees" << endl));
  bool verbose = (theta == 0.0);
  verbose = false;
  
  double sinTheta = sin(theta);
  double cosTheta = cos(theta);
  double pxfb = sp.x() + dist * firstBit * cosTheta;
  double pyfb = sp.y() + dist * firstBit * sinTheta;
  double pxlb = sp.x() + dist * lastBit  * cosTheta;
  double pylb = sp.y() + dist * lastBit  * sinTheta;
  // Use QCanvasPolygon for packets
  QPointArray qpa(5); // Four Points for rectangle, plus closing point
  // Compute points 2 pixels left and right of specified
  // Also translate the packet so we are 4 pixels "to the right"
  // of the link line, allowing bi-directional packets to not
  // interfere with each other.
  double sinThetaM2 = sin(theta + M_PI_2);
  double cosThetaM2 = cos(theta + M_PI_2);
  double width = 2.0;

  double pxlfb = pxfb - width * sinTheta + 4 * cosThetaM2;
  double pylfb = pyfb + width * cosTheta + 4 * sinThetaM2;
  double pxrfb = pxfb + width * sinTheta + 4 * cosThetaM2;
  double pyrfb = pyfb - width * cosTheta + 4 * sinThetaM2;
  qpa.setPoint(0, (int)pxlfb, (int)pylfb);
  qpa.setPoint(1, (int)pxrfb, (int)pyrfb);
  double pxllb = pxlb - width * sinTheta + 4 * cosThetaM2;
  double pyllb = pylb + width * cosTheta + 4 * sinThetaM2;
  double pxrlb = pxlb + width * sinTheta + 4 * cosThetaM2;
  double pyrlb = pylb - width * cosTheta + 4 * sinThetaM2;
  qpa.setPoint(2, (int)pxrlb, (int)pyrlb);
  qpa.setPoint(3, (int)pxllb, (int)pyllb);
  qpa.setPoint(4, (int)pxlfb, (int)pylfb); // Close the polygon
  
  if (verbose)
    {
      cout << "cosT  " << cosTheta << " sinT " << sinTheta << endl;
      cout << "FB  x " << pxfb << " y " << pyfb << endl;
      cout << "LB  x " << pxlb << " y " << pylb << endl;
      cout << "LFB x " << qpa.point(0).x() << " y " << qpa.point(0).y() <<endl;
      cout << "RFB x " << qpa.point(1).x() << " y " << qpa.point(1).y() <<endl;
      cout << "RLB x " << qpa.point(2).x() << " y " << qpa.point(2).y() <<endl;
      cout << "LLB x " << qpa.point(3).x() << " y " << qpa.point(3).y() <<endl;
    }
  
  QCanvasPolygon* cp = new QCanvasPolygon(canvas);
  packetItems.push_back(cp);
  cp->setPen(Qt::black);
  cp->setBrush(c);
  cp->show();
  cp->setPoints(qpa);
}

void QTWindow::WaitWhilePaused()
{
  while (app && paused && !quit && !oneShot)
    {
      app->processEvents(1);
    }
  oneShot = false;
}

QPoint QTWindow::LocationToPixels(const Location& l)
{
  return QPoint((QTCoord_t)((l.X() - smallestX)*metersToPixelsX) 
                 - nodePixelSize / 2 + border * 2,
                (QTCoord_t)((largestY - l.Y())*metersToPixelsY)
                + nodePixelSize / 2 + border * 2);
}

QPoint QTWindow::LocationToPixelsAbs(const Location& l)
{
  return QPoint((QTCoord_t)((l.X() - smallestX)*metersToPixelsX),
                (QTCoord_t)((largestY - l.Y())*metersToPixelsY));
}

void   QTWindow::WirelessTxStart(Node* n, InterfaceWireless* iFace, Node* d,
                                 Meters_t range)
{
  if (!n->IsReal()) return;
  DEBUG(1, (cout << "Anim Wireless Tx Start, range " << range
          << " time " << Simulator::Now() << endl));
  WirelessTxStart(n, iFace, d, range, 0);
}

void   QTWindow::WirelessTxEnd(Node* n, InterfaceWireless* iFace, Node* d,
                               Meters_t range)
{
  if (!n->IsReal()) return;
  DEBUG(1, (cout << "Anim Wireless Tx End, range " << range
          << " time " << Simulator::Now() << endl));
  WirelessTxEnd(n, iFace, d, range, 0);
}

// Private for animating the wireless tx

void   QTWindow::WirelessTxStart(Node* n, InterfaceWireless* iFace, Node* d,
                                 Meters_t range, Count_t it)
{
  if (it)
    {
      // Add the circle to the canvas items list (if not zeroth iteration)
      // First find the radius of the circle in pixels
      Location l = n->GetLocation();
      QPoint   lp = LocationToPixels(l);
      n->Display(lp, this);
      Meters_t r = range / 10.0 * it;
      Location rl = l;
      rl.X(rl.X() + r);
      QPoint  rp = LocationToPixels(rl);
      QCanvasCircle* c = new QCanvasCircle(rp.x() - lp.x(), canvas);
      // Set the color, with reduced intensity farther  away
      const QColor& col = n->WirelessTxColor();
      int red = col.red();
      int green = col.green();
      int blue = col.blue();
      int nred   = red   + (int)((double)(255-red)   * (it-1)/10.0);
      int ngreen = green + (int)((double)(255-green) * (it-1)/10.0);
      int nblue  = blue  + (int)((double)(255-blue)  * (it-1)/10.0);
      c->setPen(QColor(nred, ngreen, nblue));
      c->move(lp.x(), lp.y());
      c->show();
      n->PushWirelessTx(c);
      // Now re-display any nodes in the power range just processed
      Meters_t prevr = range / 10.0 * (it - 1);
      const RadioVec_t& rv = n->GetRadioInterfaceList();
      for (RadioVec_t::const_iterator i = rv.begin(); i != rv.end(); ++i)
        {
          InterfaceWireless* iFace1 = i->inf;
          Meters_t dist1 = i->dist;
          if (dist1 > prevr && dist1 <= r)
            { // Need to update this one
              Node* n1 = iFace1->GetNode();
              Location l1 = n1->GetLocation();
              QPoint   lp1 = LocationToPixels(l1);
              n1->Display(lp1, this);
            }
        }
          
      // Update the simtime
      simTime->display(Simulator::Now());
      canvas->update(); // Show the changes
      ProcessEvents();
      if (paused) WaitWhilePaused();
    }
  
  if (it < 10)
    {
      QTEvent* ev = new QTEvent(QTEvent::WIRELESS_TX_START);
      ev->src = n;
      ev->dst = d;
      ev->iFace = iFace;
      ev->range = range;
      ev->iter = it;
      Time_t perIteration = ev->range / SPEED_LIGHT;
      // Schedule next update event
      Simulator::instance->Schedule(ev, perIteration, this);
    }
}

void   QTWindow::WirelessTxEnd(Node* n, InterfaceWireless* iFace, Node* d,
                               Meters_t range, Count_t it)
{
  if (it)
    { // Remove the circles one-by-one to animate end of transmission
      // Must be real node here
      Location l = n->GetLocation();
      QPoint   lp = LocationToPixels(l);
      n->Display(lp, this);
      QCanvasItem* c = n->PopWirelessTx();
      if(c)delete c;
      // Now re-display any nodes in the power range just processed
      Meters_t r = range / 10.0 * it;
      Meters_t prevr = range / 10.0 * (it - 1);
      const RadioVec_t& rv = n->GetRadioInterfaceList();
      for (RadioVec_t::const_iterator i = rv.begin(); i != rv.end(); ++i)
        {
          InterfaceWireless* iFace1 = i->inf;
          Meters_t dist1 = i->dist;
          if (dist1 > prevr && dist1 <= r)
            { // Need to update this one
              Node* n1 = iFace1->GetNode();
              Location l1 = n1->GetLocation();
              QPoint   lp1 = LocationToPixels(l1);
              n1->Display(lp1, this);
            }
        }
          
      // Update the simtime
      simTime->display(Simulator::Now());
      canvas->update(); // Show the changes
      ProcessEvents();
      if (paused) WaitWhilePaused();
    }
  
  if (it < 10)
    {
      QTEvent* ev = new QTEvent(QTEvent::WIRELESS_TX_END);
      ev->src = n;
      ev->dst = d;
      ev->iFace = iFace;
      ev->range = range;
      ev->iter = it;
      Time_t perIteration = ev->range / SPEED_LIGHT;
      // Schedule next update event
      Simulator::instance->Schedule(ev, perIteration, this);
    }
}

void QTWindow::RecordNextFrame(QTEvent* e)
{ // Records the PPM file for next display frame
  if (!recording) return;
  char work[128];
  sprintf(work, "GTNetS%05d.ppm", recordingFrame++);
  //QPixmap pm = QPixmap::grabWidget(view);
  QPixmap pm = QPixmap::grabWidget(box);
  pm.save(work, "PPM");
  cout << "Saved file " << work << endl;
  // Schedule next event at the appropriate time in the future
  Time_t future = 1.0 / (25.0 * Simulator::recorderSpeedup);
  Simulator::instance->Schedule(e, future, this);
}

#endif
  
#endif
