/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Rate and Time Parsing classes
// George F. Riley.  Georgia Tech, Spring 2002

// Classes to allow rate and time specification with common abbreviations

#include <iostream>
#include <string>
#include <stdlib.h>
#include <stdio.h>

using namespace std;

#include "ratetimeparse.h"

Rate::Rate(const string s)
{
  rate = Parse(s);
}

Rate::Rate(const char* s)
{
  rate = Parse(s);
}

Rate::operator Rate_t () const
{
  return rate;
}

Rate_t Rate::Parse(const string s)
{
  string::size_type n = s.find_first_not_of("0123456789.");
  if (n != string::npos)
    { // Found non-numeric
      string numeric = s.substr(0, n);
      string trailer = s.substr(n, string::npos);
      Rate_t r = atof(numeric.c_str());
      if (trailer == string("b")) return r;     // Bits
      if (trailer == string("B")) return r * 8.0; // Bytes
      if (trailer == string("Kb"))return r * 1000.0; // KiloBits
      if (trailer == string("KB"))return r * 8000.0; // KiloBytes
      if (trailer == string("Mb"))return r * 1000000.0; // MegaBits
      if (trailer == string("MB"))return r * 8000000.0; // MegaBytes
      if (trailer == string("Gb"))return r * 1000000000.0; // GigaBits
      if (trailer == string("GB"))return r * 8000000000.0; // GigaBytes
      cout << "Can't parse rate " << s << endl;
      exit(0);
    }
  return atof(s.c_str()); // No special formatting, just convert value
}

// Time conversion
Time::Time(const string s)
{
  time = Parse(s);
}

Time::Time(const char* s)
{
  time = Parse(s);
}

Time::operator Time_t () const
{
  return time;
}

Time::operator string () const
{
  char work[64];
  // ! need to add significant digits class var
  sprintf(work, "%8.4f", time);
  return string(work);
}

Time_t Time::Parse(const string s)
{
  string::size_type n = s.find_first_not_of("0123456789.");
  if (n != string::npos)
    { // Found non-numeric
      string numeric = s.substr(0, n);
      string trailer = s.substr(n, string::npos);
      Time_t r = atof(numeric.c_str());
      if (trailer == string("h")) return r * 3600.0;       // Hours
      if (trailer == string("m")) return r * 60.0;         // Minutes
      if (trailer == string("s")) return r;                // Seconds
      if (trailer == string("sec")) return r;              // Seconds
      if (trailer == string("ms"))return r / 1000.0;       // Milliseconds
      if (trailer == string("us"))return r / 1000000.0;    // Microseconds
      if (trailer == string("ns"))return r / 1000000000.0; // Nanoseconds
      cout << "Can't parse time " << s << endl;
      exit(0);
    }
  return atof(s.c_str()); // No special formatting, just convert value
}

