/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Manage 32 bit unsigned sequence numbers
// George F. Riley.  Georgia Tech, Spring 2002

// Class to manage arithmetic operations on sequence numbers (mod 2^32)

#ifndef __seq_h__
#define __seq_h__

#include "common-defs.h"

#define MAX_SEQ ((Seq_t)0xffffffff)

//Doc:ClassXRef
class Seq {
public:
  // Constructors
  Seq() : seq(0) { }
  Seq(const Seq_t s) : seq(s) { }
  // Typecast
  operator Seq_t() const { return seq;}
  // Assignment
  Seq& operator=(const Seq_t s) { seq = s; return *this;}
  // Arithmetic operators
  Seq& operator+=(const Seq_t s) { seq += s; return *this;}
  Seq  operator++()              { seq++; return *this;}
  Seq  operator++(int)           { Seq ss(seq); seq++; return ss;}
  Seq& operator-=(const Seq_t s) { seq -= s; return *this;}
  Seq  operator--()              { seq--; return *this;}
  Seq  operator--(int)           { Seq ss(seq); seq--; return ss;}
public:
  Seq_t seq;
};

// Comparison operators

bool operator< (const Seq l, const Seq r);
bool operator<=(const Seq l, const Seq r); 
bool operator> (const Seq l, const Seq r);
bool operator>=(const Seq l, const Seq r);

// Non-member arithmetic operators
Seq  operator+ (const Seq l, const Seq r);
Seq  operator- (const Seq l, const Seq r);

#endif


