/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

1// Georgia Tech Network Simulator - Socket Base lass
// George F. Riley.  Georgia Tech, Spring 2002

// Simulation interface mimicking the UNIX socket interface.

#ifndef __simsocket_h__
#define __simsocket_h__

#include "common-defs.h"
#include "node.h"

#include <socket.h>

class SockAddr;

//Doc:ClassXRef
class Socket {
public:
  Socket(Node*); // All socket apps must have associated node
  void Start();  // All socket apps must implement "Start"
public:
  int socket(int, int, int) = 0;
  int connect(int, SockAddr*, int) = 0;
  int write(int, void*, int) = 0;
  int read(int, void*, int) = 0;
  int close(int) = 0;
  int bind(int, SockAddr*, int) = 0;
  int listen(int, int) = 0;
  int accept(int, SockAddr*, Size_t*) = 0;
  int recv(int, void*, int, int) = 0;
  int recvfrom(int, void*, int, int, SockAddr*, Size_t*) = 0;
  int recvmsg(int, struct msghdr*, flags) = 0;
  int send(int, void*, int, int) = 0;
  int sendto(int, void*, int, int, struct sockaddr*, socklen_t) = 0;
  int sendmsg(int, struct msghdr*, flags) = 0;
  int shutdown(int, int) = 0;
  int getpeername(int, struct sockaddr*, socklen_t*) = 0;
  int getsockopt(int, int, int, void*, socklen_t*) = 0;
  int setsockopt(int, int, int, void*, socklen_t) = 0;
private:
  Protocol* pL4Proto; // Points to the associated L4 protocol (TCP/UDP)
};

#endif

