/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/


// SubDomain Class Definitions
// Talal Jaafar, Georgia Tech, Spring 2003

#ifndef __SUBDOMAIN_H__
#define __SUBDOMAIN_H__

#include "node.h"
#include <vector>

typedef std::vector<Node*>  NodeList_t;

//Doc:ClassXRef
class SubDomain {
  //Doc:Class Defines an interface for the subdomain
  int SubId;                  // SubDomain id 
  double TNodeid;             // Transit Node id to which the Sub belongs
  NodeList_t NodeList;        // Node list that are contained in the Sub
  int MaxNodeId;
 public:
  //Doc:Method
  SubDomain(double iSubnum, int Id, Node *node){
    TNodeid = iSubnum;         
    SubId = Id;                 
    NodeList.push_back(node);
    MaxNodeId = node->Id();
  }
  //Doc:Desc This method is the default constructor (sets sub id, first subnode id)
  //Doc:Arg1 The transit node id
  //Doc:Arg2 The subdomain id
  //Doc:Arg3 A pointer to first node to be added to the subdomain

  //Doc:Method
  void AddNode(Node *node){       
    NodeList.push_back(node);
    MaxNodeId = node->Id();
  }
  //Doc:Desc This method is used to add a new node to the subdomain
  //Doc:Arg1 A pointer to the node to be added to the subdomain
  
  //Doc:Method
  int GetSubId() { return SubId; }
  //Doc:Desc This method is used to return the subdomain id
  //Doc:Return An integer (sub id)

  //Doc:Method
  double GetSubTNodeid() { return TNodeid; }
  //Doc:Desc This method is used to return the sub transit node id
  //Doc:Return A double (transit node id)

  //Doc:Method
  NodeList_t GetNodeList() { return NodeList; }
  //Doc:Desc This method is used to return the list of the nodes associated with this subdomain
  //Doc:Return A vector of nodes

  //Doc:Method
  int GetMaxNodeId() { return MaxNodeId; }
  //Doc:Desc This method is used to return the highest node id in the subdomain
  //Doc:Return An integer (highest node id)
  
};

#endif
