/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - TCP-Tahoe Class
// George F. Riley.  Georgia Tech, Spring 2002

#include <iostream>

//#define DEBUG_MASK 0x03
#include "debug.h"
#include "tcp-tahoe.h"
#include "simulator.h"
#include "globalstats.h"

using namespace std;

// Constructors
TCPTahoe::TCPTahoe() : TCP() 
{
}

TCPTahoe::TCPTahoe(Node* n) : TCP(n) 
{
}

TCPTahoe::TCPTahoe(const TCPTahoe& c) : TCP(c)
{
}

// L4Protocol Methods
L4Protocol* TCPTahoe::Copy() const
{
  return new TCPTahoe(*this); // Create a copy of this protocol
}

// TCP Overridden methods
void TCPTahoe::DupAck(const TCPHeader& t, Count_t c)
{ // Dup ack received
  DEBUG(1,(cout << "Hello from TCPTahoe::DupAck " << t.ackNumber
           << ", count " << c
           << ", time " << Simulator::Now()
           << endl));
  if (c == 3)
    { // Count of three indicates triple duplicate ack
      // flag_ishwar added two flags
      fastReTxFlag = 1;
      tflag = 1;
      // end Ishwar added
      ssThresh = cWin / 2; // Reduce slow start threshold to half cwin
      cWin = segSize;      // Collapse congestion window
      DEBUG(1,(cout << "Retransmitting" << endl));
      DEBUG(2,(cout << "TCP " << this << " retx, tripledup ack" 
               << t.ackNumber << endl));
      Retransmit();        // Retransmit the packet
      Stats::tcpDupAckRetx++;
    }
}

void TCPTahoe::ReTxTimeout()
{ // Retransmit timeout
  DEBUG0((cout << "Hello from TCPTahoe::Timeout" 
           << ", time " << Simulator::Now()
          << endl));
  ssThresh = cWin / 2;      // Reduce slow start threshold to half cwin
  cWin = segSize;           // Collapse congestion window
  nextTxSeq = highestRxAck; // Start from highest Ack
  Retransmit();             // Retransmit the packet
}


