/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Timer Class
// George F. Riley.  Georgia Tech, Spring 2002

#include "debug.h"
#include "timer.h"
#include "simulator.h"

using namespace std;

// Provide a constructor
Timer::Timer()
{
}

void Timer::Handle(Event* e, Time_t t)
{
  TimerEvent* ev = (TimerEvent*)e;
  if (!ev->timerHandler)
    {
      cout << "HuH?  Timer::Handle with no timerHandler" << endl;
      delete e;
      return;
    }
  ev->timerHandler->Timeout(ev);
}

void Timer::Timeout(TimerEvent* ev)
{ // Override if needed
  cout << "default timeout handler " << endl;
  delete ev;
}

void Timer::Schedule(TimerEvent* ev, Time_t when, TimerHandler* h )
{
  // Set the specified timeout handler
  ev->timerHandler = h;
  ev->timer = this;
  // Schedule the event "when" in the future
  DEBUG0((cout << "Scheduling timeout event at time " << when << endl));
  Simulator::instance->Schedule(ev, when, this);
}

void Timer::Schedule(TimerEvent* ev, Time_t when )
{
  Schedule(ev, when, this); // Use self as the handler for this event
}

void Timer::Cancel(TimerEvent* ev)
{ // Cancelling DOES NOT delete the event (can be re-used)
  Simulator::instance->Cancel(ev);
}

void Timer::Pause(TimerEvent* ev)
{
  ev->remainingTime = ev->Time() - Simulator::instance->Now();
  // remove the timeout event from the queue
  Cancel(ev);
}

void Timer::Resume(TimerEvent* ev)
{
  Simulator::instance->Schedule(ev, ev->remainingTime, this);
}

void Timer::ResumeAfter(TimerEvent* ev, Time_t time)
{
  Simulator::instance->Schedule(ev, ev->remainingTime + time, this);
}

void Timer::Resume(TimerEvent* ev, TimerHandler* h )
{
  ev->timerHandler=h;
  Simulator::instance->Schedule(ev, ev->remainingTime, this);
}
