/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - UDP Port Demultiplexer
// George F. Riley.  Georgia Tech, Spring 2002

// Picks the UDP instance by port number

#include <iostream>

#include "debug.h"
#include "udpdemux.h"
#include "node.h"
#include "protograph.h"
#include "udp.h"
#include "droppdu.h"
#include "icmp.h"

using namespace std;

// Define the pointer for the single tcpdemux instance
UDPDemux* UDPDemux::instance = NULL;

UDPDemux::UDPDemux()
{
  if (instance == NULL)
    { // First instance
      instance = this;
      ProtocolGraph::CreateCommon();
      DEBUG0((cout << "Inserting udpdemux, l4, proto " 
              << UDP::ProtoNumber << endl));
      ProtocolGraph::common->Insert(4, UDP::ProtoNumber, this); // Register
    }
}

#ifdef SIM
void UDPDemux::DataIndication(Node* n, Packet* p, IPAddr_t ip, 
                              L3Protocol& l3, Interface* i, IPAddr_t dst)
#else
void UDPDemux::DataIndication(Node* n, Packet* p, IPAddr_t ip, 
                              L3Protocol& l3, Interface* i)
#endif //SIM
{
  UDPHeader* pdu = (UDPHeader*)p->PeekPDU(); // Get the UDP pdu header (peek)
  UDP* udp = (UDP*)n->LookupByPort(UDP::ProtoNumber, pdu->destPort);
  DEBUG0((cout << "UDPDemux::DataInd, udp instance " << udp 
          << " dstPort " << pdu->destPort << endl));
  if (!udp)
    { // No port, discard and log
      DropPDU d("L3-NP");
      n->TracePDU(&l3, &d);
      Stats::pktsDropped++;
      // Notify ICMP of port unreachable
      IPV4Header* iphdr = (IPV4Header*)p->PeekPDU(-1);
      ICMP::DestinationUnreachable(n, p, iphdr, ICMP::PORT_UNREACHABLE);
      delete p;
    }
  else
    {
      if (udp->extraRxDelay != 0)
        {// Need to schedule an event for later
          Simulator::instance->Schedule(
              new L4DelayedRxEvent(udp, n, p, ip, i),
              udp->extraRxDelay, udp);
          
        }
      else
        {
          udp->DataIndication(n, p, ip, i); // Pass to the UDP instance
        }
    }
}

// Static methods
UDPDemux* UDPDemux::Instance()
{
  return instance;
}

void UDPDemux::Initialize()
{
  if (!instance) new UDPDemux();
}
