/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Common code for  all GTNetS validation programs
// George F. Riley, Georgia Tech, Spring 2004

#include "validation.h"
#include "rng.h"

bool Validation::sameSeed    = false;
bool Validation::noAnimation = false;
bool Validation::forceTrace  = false;

void Validation::Init(int& argc, char** argv)
{
  for (int i = 1; i < argc; )
    {
      std::string arg(argv[i]);
      if (arg == std::string("-sameseed"))
        {
          sameSeed = true;
          RemoveArg(i, argc, argv);
        }
      else if (arg == std::string("-noanimate"))
        {
          noAnimation = true;
          RemoveArg(i, argc, argv);
        }
      else if (arg == std::string("-forcetrace"))
        {
          forceTrace = true;
          RemoveArg(i, argc, argv);
        }
      else ++i;
    }
  if (sameSeed)
    {
      Random::GlobalSeed(12341243,34124353,97392145,
                            33972914,87672334,98896666);
    }
}

void Validation::RemoveArg(int i, int& argc, char** argv)
{ // Move the argc list down one position
  for (int j = i; j < (argc - 1); ++j) argv[j] = argv[j+1];
  argc--;
}


