/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - VirusThrottle class
// Mohamed Abd Elhafez.  Georgia Tech, Spring 2004

// Implements the virus throttle algorithm for worm containment


#include <iostream>

#include "debug.h"
#include "virusthrottle.h"
#include "l2proto802.3.h"
#include "node.h"
#include "droppdu.h"
#include "hex.h"
//#include "math.h"

using namespace std;

#define DELAY_QUEUE_FULL 100
#define WORKING_QUEUE_FULL 5
#define SENSITIVITY 1

//class QColor;

int VirusThrottle::delayqueuelimit = DELAY_QUEUE_FULL;
int VirusThrottle::workingsetsize = WORKING_QUEUE_FULL;
int VirusThrottle::blocked = 0;
int VirusThrottle::falsedetection = 0;
bool VirusThrottle::routerthrottle = false;

VirusThrottle::VirusThrottle():  conn(true)
{
  cout << "creating virus throttle " << endl;
}


void VirusThrottle::ProcessOutPacket( Packet* p, IPAddr_t i,  int type, Interface* iface)
{
  IPV4Header* iphdr = (IPV4Header*)p->PeekPDU();
  if(iphdr->src != iface->GetIPAddr() && !routerthrottle)
    {
      iface->Xmit(p, i, type);
      return;
    }

  if(iphdr->protocol == TCP_PROTOCOL)
    {
      TCPHeader* tcphdr = (TCPHeader*)p->PeekPDU(1);
      if(tcphdr->flags != TCP::SYN)
	{
	  // cout << "VirusThrottle:: not SYN packet" << endl;
	  iface->Xmit(p, i, type);
	  return;
	}
      Delay(iphdr, p, i, type, iface);
    }
  else if(iphdr->protocol == UDP_PROTOCOL)
    {
      //cout << "VirusThrottle:: udp protocol is used " << endl;
      Delay(iphdr, p, i, type, iface);
    }
}


  void VirusThrottle::Delay(IPV4Header* iphdr, Packet* p, IPAddr_t i, int type, Interface* iface)
    {
      if(conn == false)
	{
	  //cout << "VirusThrottle:: connection is blocked" << endl;
	  //cout << "VirusThrottle queue size =  " << delayqueue.size() << endl;
	  // Dropping packet
	  cout << "dropping packet " << p->uid << endl;
	  DropPDU d("L3-VT", p);
	  iface->GetNode()->TracePDU(nil, &d);
	  DEBUG(0,(cout << "Packet Dropped " << endl));
	  Stats::pktsDropped++;
	  delete p;
	  
	  return;
	}
      if( InWorkingSet(iphdr->dst) )
	{
	  iface->Xmit(p, i, type);
	  return;
	}
      if(workingset.size() < workingsetsize)
	{
	  workingset.push_back(iphdr->dst);
	  iface->Xmit(p, i, type);
	  return;
	}
       
      cout << "VirusThrottle:: working set Contents ------------" << endl;
      for (int j = 0; j < workingset.size(); j++)
	{
	  cout << IPAddr::ToDotted(workingset[j]) << endl;

	}
	//	cout << "VirusThrottle:: Queuing packet " << p->uid << endl;
	delayqueue.push(PacketPair_t(p, type));
	if(delayqueue.size() == delayqueuelimit)
	  {
	    conn = false;
	    blocked++;
	
	    // this is plain wrong. need fixing   
#ifdef HAVE_QT 
	    // this is just a hack to find out if the node is really infected or not
	    QColor col = iface->GetNode()->Color();
	    if( col == Qt::red)
	      {
		cout << "VirusThrottle:: Worm is really infected" << endl;
	      }
	    else
	      {
		cout << "VirusThrottle:: False detection" << endl;
		falsedetection++;
	      }
	    
	    // try to turn the blocked node yellow
	    iface->GetNode()->Color(Qt::yellow);
#endif
	    return;
	  }
	// Schedule the next event
      if(!delayqueue.empty())
	{
	  //cout << "VirusThrottle:: Scheduling process queue event" << endl;
	  //cout << "VirusThrottle queue size =  " << delayqueue.size() << endl;
	  //Simulator::instance->Schedule(new ThrottleEvent(ThrottleEvent::PROCESS_QUEUE, i, iface), pow((double)2,(int)delayqueue.size()) * SENSITIVITY, this);
	  Simulator::instance->Schedule(new ThrottleEvent(ThrottleEvent::PROCESS_QUEUE, i, iface), SENSITIVITY, this);
		  
	}      
    }


  
void VirusThrottle::Handle(Event* e, Time_t t)
{
  ThrottleEvent* te = (ThrottleEvent*)e;
  IPV4Header* iphdr ;

  PacketPair_t pp;
  switch(te->event) {
  case ThrottleEvent::PROCESS_QUEUE :
    if(workingset.size() == workingsetsize)
      {
	//cout << "VirusThrottle working set queue size =  " << workingset.size() << endl;
	workingset.erase(workingset.begin());
      }
    if(!delayqueue.empty() && conn)
      {
	pp = delayqueue.front();
	iphdr = (IPV4Header*)pp.first->PeekPDU();
	te->iface->Xmit(pp.first, te->dst, pp.second);
	workingset.push_back(iphdr->dst);
	delayqueue.pop();

	//Simulator::instance->Schedule(new ThrottleEvent(ThrottleEvent::PROCESS_QUEUE, te->dst, te->iface), pow((double)2, (int)delayqueue.size())*SENSITIVITY, this);
	Simulator::instance->Schedule(new ThrottleEvent(ThrottleEvent::PROCESS_QUEUE,te->dst ,  te->iface), SENSITIVITY, this);

      }
    break;
  default:
    break;
  }
  delete te;
}



bool VirusThrottle::InWorkingSet(IPAddr_t ipaddr)
{
  bool found = false;
  int size = workingset.size();

  for(int i = 0; i < size; i++)
    {
      if( workingset[i] == ipaddr )
	{
	  found = true;
	  break;
	}
    }
  return found;
}
