/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Grid Network Object
// George F. Riley.  Georgia Tech, Spring 2002

// Define a two dimensional grid of wireless nodes, manually placed

#include "wireless-grid-manual.h"

using namespace std;

// Constructors
WirelessGridManual::WirelessGridManual( IPAddr_t i)
  : WirelessGrid(i), adjustRegion(true)
{
}

WirelessGridManual::WirelessGridManual(
                           const Location& l, // Lower left corner
                           const Location& u, // Upper right corner
                           IPAddr_t        i)
  : WirelessGrid(i),
    lowerLeft(l), upperRight(u), adjustRegion(false)
{
  Simulator::instance->NewLocation(l);
  Simulator::instance->NewLocation(u);
}

WirelessGridManual::~WirelessGridManual()
{
}

Location WirelessGridManual::GetRandomLocation()
{
  return Location(0,0);
}

Node* WirelessGridManual::AddNode()
{
  return AddNode(Location());
}

Node* WirelessGridManual::AddNode(const Location& l)
{
  return AddNode(new Node());
}

Node* WirelessGridManual::AddNode(Node* n)
{
  if (size == 0)
    {
      first = n->Id();
      startingIP = n->GetIPAddr();
    }
  size++;
  if (adjustRegion)
    {
      Location l = n->GetLocation();
      lowerLeft.X(min(lowerLeft.X(), l.X()));
      lowerLeft.Y(min(lowerLeft.Y(), l.Y()));
      upperRight.X(max(upperRight.X(), l.X()));
      upperRight.Y(max(upperRight.Y(), l.Y()));
    }
  nodes.push_back(n);
  return n;
}

