/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Random Polar Wireless Object
// George F. Riley.  Georgia Tech, Spring 2002

// Define a two dimensional grid of wireless nodes

#include <math.h>

#include "wireless-grid-polar.h"
#include "node.h"
#include "wlan.h"

// Constructors
WirelessGridPolar::WirelessGridPolar(
                             const Location& l, // Location of center
                             const Random& s,   // Rng for number of nodes
                             const Random& r,   // Rng for radius
                             const Random& a,   // Rng for angle
                             IPAddr_t      i)   // First IP Address
  : WirelessGrid(i),
    location(l), sizeRng(s.Copy()), radiusRng(r.Copy()),
    angleRng(a.Copy())
{
  size = sizeRng->IntValue();

#ifdef USAP_ADDON
	WirelessLink* wlan = new  WirelessLink(size, startingIP, Mask(24), L2Proto802_11());
#else
  WirelessLink* wlan = new  WirelessLink(size, i, Mask(24));
#endif // USAP_ADDON
  //! Need to fix mask value...add constructor parameter
  for (Count_t ni = 0; ni < size; ++ni)
    {
      Node*  n = wlan->GetNode(ni);
      //Node* n = new Node();
      Location l = GetRandomLocation();
      n->SetLocation(l.X(),l.Y());
      if (startingIP != IPADDR_NONE)
        {
          n->SetIPAddr(startingIP + ni);
        }
    }
}

// Destructor
WirelessGridPolar::~WirelessGridPolar()
{
  delete sizeRng;
  delete radiusRng;
  delete angleRng;
}

// Public Methods
Location WirelessGridPolar::GetRandomLocation()
{
  Meters_t r = radiusRng->Value();
  double   t = angleRng->Value();
  Meters_t x = r * cos((t * 2.0 * M_PI) / 360.0);
  Meters_t y = r * sin((t * 2.0 * M_PI) / 360.0);
  return Location(x,y);
}

