/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Random Polar Wireless Object
// George F. Riley.  Georgia Tech, Spring 2002

// Define a two dimensional grid of wireless nodes

#ifndef __wireless_polar_h__
#define __wireless_polar_h__

#include "common-defs.h"
#include "wireless-grid.h"

//Doc:ClassXRef
class WirelessGridPolar : public WirelessGrid {
public:
  WirelessGridPolar(const Location&,      // X/Y position of center
                    const Random&,        // Number of nodes
                    const Random&,        // Radius for each node
                    const Random& = Uniform(0.0, 360.0), // Angle
                    IPAddr_t = IPADDR_NONE);        // First IP Address
  ~WirelessGridPolar();
  Location GetRandomLocation();
private:
  Location location;        // Center of circle
  Random*  sizeRng;
  Random*  radiusRng;
  Random*  angleRng;
  Meters_t radius;          // Radius of circle (meters)
};

#endif
