/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/


// Georgia Tech Network Simulator - Worm Application Base class

// Monirul I Sharif,  Georgia Tech, Fall 2003
// All worm applications are descendents of this class.

// These classes will be updated soon with a completely new framework.

//#define DEBUG_MASK 0x1

#include "debug.h"
#include "worm.h"

#include "node.h"
#include "simulator.h"
#include <map>

#ifdef HAVE_QT
#include <qnamespace.h>
#endif

using namespace std;

#define DEFAULTSCANRANGE 65536

bool     Worm::initialized = false;
// this is for setting vulnerabilities
Random*  Worm::rngV = nil;
Size_t   Worm::payloadlength = 376;
PortId_t Worm::infectionport = 1434;
//100% machines are vulnerable, please change the folowing for
// for making host's vulnerability be selected randomly
double   Worm::vulnerability = 1.0;

IPAddr_t Worm::baseIP = IPAddr("192.168.0.0");

Count_t  Worm::totalinfected = 0;
Count_t  Worm::totalvulnerable = 0;
Count_t  Worm::totalinstances = 0;

string   Worm::signature = "WORM";
char *   Worm::wormdata = NULL;

// By default we are setting the target vector to be NULL
// The code then selects the uniform target vector object by default

WormTargetVector* Worm::defaultTV = nil;

// Constructors

Worm::Worm()
  : infected(false), vulnerable(true), started(false), node(nil), targetV(nil)
{
  copyOnConnect=false;

  if (!initialized) {
    Initialize();
    initialized = true;
  }

  if (!wormdata) {
    PrepareWormData(wormdata);
  }

  if (vulnerability!=1.0)   {
    if (!rngV) rngV = new Uniform(0, 1);
  }

  // if there is no target vector object defined then we are setting the 
  // it to be the Uniform random target vector class.
  if (!defaultTV) {
    defaultTV = new WTVUniform(DEFAULTSCANRANGE);
  }
}

// Application Methods
void Worm::StartApp()    // Called at time specified by Start
{
  if (infected)
	Activate();
  started = true;
}

void Worm::StopApp()     // Called at time specified by Stop
{
  started = false;
}

void Worm::AttachNode(Node* n) 
{
  node = n;
  totalinstances++;
  
  if (vulnerability!=1.0)
    vulnerable = rngV->Value()<=vulnerability;

  if (vulnerable)
    totalvulnerable++;
}

Application* Worm::Copy() const
{
  return new Worm(*this);
}

void Worm::Infect()
{
  infected = true;
  totalinfected++;

  if (started)        // if the application has started then Activate the worm
    Activate();
}

// this is a virtual class that is actually overriden in the subclasses
void Worm::Activate()
{
#ifdef HAVE_QT
  node->Color(Qt::red);
#endif
}

// the place where you put your initialization code
void Worm::Initialize()
{
  cout<<"worm initialize"<<endl;
}

bool Worm::IsInfected()
{
  return infected;
}

void Worm::ConnectionComplete(L4Protocol *proto)
{
}

void Worm::Sent(Count_t c, L4Protocol *proto)
{
}

void Worm::ConnectionFailed(L4Protocol *proto)
{
}

void Worm::CloseRequest(L4Protocol *proto)
{
}

void Worm::Closed(L4Protocol *proto)
{
}

IPAddr_t Worm::GenerateNextIPAddress()
{
  // randomx = randomx * 214013 + 0xffd9613c;
  // I am not following the random number generator of the Slammer worm
  IPAddr_t IP;
  DEBUG0((cout<<"Generating next IP address"<<endl));

  if (!targetV){
    targetV = defaultTV->Copy();
    targetV->Initialize(node->GetIPAddr());
  }

  IP = baseIP + targetV->Generate();

  return IP;
}

void Worm::PrepareWormData(char *&buffer)
{
  buffer = new char[payloadlength];
  for (unsigned int i=0; i < payloadlength; i++)
    buffer[i]='0'+i%10;
  strncpy(buffer, signature.c_str(), signature.length());
  DEBUG(1,(cout<<"WORM CONTENTS: "<<buffer<<endl));
}

void Worm::SetBaseIP(IPAddr_t base)
{
  baseIP = base;
}

void Worm::SetVulnerability(double v)
{
  vulnerability = v;
}

void Worm::SetInfectionPort(PortId_t p)
{
  infectionport = p;
}

void Worm::SetPayloadLength(Size_t l)
{
  payloadlength = l;
}

void Worm::SetTargetVector(const WormTargetVector &targetvector)
{
  defaultTV = targetvector.Copy();
}

void Worm::SetSignature(const string s)
{
  signature = s;
}

void Worm::Receive(Packet *p, L4Protocol *proto, Seq_t)
{
  delete p;
}

// Right now every packet received is a worm
bool Worm::PacketIsWorm(Packet *p)
{
  // for now let's just return true
  return true;

  /*
  char *ptr,*buffer;
  PDU *pdu;
  bool flag = false;
  Size_t size = p->SSize();

  buffer = new char[size];

  DEBUG(1,(cout<<"Entering Packet is worm"<<endl));
  DEBUG(1,(cout<<"Packet size "<<p->Size()<<endl));
  DEBUG(1,(cout<<"PDUs "<<p->PDUs.size()<<endl));

  pdu = p->PeekPDU();

  DEBUG(1,(cout<<"Packet size "<<pdu->Size()<<endl));
  DEBUG(1,(cout<<"Serializable size "<<pdu->SSize()<<endl));

  ptr = pdu->Serialize(buffer, size);
  DEBUG(1,(cout<<"Serialize size "<<size<<endl));


  DEBUG(1,(cout<<"Contents:"<<ptr<<endl));
  for (int i=0; i<pdu->Size(); i++) {
    cout<<ptr[i];
  }
  cout<<endl;

  DEBUG(1,(cout<<"Now comparing..."<<endl));

  if (strncmp(ptr, signature.c_str(), signature.length())==0) {
    DEBUG(1,(cout<<"Worm signature found..."<<endl));
    flag = true;
  }

  delete buffer;
  

  return flag;

  */
}

