/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - Worm Application class

// Monirul I Sharif.  Georgia Tech, Spring 2004

// Helper Class for worm

#include "common-defs.h"
#include "application.h"
#include "randomtree.h"
#include "worm.h"
#include "tcp.h"
#include "rng.h"
#include "node.h"
#include "application-webserver.h"
#include "application-webbrowser.h"
#include "wormhelper.h"

#ifdef HAVE_QT
#include <qnamespace.h>
#endif

using namespace std;

Count_t AddressRangeOfRandomTreeNetworks
(int trees, int depth, int fanout, int depth_fs, int fanout_fs
 )
{
  Count_t rangeHostsPerTree = (Count_t) pow((double)fanout, (double)depth-depth_fs-1) * 
    (Count_t) pow((double)fanout_fs, (double)depth_fs);
  return rangeHostsPerTree * trees;
}

void CreateRandomTreeNetworksWithWorms(int trees,
				       int depth,
				       int fanout,
				       const Linkp2p &lk,
				       int depth_fs,
				       int fanout_fs,
				       const Linkp2p &hlk,
				       double treelegprob,
				       const Linkp2p &blk,
				       IPAddr_t baseIP,
				       const Worm &worm
				       )
{
  Node *gateway = new Node();
  gateway->SetLocation(Location(5,10.3));

  // Setting colors let's see if it works
#ifdef HAVE_QT
  gateway->Color(Qt::blue);
#endif 
  Count_t rangeHostsPerTree = (Count_t) pow((double)fanout, (double)depth-depth_fs-1) * 
    (Count_t) pow((double)fanout_fs, (double)depth_fs);
  
  IPAddr_t incrIP = rangeHostsPerTree;
  
  for (int treenum=0; treenum<trees; treenum++) {
    RandomTree* t= new RandomTree(depth, fanout, lk,
				  depth_fs, fanout_fs, hlk, 
				  baseIP + incrIP * treenum, treelegprob);
    
    t->BoundingBox(Location(3,4), Location(7,10.3), treenum*(360/trees));
    t->GetRoot()->AddDuplexLink(gateway, blk);
    t->GetRoot()->DefaultRoute(gateway);
    
    cout<<"Tree has "<<t->leafCount<<" leaves"<<endl;
    
    for (int j=0; j<(int)t->leafCount; j++) {
      DEBUG0((cout<<"Setting worm app on tree"<< treenum<<" leaf "<<j));
      Node *n =t->GetLeaf(j);
      DEBUG0((cout<<" IP Address:"<<IPAddr::ToDotted(n->GetIPAddr())<<endl));
            
      Worm* app = (Worm*)n->AddApplication(worm);
      
      if (treenum==0 && j==0) 
	      app->Infect();
      app->Start(0.0);
    } 
  }

}

// Histogram* CreateRandomNetworksWithWorms(const Worm &worm, bool usethrottle, bool partial)
// {
//   Histogram* h = new Histogram(7.0, 70);  // 0-7.0 seconds, 70 bins
//   IPAddrVec_t servers;
  
//   Linkp2p l1(Rate("1Mb"), Time("10ms"));
//   Linkp2p l10(Rate("10Mb"), Time("10ms"));
//   Linkp2p l50(Rate("50Mb"), Time("10ms"));
//   Linkp2p l500(Rate("500Mb"), Time("10ms"));
//   Linkp2p l100(Rate("100Mb"), Time("10ms"));
//   Linkp2p l1g(Rate("1Gb"), Time("10ms"));

  
//   Node *gateway = new Node();
//   gateway->SetLocation(Location(15,15));

//   // Setting colors let's see if it works
// #ifdef HAVE_QT
//   gateway->Color(Qt::blue);
// #endif 

//   // creating ECE servers 
//   //Star star1(10, l10, IPAddr("192.170.11.1"));
//   Star star1(20, l10, IPAddr("10.0.0.1"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star1.GetHub()->SetLocation(Location(20, 35));
//   // Specify the bounding box
//   star1.BoundingBox(Location(15,30), Location(25,40));
   
//   // Create the servers
//   for (Count_t i = 0; i < star1.LeafCount(); ++i)
//     {
//       Node* l = star1.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       servers.push_back(l->GetIPAddr());
//       WebServer* WebApp = (WebServer*)l->AddApplication(WebServer());
//       WebApp->Start(0.0);

//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);
//     }


//   // creating COC servers 
//   //  Star star2(10, l10, IPAddr("192.171.10.1"));
//   Star star4(10, l10, IPAddr("10.0.0.135"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star4.GetHub()->SetLocation(Location(5, 22));
//   // Specify the bounding box
//   star4.BoundingBox(Location(0,17), Location(10,27));
   
//   // Create the servers
//   for (Count_t i = 0; i < (star4.LeafCount() - 5); ++i)
//     {
//       Node* l = star4.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       servers.push_back(l->GetIPAddr());
//       WebServer* WebApp = (WebServer*)l->AddApplication(WebServer());
//       WebApp->Start(0.0);

//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);
//     }
  
//   // creating external servers 
//   //Star star3(20, l1, IPAddr("192.168.10.1"));
//   Star star5(20, l1, IPAddr("10.0.0.160"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star5.GetHub()->SetLocation(Location(25, 5));
//   // Specify the bounding box
//   star5.BoundingBox(Location(20,0), Location(35,10));
   
//   // Create the servers
//   for (Count_t i = 0; i < (star5.LeafCount() - 15); ++i)
//     {
//       Node* l = star5.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       //l->UseThrottle(true);
//       servers.push_back(l->GetIPAddr());
//       WebServer* WebApp = (WebServer*)l->AddApplication(WebServer());
//       WebApp->Start(0.0);

//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);
//     }

//   Uniform* RandomStart = new Uniform(0,1);

//   // creating web browsers in COC lab
//   for (Count_t i = (star4.LeafCount() - 5); i < star4.LeafCount() ; ++i)
//      {
//        Node* l = star4.GetLeaf(i);
//        l->UseThrottle(usethrottle);
//        //l->UseThrottle(true);
//        WebBrowser* wb = (WebBrowser*)l->AddApplication(WebBrowser(servers, Uniform(0, servers.size())));
//        wb->ThinkTimeBound(60.0);
//        wb->SetStatistics(h);
//        wb->Start(RandomStart->Value());

//        Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);       
//       }

//   // creating web browsers in external lab
//   for (Count_t i = (star5.LeafCount() - 15); i < star5.LeafCount() ; ++i)
//     {
//       Node* l = star5.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       //l->UseThrottle(true);
//       WebBrowser* wb = (WebBrowser*)l->AddApplication(WebBrowser(servers, Uniform(0, servers.size())));
//       wb->ThinkTimeBound(60.0);
//       wb->SetStatistics(h);
//       wb->Start(RandomStart->Value());

//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);
//     }

//   // creating web browsers in ECE lab
//   //  Star star4(20, l10, IPAddr("192.170.12.1"));
//   Star star2(20, l10, IPAddr("10.0.0.43"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star2.GetHub()->SetLocation(Location(5, 35));
//   // Specify the bounding box
//   star2.BoundingBox(Location(0,30), Location(10,40));
   
//   // Create the browsers
//   for (Count_t i = 0; i < star2.LeafCount() ; ++i)
//      {
//        Node* l = star2.GetLeaf(i);
//        l->UseThrottle(usethrottle);

//        //l->UseThrottle(true);
       
//        WebBrowser* wb = (WebBrowser*)l->AddApplication(WebBrowser(servers, Uniform(0, servers.size())));
//        wb->ThinkTimeBound(60.0);
//        wb->SetStatistics(h);
//        wb->Start(RandomStart->Value());

//        Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);

//      }

//   // creating web browsers in office
//   //  Star star5(20, l10, IPAddr("192.169.10.1"));
//   Star star3(20, l10, IPAddr("10.0.0.90"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star3.GetHub()->SetLocation(Location(30, 30));
//   // Specify the bounding box
//   star3.BoundingBox(Location(25,25), Location(35,35));
   
//   // Create the browsers
//   for (Count_t i = 0; i < star3.LeafCount() ; ++i)
//     {
//       Node* l = star3.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       //l->UseThrottle(true);

//       WebBrowser* wb = (WebBrowser*)l->AddApplication(WebBrowser(servers, Uniform(0, servers.size())));
//       wb->ThinkTimeBound(60.0);
//       wb->SetStatistics(h);
//       wb->Start(RandomStart->Value());
      
//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);      
//     }

//   // creating web browsers in second external lab
//   //  Star star6(20, l10, IPAddr("192.168.11.1"));
//   Star star6(20, l10, IPAddr("10.0.0.205"));

//   // Assign a location to the hub to test the "relative" bounding box
//   star6.GetHub()->SetLocation(Location(5, 5));
//   // Specify the bounding box
//   star6.BoundingBox(Location(0,0), Location(10,10));
   
//   // Create the browsers
//   for (Count_t i = 0; i < star6.LeafCount() ; ++i)
//     {
//       Node* l = star6.GetLeaf(i);
//       l->UseThrottle(usethrottle);
//       // l->UseThrottle(true);

//       WebBrowser* wb = (WebBrowser*)l->AddApplication(WebBrowser(servers, Uniform(0, servers.size())));
//       //wb->ThinkTimeBound(60.0);
//       wb->ThinkTimeBound(60.0);
//       wb->SetStatistics(h);
//       wb->Start(RandomStart->Value());

//       Worm* app = (Worm*)l->AddApplication(worm);
//       app->Start(0.0);
//     }

//   // creating router nodes
//   Node* r1 = new Node();
//   Node* r2 = new Node();
//   r1->SetLocation(Location(15, 30));
//   r2->SetLocation(Location(15, 20));
 

//   // connecting everything together
//   r1->AddDuplexLink(star2.GetHub(), l100);//, IPAddr("10.0.0.83"), IPAddr("10.0.0.84"));
//   r1->AddDuplexLink(star1.GetHub(), l100);//, IPAddr("10.0.0.41"), IPAddr("10.0.0.42"));
//   r1->AddDuplexLink(r2, l500);//, IPAddr("10.0.0.85"), IPAddr("10.0.0.86"));

//   r2->AddDuplexLink(star4.GetHub(), l100);//, IPAddr("10.0.0.89"), IPAddr("10.0.0.134"));
//   r2->AddDuplexLink(star3.GetHub(), l50);//, IPAddr("10.0.0.87"), IPAddr("10.0.0.88"));
//   r2->AddDuplexLink(gateway, l500);//, IPAddr("10.0.0.132"), IPAddr("10.0.0.133"));

//   gateway->AddDuplexLink(star5.GetHub(), l1g);//,IPAddr("10.0.0.156"), IPAddr("10.0.0.157"));
//   gateway->AddDuplexLink(star6.GetHub(), l1g);//, IPAddr("10.0.0.202"), IPAddr("10.0.0.203"));


//   r1->UseThrottle(partial);
//   r2->UseThrottle(partial);
//   gateway->UseThrottle(partial);


//   // // add worm
//   Worm* app = (Worm*)star6.GetLeaf(0)->AddApplication(worm);
  
//   app->Infect();
//   app->Start(0.0);
  

//   return h;
// }
