/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - UDP Worm class

// Monirul I Sharif,  Georgia Tech, Fall 2003

// Define a Worm Application class

// Uncomment below to enable debug level 0

//#define DEBUG_MASK 0x1

#include "debug.h"
#include "wormudp.h"
#include "udp.h"
#include "node.h"
#include "simulator.h"

#ifdef HAVE_QT
#include <qnamespace.h>
#endif

using namespace std;


Count_t  WormUDP::scanrate = 50;

// In this version of the worm application I am using a constant
//    packets per second method to sent UDP packets.
// I will later change this into a more robust method.

// Constructors


WormUDP::WormUDP()
    : udp(nil), timeoutevent(nil)
{

}

WormUDP::~WormUDP()
{
}

// Application Methods
void WormUDP::StartApp()    // Called at time specified by Start
{
  Worm::StartApp();
}

void WormUDP::StopApp()     // Called at time specified by Stop
{
  Worm::StopApp();
  if (timeoutevent) timer.Cancel(timeoutevent);
}

void WormUDP::AttachNode(Node* n) 
{
  Worm::AttachNode(n);

  udp = new UDP(n);
  
  udp->Attach(n);
  udp->Bind(infectionport);
  udp->AttachApplication(this);
  DEBUG0((cout << "udp " << udp << " , attach App " << this << endl));
}

Application* WormUDP::Copy() const
{
  return new WormUDP(*this);
}


void WormUDP::Activate()
{
  DEBUG0(( cout<<"Infected machine:"<< IPAddr::ToDotted(node->GetIPAddr())<<endl ));
  DEBUG0(( cout<<" Now starting timer event"<<endl));
  // Set the node to red on animation
#ifdef HAVE_QT
  node->Color(Qt::red);
#endif
  SendWorm();
  ScheduleNextPacket();
}


void WormUDP::Initialize()
{

}

void WormUDP::SendWorm()
{
  IPAddr_t target = GenerateNextIPAddress();  

  DEBUG0((cout<<"SendWorm["<<(string)IPAddr(node->GetIPAddr())<<
	  "]: Sending worm packet to "<<(string)IPAddr(target)<<endl));

  udp->SendTo(payloadlength, target, infectionport);
}

void WormUDP::SetScanRate(Count_t rate)
{
  scanrate = rate;
}

void WormUDP::Receive(Packet *p, L4Protocol *proto, Seq_t)
{
   DEBUG0((cout<<"Receive["<<(string)IPAddr(node->GetIPAddr())<<"]: Received TCP packet of size "<<p->Size()<<endl));
   
   if (PacketIsWorm(p)) {
     if (vulnerable && !infected) {
       Infect();
      }
   }
   delete p;
}

void WormUDP::ScheduleNextPacket()
{
  if (!timeoutevent) timeoutevent = new TimerEvent();
  timer.Schedule(timeoutevent, 1.0 / (double)scanrate, this);
}

void WormUDP::Timeout(TimerEvent *ev)
{
  if (!node)
    {
      cout << "WormUDP::Timeout with no attached node "<<endl;
      return;
    }

  DEBUG0((cout<< "WormUDPWorm::Timeout "<<Simulator::Now()<<endl));

  // Sending the worm packet
  SendWorm();
  ScheduleNextPacket();
}
