/*

* GTNetS provides a portable C++ class library for network simulations
*
* Copyright (C) 2003 George F. Riley
*
* No guarantees or warranties or anything are provided or implied in any way
* whatsoever.  Use this program at your own risk.  Permission to use this
* program for any purpose is given, as long as the copyright is kept intact.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

* Permission is hereby granted for any non-commercial use of this code

*/

// Georgia Tech Network Simulator - WormUDP Application class

// Monirul I Sharif.  Georgia Tech, Spring 2004

// Define a WormUDP Application class

#ifndef __wormudp__h__
#define __wormudp__h__

#include "common-defs.h"
#include "application.h"
#include "timer.h"
#include "udp.h"
#include "rng.h"
#include "worm.h"

#include <map>

// This version only supports TCP protocol

//Doc:ClassXRef
class WormUDP : public Worm, public TimerHandler {
  //Doc:Class Define a application that works as the Sapphire worm
  //Doc:Class 
public:
  // Define a number of constructors for different needs
  //Doc:Method
  WormUDP();   // Just specify endpoint
    //Doc:Desc Specifies a node to attach the worm application to. 

  virtual void AttachNode(Node*); // Note which node attached to
  virtual ~WormUDP();
  virtual Application* Copy() const;// Make a copy of the application
  
  static void SetScanRate(Count_t );

  virtual void Initialize();
public:
  // newly defined in this class
  virtual void Activate();

private:
  UDP*		udp;
  Timer         timer;
  TimerEvent*   timeoutevent;

  static Count_t scanrate;

  void SendWorm();

  void ScheduleNextPacket();  

  // the following method is called when packets are received from the port
  virtual void Receive(Packet *, L4Protocol *, Seq_t); // Data received

  void Timeout(TimerEvent *);  // Timer Handler

  virtual void StartApp();    // Called at time specified by Start
  virtual void StopApp();     // Called at time specified by Stop
};

#endif

