eval 'exec perl -S $0 ${1+"$@"}'  # -*-Mode:Perl; perl-indent-level:8-*-
    if 0;

use POSIX;
use Math::Stat; # Average and stddev on vectors (see CPAN.org)

# Generic option-grabbing code
sub Getopt
{
	local($spec) = @_;      # name of variables to be initialized from %opt
	&Usage  if ($#ARGV < 0);
	while ($_ = $ARGV[0], /^-(\w+)/) {
		shift @ARGV;
		$key = $1;
		$opt{$key} = (index($spec, "$key:") >= 0) ? 1 : &Usage ;
	}
}

sub Usage
{
        $0 =~ s/.*\/([^\/]+)$/$1/o;
        print <<EOF;
Usage:  $0 [-maxmin] [-std] <DATA_FILES>

This script averages the numerical values of any identically formatted
files.  It was designed to generate averages and standard deviations
of a number of simulation output files created using different random seeds.

The program uses spaces as number delimiters, and output is undefined
if the input files are not identically formatted.  

This script takes in a list of data files in the same format.
The script computes the mean, standard deviation, max, and min of
the numerical values. 

The following arguments control the output behavior:
-maxmin:  TBD (unimplemented)
-std: compute standard deviation in std.out file 
-both: add standard deviation to mean file (in parentheses)

Example usage w/o arguments:  "$0 *.dat" or "$0 foo1.dat foo2.dat .. foon.dat"
EOF
exit 1;
}

&Getopt("maxmin:std:both:");

$results_dir = "./";
if ($opt{'std'}) {
		$results_file = "std.out";
} else {
		$results_file = "mean.out";
}
$outputfile = $results_dir . "/" . $results_file;


# Organize input data into two-dimensional array.
# First index is the number's offset into the file
# Second index is the file (of the set of files in @files) that we are on

@files = @ARGV;
$filecnt = 0;

foreach $file(@files) {
	$offset = 0;
	open(FILE, "< ".$file);
	while(<FILE>)
	{
		@col = split;
		foreach $element(@col) {
			next if ($element !~ /^(-?\d+\.?\d*|-?\.\d+)$/); # not a number
			$data[$offset][$filecnt] = $element;
			$offset++;
		}
	}
	close(FILE);
	$filecnt++;
}

# Get means and stddev
$j = 0;
while ($j < $offset) {
	$aref = $data[$j];
	my $stat = Math::Stat->new(\@$aref, {Autoclean => 1});
	$avg[$j] = $stat->average();
	$stddev[$j] = $stat->stddev();
	$j++;
}

# Generate output file

# We preserve up the formatting in the original file, as much as possible.  
# Use the first file as a template, and use a substitute-as-you-go strategy.
# If we overrun the space allocated to a field, the perfect alignment may
# be destroyed, but at least make sure it is preceded by one whitespace

$firstfile = shift @files;
open(FILE, "<".$firstfile);

open(OUT, "> ".$outputfile);

# Work on it one line at a time
while(<FILE>) {
  chomp; # newline
  @nums = $_ =~ /(\s+-?[\d\.]+\b)/g;  
  @fields = $_ =~ /(\s*[\S]+)/g;
  while ($field = shift(@fields)) {
#    printf("field=%s, num[0]=%s\n", $field, $nums[0]);
    if ($field eq $nums[0]) {
      # Number to make a substitution for
      $num = shift(@nums);
      $precision = 0;
      $ws = $num =~ tr/" "//;
      $numsize = $num =~ tr/0-9\.//;
      if ($num =~ /-?(\d+)\.(\d+)/) {
        $precision = $2 =~ tr/0-9//;
      }
      # We are replacing a number with $ws leading 
      # whitespace of width $numsize and with $precision 
      # digits after decimal point
      $avg_ = shift(@avg);
      $stddev_ = shift(@stddev);
      $replacement = $opt{'std'} ? $stddev_ : $avg_;
      # format $replacement to have same precision as what 
      # it is replacing
      if ($precision) {
        $replacement = sprintf "%.${precision}f", $replacement;
      } else {
        $replacement = sprintf "%d", $replacement;
      }
      $rep_size = $replacement =~ tr/0-9\.//;
      # compute adjusted amt of whitespace to insert
      $ws_to_insert = ($ws - ($rep_size - $numsize));
      $ws_to_insert = 1 if ($ws_to_insert < 1);
      for ($i = 0; $i < $ws_to_insert; $i++) {
        print OUT " ";
      }
      print OUT $replacement;
      if ($opt{'both'}) {
        # Print stddev in parentheses after mean
        if ($precision) {
          $stddev_fmt = sprintf "%.${precision}f", $stddev_;
        } else {
          $stddev_fmt = sprintf "%d", $stddev_;
        }
        print OUT "(${stddev_fmt})";
      }
    } else {
      print OUT $field;
    }
  }
  print OUT "\n";
}

close(OUT);
close(FILE);
