// tw.cc
// NVR Ad Hoc version test, GTNetS
// Young J. Lee, Georgia Tech, Feb. 2003
// $Id: tw.cc,v 1.1.1.1 2006/12/13 13:09:35 dnguyen Exp $

#include "simulator.h"
#include "node.h"
#include "wlan.h"
#include "ratetimeparse.h"
//#include "application-tcpserver.h"
//#include "application-tcpsend.h"
//#include "tcp-tahoe.h"
#include "application-cbr.h"
#include "udp.h"
#include "routing-nvr.h"
//#include "mobility-specific-waypoint.h"
//#include "wireless-grid-rectangular.h"
//#include "mobility-random-waypoint.h"
#include "mystat.h"

/*
#define XWIDTH			1500
#define YWIDTH			300
#define NO_NODE			50
*/

#define XSIZE			10
#define YSIZE			5
#define NO_NODE			(XSIZE * YSIZE)
#define NODE_SPACING	160
//#define NODE_SPACING	110

#define NO_SRC			10

//#define RANDOM_SRC

struct MyPair
{
  int s;    // src node index
  int d;    // dst node index
};

#ifdef RANDOM_SRC
MyPair sd[NO_SRC];
#else
MyPair sd[] = {
  //{0,49}

  {0, 47},{1, 13},{4, 0},{11, 9},{40, 11},
  {39, 30},{7, 34},{21, 7},{6, 12},{4, 9}

  //{45, 42},{14, 23},{12, 43},{31, 45},{4, 6},
  //{14, 45},{19, 16},{8, 10},{2, 14},{17, 3}

  //{34, 36},{16, 31},{11, 3},{34, 41},{17, 29},
  //{36, 10},{0, 32},{21, 2},{16, 48},{37, 30}

  //{45, 46},{31, 12},{28, 20},{46, 44},{17, 4},
  //{38, 30},{22, 7},{15, 31},{20, 0},{25, 30}

  /*
  {24, 2},{13, 47},{2, 0},{20, 12},{22, 10},
  {23, 29},{31, 5},{8, 37},{1, 13},{19, 22},
  {46, 22},{44, 22},{47, 43},{33, 41},{26, 16},
  {23, 45},{25, 34},{29, 3},{32, 3},{22, 1},
  {4, 26},{31, 33},{9, 17},{37, 38},{2, 23},
  {38, 47},{22, 9},{26, 16},{26, 12},{30, 47}
  */
};
#endif

using namespace std;

int main(char* argc, char** argv)
{
  Simulator s;
  Routing::SetRouting(new RoutingNVR);   // Set NVR to default routing
  s.HasMobility(true);  // Hack! fix later
  //Trace* tr = Trace::Instance();
  //tr->IPDotted(true);
  //tr->Open("test.txt");
  //TCP::LogFlagsText(true);
  //IPV4::Instance()->SetTrace(Trace::ENABLED);

  WirelessLink wlink(NO_NODE, IPAddr("192.168.0.1"), MASK_ALL);

  //Uniform urvx(0, XWIDTH), urvy(0, YWIDTH);
  //WirelessGridRectangular g(Location(0,0), Constant(NO_NODE), urvx, urvy);
  Node* n[NO_NODE];
  for (int i = 0; i < NO_NODE; i++) {
	n[i] = wlink.GetNode(i);
	n[i]->SetRadioRange(250);
	//n[i]->SetRadioRange(100);
	//n[i]->SetLocation(urvx.Value(), urvy.Value());
	//n[i]->AddMobility(RandomWaypoint(g, Constant(10), Uniform(1,20)));
  }

/*
  // Delayed NVREQ test
  n[0]->SetLocation(60, 0);
  n[1]->SetLocation(0, 60);
  n[2]->SetLocation(120, 60);
  n[3]->SetLocation(60, 120);
*/

  for (int j = 0; j < YSIZE; j++)
	for (int i=0; i<XSIZE; i++)
	  n[i+XSIZE*j]->SetLocation(NODE_SPACING*i, NODE_SPACING*j);

/*
  // Scenario 1, source is moving
  n[0]->SetLocation(0, 0);
  n[1]->SetLocation(0, 60);
  n[2]->SetLocation(120, 60);
  n[3]->SetLocation(60, 120);

  SpecificWaypoint* m1 =
	(SpecificWaypoint*)n[0]->AddMobility(SpecificWaypoint());
  m1->AddWaypoint(n[0], Waypoint(12, Location(120,0)));
*/
/*
  // Scenario 2, destination is moving
  n[0]->SetLocation(60, 0);
  n[1]->SetLocation(0, 60);
  n[2]->SetLocation(120, 60);
  n[3]->SetLocation(0, 120);

  SpecificWaypoint* m1 =
	(SpecificWaypoint*)n[3]->AddMobility(SpecificWaypoint());
  m1->AddWaypoint(n[3], Waypoint(12, Location(120,120)));
*/
  
  //double d = atof(argv[1]);
  double d = 4;
  Rate_t rate = (Rate_t)Rate("4.096Kb") * d;
  CBRApplication* cbr[NO_SRC];

#ifdef RANDOM_SRC
  Uniform urv1(0, NO_NODE-1), urv2(0, NO_NODE-1);

  for (int i = 0; i < NO_SRC; i++) {
	sd[i].s = urv1.IntValue();
	sd[i].d = urv2.IntValue();
	while( sd[i].d == sd[i].s )
	  sd[i].d = urv2.IntValue();
  }
#endif

/*
  TCPServer* server = new TCPServer(TCPTahoe());
  server->BindAndListen(n[2], 80);
  //server->SetTrace(Trace::ENABLED);

  TCPSend* client =
  new TCPSend(TCPTahoe(n[1]), n[2]->GetIPAddr(), 80, Uniform(1000, 10000));
  //client->SetTrace(Trace::ENABLED);
  client->Start(0);
*/

  for (int i = 0; i < NO_SRC; i++)
	cbr[i] = new CBRApplication(n[sd[i].s], n[sd[i].d], UDP(), rate, 512);

  for (int i = 0; i < NO_SRC; i++) {
	//cbr[i]->Start(0);        // extreme case
	cbr[i]->Start(1.0*i);    // normal injection
	cbr[i]->Stop(900.0);
  }

  s.Progress(1.0);
  s.StopAt(1000);
  s.StartAnimation(0, true);
  s.AnimationUpdateInterval(Time("1ms")); // 1ms initial update rate
  s.Run();

#ifdef RANDOM_SRC
  for (int i = 0; i < NO_SRC; i++) {
	cout << "{" << sd[i].s << ", " << sd[i].d << "},";
	if( (i+1)%5 == 0 ) cout << endl;
  }
  cout << endl;
#endif

  MyStat::Print();
  cout << "Simulation complete" << endl;
}
